#Stanford Deep Network Library Make Include File
#Ian Goodfellow <ia3n@cs.stanford.edu>
#
#Based on the STAIR Vision Library Make Include File by Stephen Gould
#This should only need to change in extreme circumstances. You almost definitely do not
#need to edit this file.
#If you are having trouble linking against an external library you need to edit your
#local makefile ( local/make.${HOST} ).
#If you added a new file to the library, you need to edit a makefile src/lib
#If you added a new file to the apps, you need to edit a makefile in src/apps

OS = $(shell uname)
BIN_PATH = $(SDN_PATH)/bin
LIB_PATH = $(SDN_PATH)/lib
EXT_PATH = $(SDN_PATH)/external
INC_PATH = $(SDN_PATH)/include
SRC_PATH = $(SDN_PATH)/src
CONFIG_PATH=${SDN_PATH}/config
EIGEN_PATH=${EXT_PATH}/eigen2

HOST_NAME=$(shell hostname)
LOCAL_MAKEFILE=${SDN_PATH}/local/make.${HOST_NAME}

VERSION=0
SUBVERSION=1.0

# include local makefile to define necessary paths
ifeq ($(wildcard $(LOCAL_MAKEFILE)), $(LOCAL_MAKEFILE))
	-include $(LOCAL_MAKEFILE)
endif

CCC = g++ 
NVCC := $(CUDA_PATH)/bin/nvcc

# force 32-bit compile (set to 1 in local makefile if desired)
FORCE32BIT = 0
ifeq (,$(findstring x86_64,$(shell uname -m)))
  FORCE32BIT = 1
endif

# application build settings (set to 0 or 1 in make.local)
#BUILD_GUI_APPS = 1 we may use this variable later if we start using wxWidgets for visualization

# use shared libraries (set to 1 in make.local)
SDN_SHARED_LIBS = 0

# compile cuda .cu files (set to 1 in make.local)
BUILD_CUDA_KERNELS = 0

# platform flags
PLAT_CFLAGS = -D__LINUX__
PLAT_LFLAGS = 

JPEG_LFLAGS = -ljpeg

# link to OpenCV should be in external/opencv
OPENCV_HEADERS_PATH ?= /usr/include
OPENCV_CFLAGS = -I$(OPENCV_HEADERS_PATH)
OPENCV_LFLAGS = -L$(OPENCV_LIB_PATH) -lcv -lhighgui -lcxcore -lml
# assume OpenCV 1.0
OPENCV_DEVEL = 0

# EIGEN_PATH should be set in local makefile
EIGEN_CFLAGS = -I$(EIGEN_PATH) -msse2 
#-DEIGEN_NO_DEBUG 
EIGEN_LFLAGS = -msse2

#Boost flags
#BOOST_INCLUDE_PATH ?= /usr/include
BOOST_LIB_PATH ?= /usr/lib
BOOST_CFLAGS = -I$(BOOST_INCLUDE_PATH)
BOOST_LFLAGS = -L$(BOOST_LIB_PATH) ${BOOST_LIBRARIES} 

#gtest flags
GTEST_INCLUDE_PATH ?= /usr/include
GTEST_CFLAGS = -I${GTEST_INCLUDE_PATH}
GTEST_LFLAGS = -L${GTEST_LIB_PATH} -lgtest -lgtest_main 

# if we decide to use wxWidgets later, WXDIR should be defined in local makefile
#WX_CFLAGS = `$(WXDIR)/buildgtk/wx-config --cflags`
#WX_LFLAGS = `$(WXDIR)/buildgtk/wx-config --libs core,base,gl`


# initialize extra cflags
EXTRA_CFLAGS = -I$(INC_PATH) -I$(EXT_PATH) -DOPENCV_DEVEL=${OPENCV_DEVEL} -DBUILD_CUDA_KERNELS=${BUILD_CUDA_KERNELS} -DCONFIG_PATH=${CONFIG_PATH} -DSDN_PATH=${SDN_PATH} -DSDN_VERSION=${VERSION}.${SUBVERSION}

# cuda flags
OSLOWER = $(shell uname -s 2>/dev/null | tr [:upper:] [:lower:])
CUDA_CFLAGS = -DUSE_CUDA -I$(CUDA_PATH)/include -I$(CUDA_SDK_PATH)/common/inc -I/usr/local/cula/include
CUDA_LFLAGS = -L$(CUDA_SDK_PATH)/lib -lcutil -lcuda -L$(CUDA_SDK_PATH)/common/lib/$(OSLOWER) \
        -L$(CUDA_PATH)/lib64 -L$(CUDA_PATH)/lib -lcudart -lcublas \

ifndef COMPILE_WITH_EIGEN
CUDA_CFLAGS += -DSDN_USE_CUDA=1
endif


# include local makefile a second time to override any settings
ifeq ($(wildcard $(LOCAL_MAKEFILE)), $(LOCAL_MAKEFILE))
  -include $(LOCAL_MAKEFILE)
endif

# extra compiler and linker flags
ifeq ($(NDEBUG), 1)
  EXTRA_CFLAGS += -DNDEBUG 
endif

ifeq ($(USE_OPENCV), 1)
  EXTRA_CFLAGS += $(OPENCV_CFLAGS)
  EXTRA_LFLAGS += $(OPENCV_LFLAGS) $(JPEG_LFLAGS)
endif

ifeq ($(USE_EIGEN), 1)
  EXTRA_CFLAGS += $(EIGEN_CFLAGS)
  EXTRA_LFLAGS += $(EIGEN_LFLAGS)
endif

ifeq ($(USE_GTEST), 1)
  EXTRA_CFLAGS += $(GTEST_CFLAGS)
  EXTRA_LFLAGS += $(GTEST_LFLAGS)
endif

#Will need to enable this if we decide to use wxWidgets
#ifeq ($(USE_WX), 1)
#  EXTRA_CFLAGS += $(WX_CFLAGS)
#  EXTRA_LFLAGS += $(WX_LFLAGS)
#endif

ifeq ($(BUILD_CUDA_KERNELS), 1)
  EXTRA_CFLAGS += $(CUDA_CFLAGS) 
  EXTRA_LFLAGS += $(CUDA_LFLAGS)  
endif

# SDN libraries
STATICSDN = $(LIB_PATH)/libsdnBase.a $(LIB_PATH)/libsdnDataset.a $(LIB_PATH)/libsdnRuntime.a \
        $(LIB_PATH)/libsdnGui.a $(LIB_PATH)/libsdnProjects.a $(LIB_PATH)/libsdnStable.a $(LIB_PATH)/libsdnOld.a $(LIB_PATH)/libsdnRegistry.a 
ifeq ($(BUILD_CUDA_KERNELS), 1)
  STATICSDN += $(LIB_PATH)/libsdnCuda.a
endif

SHAREDSDN = $(LIB_PATH)/libsdnBase.so.${VERSION}.${SUBVERSION} $(LIB_PATH)/libsdnDataset.so.${VERSION}.${SUBVERSION} $(LIB_PATH)/libsdnRuntime.so.${VERSION}.${SUBVERSION} \
        $(LIB_PATH)/libsdnGui.so.${VERSION}.${SUBVERSION} $(LIB_PATH)/libsdnProjects.so.${VERSION}.${SUBVERSION} $(LIB_PATH)/libsdnRegistry.so.${VERSION}.${SUBVERSION} \
				$(LIB_PATH)/libsdnStable.so.${VERSION}.${SUBVERSION} $(LIB_PATH)/libsdnOld.so.${VERSION}.${SUBVERSION}
ifeq ($(BUILD_CUDA_KERNELS), 1)
  SHAREDSDN += $(LIB_PATH)/libsdnCuda.so.${VERSION}.${SUBVERSION}
endif

EXTLIB = $(EXT_PATH)/xmlParser/xmlParser.a
#$(EXT_PATH)/lbfgs/lbfgs.a $(EXT_PATH)/libsvm/libsvm.a

ifeq ($(SDN_SHARED_LIBS), 0)
  LIBSDN = $(STATICSDN)
else
  LIBSDN = $(SHAREDSDN)
  EXTRA_LFLAGS += -Wl,-rpath,$(LIB_PATH) -L$(LIB_PATH) \
        -lsdnBase -lsdnDataset -lsdnRuntime -lsdnStable -lsdnOld
ifeq ($(BUILD_CUDA_KERNELS), 1)
  EXTRA_LFLATS += -lsdnCuda
endif
endif

DEBUG_FLAGS = 


EXTRA_CFLAGS += ${BOOST_CFLAGS}
EXTRA_LFLAGS += ${BOOST_LFLAGS}

DEBUG=$(warning debug)

# compiler and linker flags
ifeq ($(strip $(FORCE32BIT)), 0)
  CFLAGS = ${DEBUG_FLAGS} -O3 -fPIC $(PLAT_CFLAGS) $(EXTRA_CFLAGS) ${LOCAL_CFLAGS} -Wall
  LFLAGS = ${DEBUG_FLAGS} -rdynamic -lm -lpthread $(PLAT_LFLAGS) $(EXTRA_LFLAGS) $(LIBSDN) $(EXTLIB)
        NVCCFLAGS = -Xcompiler -fPIC $(OPENCV_CFLAGS) $(CUDA_CFLAGS) -I$(INC_PATH)
else
  CFLAGS =  -O3 -m32 $(PLAT_CFLAGS) $(EXTRA_CFLAGS) -Wall
  LFLAGS =  -lm -m32 -lpthread $(PLAT_LFLAGS) $(EXTRA_LFLAGS) $(LIBSDN) $(EXTLIB)
        NVCCFLAGS = -Xcompiler -fPIC -m32 $(OPENCV_CFLAGS) $(CUDA_CFLAGS) -I$(INC_PATH)
endif

OBJ = $(SRC:.cc=.o)
