//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>

/**
 \file config-train.cc
 \brief Executable for training networks
*/

#include <cstdio>
#include <iostream>
#include "tclap/CmdLine.h"
#include "sdn/base.h"
#include "sdn/stable.h"

std::string configFile;
bool verbose;
bool debug;

void parseCommandLine(int argc, char *argv[]);

int main(int argc, char *argv[])
{

  //Get options from command line
  parseCommandLine(argc, argv);

  //Load additional config data
  if (configFile != "") {
    sdn::ConfigManager::get().configure(configFile.c_str(), false);
  }
  
  //Turn on debugging or verbose information if requested
  if (debug)
    sdn::Logger::setLogLevel(sdn::LOGLEVEL_DEBUG);
  else if (verbose)
    sdn::Logger::setLogLevel(sdn::LOGLEVEL_VERBOSE);

  //Set up the runtime manager
  sdn::ScriptRuntimeManager mgr;
  mgr.run();

  return 0;
}

void parseCommandLine(int argc, char *argv[]) {
  try {
    TCLAP::CmdLine parser("Stanford Deep Network Library training program", ' ', SDN_MAKE_STRING(SDN_VERSION) );

    TCLAP::ValueArg<std::string> configArg("c", "config", "Configuration file", false, "config.xml", "string", parser);
    TCLAP::SwitchArg verboseSwitch("v","verbose","print more information",parser,false);
    TCLAP::SwitchArg debugSwitch("d","debug","print debugging information (also implies -v)",parser,false);
    parser.parse( argc, argv );

    configFile = configArg.getValue();
    verbose = verboseSwitch.getValue();
    debug = debugSwitch.getValue();


  } catch (TCLAP::ArgException &e) { 
    SDN_ERROR("error: " << e.error() << " for arg " << e.argId()); 
    SDN_FATAL("parseCommandLine failed.");
  }


}



