/// \author Ian Goodfellow
/// \brief Scrambles a network's weights, for control experiments

#include <iostream>
#include "sdn/old.h"
#include "sdn/base.h"
#include <string>

using namespace sdn;
using namespace boost;
using namespace std;

void usage()
{
  cout << "scramble [options] -i <input file> -o <output file>" << endl;
  cout << "options: " << endl;
  cout << "\t--deepest-only\t\t\tScrambles only the deepest layer of the weights" << endl;
}

int main(int argc, char ** argv)
{

  char * infile = NULL;
  char * outfile = NULL;
  bool deepestOnly = false;

  for (int i = 1; i < argc; i++)
    {
      if (string(argv[i]) == "-i")
	{
	  if (infile || i+1 >= argc)
	    {
	      cout << "input specified twice or -i flag at end of input" << endl;
	      usage();
	      exit(-1);
	    }
	  i++;
	  infile = argv[i];
	}
      else if (string(argv[i]) == "-o")
	{
	  if (outfile || i+1 >= argc)
	    {
	      cout << "output specified twice or -o flag at end of input" << endl;
	      usage();
	      exit(-1);
	    }
	  i++;
	  outfile = argv[i];
	}
      else if (string(argv[i]) == "--deepest-only")
	{
	  deepestOnly = true;
	}
      else
	{
	  usage();
	  exit(-1);
	}
    }


  if (!infile || !outfile)
    {
      usage();
      exit(-1);
    }

  shared_ptr<Network> n;

  {
    ifstream ifs(infile, ios::binary);
    serial::BinInArchive ar(ifs);
    n = NetworkRegistry().load(ar);
  }

  SDN_ASSERT(n);

  n->scrambleWeights(deepestOnly);

  n->save(outfile);
}
