/// \author Ian Goodfellow
/// \file sdntest.cc
/// \brief Loads a network file and runs tests on it
#include "sdn/old.h"
#include "sdn/base.h"
#include <string>
using namespace std;
using namespace sdn;
using namespace boost;

void usage()
{
  SDN_MSG("Usage: ./sdntest [options] <filepath> " << endl
	  << "Loads network in <filepath> and runs tests" << endl
	  << "options: "<<endl
	  << "-c <config.xml>\t\tUse config.xml to load tests"<<endl
	  << "-d\t\tPrint SDN_DEBUG messages"<<endl
	  << "-v\t\tPrint SDN_VERBOSE messages");
}


class SDNTestParser : public ConfigParser
  {
  public:
    SDNTestParser() 
    {
    }
    virtual ~SDNTestParser() {}

    ///Tells its parent ConfigListener which topics to listen to
    virtual void topics(std::vector<std::string> & topics)
    {
      topics.push_back("sdn::Test");
    }
    
    virtual void usage(std::ostream & os) const
    {
      os << "Just reads sdn::Test objects-- deserializes and runs them" << endl;
    }
    
    ///Parse and take action on a message recieved via the listener
    virtual void readConfig(XMLNode & node)
    {
	  tests.push_back(TestRegistry().load(node));

    }

    vector<shared_ptr<Test> > tests;
  };


int main(int argc, char ** argv)
{
  bool debug = false;
  bool verbose = false;
  string filepath;


  for (int i = 1; i < argc; i++)
    {
      string arg = string(argv[i]);
      if (arg == "-v")
	{
	  verbose = true;
	}
      else if (arg == "-d")
	{
	  debug = true;
	}
      else if (arg == "-c")
	{
	  i++;
	  SDN_ASSERT(i < argc);
	  ConfigManager::get().configure(argv[i]);
	}
      else
	{
	  if (filepath != "")
	    {
	      usage();
	      exit(-1);
	    }
	  filepath = arg;
	}
    }

  if (filepath == "")
    {
      usage();
      exit(-1);
    }

  if (verbose)
    Logger::setLogLevel(LOGLEVEL_VERBOSE);
  if (debug)
    Logger::setLogLevel(LOGLEVEL_DEBUG);

  shared_ptr<Network> p = Network::load(filepath.c_str());

  
  shared_ptr<SDNTestParser> parser( new SDNTestParser());
  ConfigListener listener(parser);


  for (unsigned t = 0; t < parser->tests.size(); t++)
    {
      shared_ptr<Test> te = parser->tests[t];

      te->run(p, shared_ptr<Dataset>());

    }

  return 0;
}
