#include "binInArchive.h"
#include "binInClassBuilder.h"
#include "pointer.h"
#include <cstring>
#include <string>
using namespace std;
using namespace sdn;
using namespace serial;

#define CHECK_PTR  SDN_ASSERT_MSG(!expectingReportPointer_, "calls to peek that report that an object has not been serialized yet expect to be followed immediately by calls to reportPointer (did a factory fail to allocate the object?)");


BinInArchive::BinInArchive(std::istream & in)
{
  SDN_ASSERT(!in.fail());

#ifndef __LITTLE_ENDIAN
  SDN_ERROR("All our binary serialization code assumes little endian CPU, nothing has been done to rearrange the bytes. You will probably be able to serialize and deserialize on big endian machines but people with little endian machines won't be able to read your archive");
#endif

  in_ = & in;

  char * buf = new char[strlen(ARCHIVE_STRING)];

  in_->read(buf, strlen(ARCHIVE_STRING));

  SDN_ASSERT(!in_->fail());

  SDN_ASSERT( ! strncmp(buf, ARCHIVE_STRING, strlen(ARCHIVE_STRING)));

  delete [] buf;
  
  int version;
  readInt_(version);

  SDN_ASSERT(version == 0);
}

void BinInArchive::readInt_(int & rval)
{
  SDN_ASSERT(sizeof(rval) == 4);
  
  in_->read((char *) & rval, sizeof(rval));
}

void BinInArchive::endDerived_()
{
  CHECK_PTR;

  if (classBuilder_->endDerived())
    {
      classBuilder_ = ptr( (BinInClassBuilder *) NULL);
    }
}
      
void BinInArchive::endBase_()
{
  CHECK_PTR;
 
  if (classBuilder_->endBase())
    {
      classBuilder_ = ptr( (BinInClassBuilder *) NULL);
    }
}
           
void BinInArchive::beginSuper_()
{
  CHECK_PTR;

}

void BinInArchive::endSuper_()
{
  CHECK_PTR;
  classBuilder_->endSuper();
}

int BinInArchive::beginDerived_(void * addr, int version, std::string id)
{
  CHECK_PTR;


  if (classBuilder_)
    {
      version = classBuilder_->beginDerived(id);
    }
  else
    {
      classBuilder_ = BinInClassBuilder::makeDerived(this, id);
      version = classBuilder_->version;
    }

  return version;
}

int BinInArchive::beginBase_(void * addr, int version, std::string id)
{
  CHECK_PTR;

  if (classBuilder_)
    {
      version = classBuilder_->beginBase(id);
    }
  else
    {
      classBuilder_ = BinInClassBuilder::makeBase(this, id);
      version = classBuilder_->version;
    }

  return version;
}
      


Vaddr BinInArchive::peek_(void * & prevPtr,
		std::string & id, int & version)
{
  bool serialized;
  Vaddr v;

  readBool_(serialized);
  readVaddr_(v);

  if (serialized)
    {
      SDN_DEBUG(vaddrToBaseName_(v) << " object was serialized already, with virtual address "<<v);
      prevPtr = vaddrToAddrOfPtr_(v);
    }
  else
    {
      prevPtr = NULL;
      readInt_(version);

      //Read the base type name 
      int n;
      readInt_(n);
      char * buf = new char[n+1];
      buf[n] = '\0';
      in_->read(buf,n);

      //See if there's a derived type name/how long it is
      readInt_(n);
      if (n)
	{
	  delete [] buf;
	  buf = new char[n+1];
	  buf[n] = '\0';
	  in_->read(buf,n);
	}

      //Either use the new derived type name, or if there wasn't one, reuse the base type name as the derived type name
      id = std::string(buf);
      delete [] buf;
    }
  return v;
}

void BinInArchive::readBool_(bool & b)
{
  unsigned char temp;
  SDN_ASSERT(sizeof(temp) == 1);
  in_->read((char *) & temp, sizeof(temp));
  assert(temp <= 1);
  b = temp;
  //SDN_DEBUG("Read bool with value "<<(int) temp);
}

void BinInArchive::readVaddr_(Vaddr & v)
{
  SDN_ASSERT(sizeof(v) == 8);
  in_->read((char *) & v, sizeof(v));
}

void BinInArchive::attribute_(std::string name, unsigned & value)
{
  classBuilder_->attribute(value);
}

void BinInArchive::attribute_(std::string name, double & value)
{
  readDouble_(value);
}

void BinInArchive::readDouble_(double & value)
{
  SDN_ASSERT(sizeof(value) == 8);
  in_->read((char *) & value, sizeof(value));
}



void BinInArchive::attribute_(std::string name, std::string & value)
{
  read(value);
}

void BinInArchive::read(std::string & value)
{
  // read an integer, then that number of characters
  unsigned strLen;
  in_->read((char *) & strLen, sizeof(strLen));
  value.resize(strLen);
  for(unsigned i=0;i<strLen;i++)
    in_->read((char *) & (value[i]), sizeof(value[i]));
}

void BinInArchive::attribute_(std::string name, float & value)
{
  read(value);
}

void BinInArchive::read(float & value)
{
  SDN_ASSERT(sizeof(value) == 4);
  in_->read((char *) & value, sizeof(value));
}



void BinInArchive::attribute_(std::string name, bool & value)
{
  readBool_(value);
}

void BinInArchive::scalarArray_(Scalar * data, unsigned numFloats)
{
  int check;
  readInt_(check);
  if (check != numFloats)
  {
	SDN_FATAL("Archive contains array of "<<check<<" Scalars, but client requested "<<numFloats);
  }
  if (SDN_DOUBLE_PRECISION)
  {
  	SDN_ASSERT(sizeof(Scalar) == 8);
  }
  else
  {
	SDN_ASSERT(sizeof(Scalar) == 4);
  }
  SDN_DEBUG("Before reading, data begins with "<<data[0]);
  in_->read( (char *) data, numFloats * sizeof(Scalar));
  SDN_ASSERT(!in_->fail());
  SDN_DEBUG("Read Scalar array beginning with "<<data[0]);
  for (int i = 0; i < 4; i++)
	SDN_DEBUG( (int) ( ( (char *) data) [i] ) );
}
