/// \author Ian Goodfellow
/// An Archive for deserializing objects stored in binary form

#pragma once

#include <boost/smart_ptr.hpp>
#include "inArchive.h"



namespace sdn
{
  namespace serial
  {

    class BinInClassBuilder;

    class BinInArchive : public InArchive
    {
    public:
      BinInArchive(std::istream & in);

      //should only be called by the registry. after the registry allocates
      //an object from the archive, it should put a shared pointer on the
      //heap (yes, the shared pointer itself should be on the heap) and
      //give a void * to that shared pointer back to the archive so the
      //archive can keep on handing out shared pointers to that object
      void reportPointer(void * p);


    protected:

      std::istream * in_;

    protected:
      //Specialized reading functions rather than template reading functions
      //so that we can check the size of everything and make sure it all
      //behaves uniformly 
      void readInt_(int & rval);

      int beginBase_(void * addr, int version, std::string id);
      
      void endBase_();
      
      
      void beginSuper_();
      void endSuper_();

      int beginDerived_(void * addr, int version, std::string id);
      
      void endDerived_();

      Vaddr peek_(void * & prevPtr,
	  std::string & id, int & version);
      
      void readBool_(bool & b);
      void readVaddr_(Vaddr & v);      
      void readDouble_(double & v);

      void read(float & v);
      void read(std::string & v);

      //attributes and custom content can just be read right away, no need to wroute them through the class builder. while reading, it's currently impossible to ever be reading through a node that has already been serialized
      void attribute_(std::string name, unsigned & value);
      void attribute_(std::string name, double & value);
      void attribute_(std::string name, bool & value);
      void attribute_(std::string name, std::string & value) ;
      void attribute_(std::string name, float & value) ;

      void scalarArray_(Scalar * data, unsigned numFloats);

      friend class BinInClassBuilder;

      boost::shared_ptr<BinInClassBuilder> classBuilder_;
    };

  }
}
