#include "binInArchive.h"
#include "binInClassBuilder.h"
#include "pointer.h"

using namespace std;
using namespace sdn;
using namespace serial;


boost::shared_ptr<BinInClassBuilder> BinInClassBuilder::makeDerived(BinInArchive * archive, std::string id,  BinInClassBuilder * subclass)
{
  boost::shared_ptr<BinInClassBuilder> rval  = ptr(new BinInClassBuilder());

  rval->expectingSuper_ = true;
  rval->archive_ = archive;
  rval->subclass_ = subclass;
  rval->isDerived_ = true;

  if (archive->hasPeeked_)
    {
      //SDN_DEBUG("Serialized is catching up to the stuff that was peeked at");
      archive->hasPeeked_ = false;
      rval->version = archive->peekedVersion_;
      SDN_ASSERT(archive->peekedDerivedName_ == id);
    }
  else
    {
      bool serialized;
      archive->readBool_(serialized);
      //SDN_DEBUG("Derived class: serialized? " << serialized);
      if (serialized)
	{
	SDN_FATAL("Attempt to deserialize an object that has already been serialized. This may have resulted from making a pointer to a member of another object, and serializing the pointer before the containing object");
	}

      if (!subclass)
	{
	  Vaddr v;
	  archive->readVaddr_(v);
	  //SDN_DEBUG("Derived class: address: "<< v);
	  /// \todo-- right now there's no way of representing the mapping from this vaddr back to the object, since you can only make shared pointers. this means we can't yet point at objects that are members of other objects (which is dangerous anyway)
	}
      archive->readInt_(rval->version);
      int n;
      archive->readInt_(n);
      char * buf = new char[n+1];
      buf[n] = '\0';
      archive->in_->read(buf,n);
      delete [] buf;
      archive->readInt_(n);
      SDN_ASSERT(n);//this has to be a derived type
      buf = new char[n+1];
      buf[n] = '\0';
      archive->in_->read(buf,n);
      if (id != string(buf))
	{
	  SDN_FATAL("Serialization error. Archive contains \""<<buf<<"\", client asked for \""<<id<<"\"");
	}
      delete [] buf;
    }

  return rval;
}


boost::shared_ptr<BinInClassBuilder> BinInClassBuilder::makeBase(BinInArchive * archive, std::string id, BinInClassBuilder * subclass)
{
  boost::shared_ptr<BinInClassBuilder> rval = ptr(new BinInClassBuilder());
  
  rval->expectingSuper_ = false;
  rval->archive_ = archive;
  rval->subclass_ = subclass;
  rval->isDerived_ = false;

  if (archive->hasPeeked_)
    {
      SDN_ASSERT(!subclass);
      archive->hasPeeked_ = false;
      rval->version = archive->peekedVersion_;
    }
  else
    {
      if (!subclass)
	{
	  bool serialized;
	  archive->readBool_(serialized);
	  
	  SDN_DEBUG("base class. serialized? "<<serialized);
	  
	  if (serialized)
	    {
	      SDN_FATAL("Attempt to deserialize an object that has already been serialized. This may have resulted from making a pointer to a member of another object, and serializing the pointer before the containing object");
	    }

	  Vaddr v;
	  archive->readVaddr_(v);
	  /// \todo-- right now there's no way of representing the mapping from this vaddr back to the object, since you can only make shared pointers. this means we can't yet point at objects that are members of other objects (which is dangerous anyway)
	}
      archive->readInt_(rval->version);
      SDN_DEBUG("Version of base type: "<<rval->version);

      int n;

      if (!subclass)
	{
	  archive->readInt_(n);
	  char * buf = new char[n+1];
	  buf[n] = '\0';

	  archive->in_->read(buf,n);

	  if (id != string(buf))
	    {
	      SDN_FATAL("Serialization error. Archive contains "<<buf<<" but client asked for "<<id);
	    }

	  delete [] buf;

	  SDN_DEBUG("Base class with no hat and name "<<id);
	}


      //Read derived type      
      archive->readInt_(n);
      //SDN_DEBUG("Number of characters in derived name (should be zero since this is a base class): "<<n);
      SDN_ASSERT(n == 0); //This has to be a base type
    }
  return rval;
}

int BinInClassBuilder::beginDerived(std::string id)
{
  if (curChild_)
    {
      return curChild_->beginDerived(id);
    }
  
  BinInClassBuilder * subclass = NULL;

  if (expectingSuper_)
    {
      subclass = this;
      expectingSuper_ = false;
      SDN_ASSERT(!parsingSuper_);
      parsingSuper_ = true;
    }

  curChild_ = BinInClassBuilder::makeDerived(archive_,id,subclass);
  


  return curChild_->version;
}

int BinInClassBuilder::beginBase(std::string id)
{
  if (curChild_)
    {
      return curChild_->beginBase(id);
    }

  BinInClassBuilder * subclass = NULL;

  if (expectingSuper_)
    {
      subclass = this;
      expectingSuper_ = false;
      SDN_ASSERT(!parsingSuper_);
      parsingSuper_ = true;
    }
  

  curChild_ = BinInClassBuilder::makeBase(archive_,id,subclass);

  

  return curChild_->version;
}


BinInClassBuilder::BinInClassBuilder()
{
  expectingSuper_ = false;
  parsingSuper_ = false;
  subclass_ = NULL;
  parsedSuper_ = false;
  gotSuper_ = false;
}

bool BinInClassBuilder::endBase()
{
  SDN_ASSERT(!archive_->in_->fail());

  //SDN_DEBUG("Done parsing base class");

  if (curChild_)
    {
      if (curChild_->endBase())
	{
	  if (parsingSuper_)
	    {
	      parsingSuper_ = false;
	      SDN_ASSERT(!parsedSuper_);
	      parsedSuper_ = true;
	    }
	  else
	    {
	      curChild_ = ptr( (BinInClassBuilder *) NULL);
	    }
	}
      return false;
    }
  
  SDN_ASSERT(!isDerived_);
  return true;
}

bool BinInClassBuilder::endDerived()
{
  SDN_ASSERT(!archive_->in_->fail());

  //SDN_DEBUG("Done parsing derived class");

  if (curChild_)
    {
      if (curChild_->endDerived())
	{
	  if (parsingSuper_)
	    {
	      parsingSuper_ = false;
	      SDN_ASSERT(!parsedSuper_);
	      parsedSuper_ = true;
	    }
	  else
	    {
	      curChild_ = ptr( (BinInClassBuilder *) NULL);
	    }
	}
      return false;
    }

  SDN_ASSERT(isDerived_);
  SDN_ASSERT(gotSuper_);
  return true;
}

void BinInClassBuilder::endSuper()
{
  if (parsedSuper_)
    {
      SDN_ASSERT(curChild_ != NULL);
      SDN_ASSERT(isDerived_);
      parsedSuper_ = false;
      gotSuper_ = true;
      curChild_ = ptr( (BinInClassBuilder *) NULL);
    }
  else
    {
      curChild_->endSuper();
    }
}

void BinInClassBuilder::attribute(unsigned & value)
{
  if (curChild_)
    curChild_->attribute(value);
  else
    {
      int temp;
      archive_->readInt_(temp);
      value = (unsigned) temp;
    }
}


