/// \author Ian Goodfellow
/// \file binInClassBuilder.h
/// \brief A support class for binInArchive

//Ignore this file unless you are changing the way that serialization works
//This should not be a common operation
//Under most circumstances you should only need to write new serialization
//methods for whatever new class you've written that you want to serialize

#pragma once



namespace sdn
{
  namespace serial
  {

    class BinInArchive;

    //Builds a single class from a BinInArchive
    class BinInClassBuilder
    {            
    public:
      int version;
    protected:
      
      friend class BinInArchive;

      static boost::shared_ptr<BinInClassBuilder> makeDerived(BinInArchive * archive,  std::string id,  BinInClassBuilder * subclass = NULL);
      static boost::shared_ptr<BinInClassBuilder> makeBase(BinInArchive * archive,  std::string id, BinInClassBuilder * subclass = NULL);

      int beginDerived( std::string id);
      int beginBase(std::string id);
      
      //Returns true if this endBase finished this class builder itself (instead of its superclass or a class member)
      bool endBase();
      bool endDerived();



      void attribute(unsigned & value);


      void endSuper();

      //void writeHeader();

    protected:      
      //bool alreadySerialized_;
      //If true, is a derived class and should see a more derived class declared soon
      bool expectingSuper_;
      //If true, has already started parsing a super class, and should be expecting an endSuper
      bool parsingSuper_;
      //If true, has parsed the whole super class and is just waiting for endSuper
      bool parsedSuper_;
      //If true, has finished the super class portion of the object description
      bool gotSuper_;

      //void readVaddr_(Vaddr & v);
      //void readBool_ (bool & b);
      //void readInt_(int & i);


      BinInArchive * archive_;
      boost::shared_ptr<BinInClassBuilder> curChild_;
      BinInClassBuilder * subclass_;
      bool isDerived_;
      //true iff endBase/endDerived has been called at this level and getNode is ready to call
      //bool done_;
      BinInClassBuilder();
      //int version_;
      //std::string id_;
    };
  }
}
