/// \author Ian Goodfellow

#include "binOutArchive.h"
#include "binOutClassBuilder.h"
#include "pointer.h"
#include <cstring>
using namespace std;
using namespace sdn;
using namespace serial;


BinOutArchive::BinOutArchive(std::ostream & out)
{ 
  SDN_ASSERT(!out.fail());

  #ifndef __LITTLE_ENDIAN
  SDN_ERROR("All our binary serialization code assumes little endian CPU, nothing has been done to rearrange the bytes. You will probably be able to serialize and deserialize on big endian machines but people with little endian machines won't be able to read your archive");
#endif

  out_ = & out;

  out_->write(ARCHIVE_STRING, strlen(ARCHIVE_STRING));
  //Write out version
  int version = 0;
  SDN_ASSERT(sizeof(version) == 4);
  out_->write((char *) & version, sizeof(version));
}

void BinOutArchive::beginSuper_()
{

}

void BinOutArchive::endSuper_()
{
  classBuilder_->endSuper();
}

int BinOutArchive::beginDerived_(void * addr, int version, std::string id)
{
  if (classBuilder_ != NULL)
    classBuilder_->beginDerived(addr,version,id);
  else
    classBuilder_ = BinOutClassBuilder::makeDerived(this,addr,version,id);
  return version;
}

int BinOutArchive::beginBase_(void * addr, int version, std::string id)
{

  if (classBuilder_ != NULL)
    classBuilder_->beginBase(addr,version,id);
  else
    classBuilder_ = BinOutClassBuilder::makeBase(this,addr,version,id);
  return version;
}

void BinOutArchive::endBase_()
{
  if (classBuilder_->endBase())
    classBuilder_ = ptr((BinOutClassBuilder *) NULL);
}

void BinOutArchive::endDerived_()
{
  if (classBuilder_->endDerived())
    classBuilder_ = ptr((BinOutClassBuilder *) NULL);
}

BinOutArchive::~BinOutArchive()
{
  p_.processTerminal(FLUSH);
}

void BinOutArchive::attribute_(std::string name, unsigned & value)
{
  classBuilder_->attribute(value);
}

void BinOutArchive::attribute_(std::string name, double & value)
{
  classBuilder_->attribute(value);
}

void BinOutArchive::attribute_(std::string name, std::string & value)
{
  classBuilder_->attribute(value);
}

void BinOutArchive::attribute_(std::string name, float & value)
{
  classBuilder_->attribute(value);
}

void BinOutArchive::write_(float & value)
{
  SDN_ASSERT(sizeof(value) == 4);
  out_->write((char *) & value, sizeof(value));
}

void BinOutArchive::write_(std::string & value)
{
  unsigned strLen = value.size();
  out_->write( (char *) & strLen, sizeof(strLen));

  for(unsigned i=0;i<value.size();i++)  
    out_->write((char *) & (value[i]), sizeof(value[i]));
}

void BinOutArchive::write_(double & value)
{
  SDN_ASSERT(sizeof(value) == 8);
  out_->write((char *) & value, sizeof(value));
}

void BinOutArchive::write_(bool & value)
{
  unsigned char temp;
  if (value)
    temp = 1;
  else
    temp = 0;
  SDN_ASSERT(sizeof(temp) == 1);
  out_->write((char *) & temp, sizeof(temp));
}

void BinOutArchive::attribute_(std::string name, bool & value)
{
  classBuilder_->attribute(value);
}

void BinOutArchive::scalarArray_(Scalar * data, unsigned numFloats)
{
  classBuilder_->ScalarArray(data,numFloats);
}
