/// \author Ian Goodfellow
/// \file binOutArchive.h
/// \brief Archive for outputing binary to a stream

#pragma once

#include "outArchive.h"
#include <boost/smart_ptr.hpp>

namespace sdn
{

  namespace serial
 {

   class BinOutClassBuilder;

    /// Archive for outputting classes as binary (format described in binOutClassBuilder.cc)
    class BinOutArchive : public OutArchive
    {
      //Forward declaration, see xmlOutClassBuilder.h
      friend class BinOutClassBuilder;
      
    public:
      
      BinOutArchive(std::ostream & out);
                        
      
                  
      virtual ~BinOutArchive();
    protected:
      

      std::ostream * out_;
      
      boost::shared_ptr<BinOutClassBuilder> classBuilder_;


    protected:
      int beginBase_(void * addr, int version, std::string id);
      
      void endBase_();
      
      
      void beginSuper_();
      void endSuper_();

      int beginDerived_(void * addr, int version, std::string id);
      
      void endDerived_();

      void write_(double & value);
      void write_(bool & value);

      void write_(float & value);
      void write_(std::string & value);

      //attributes and custom content need to be routed through the classBuilder so they can be ignored if the class we're currently building has already been serialized
      void attribute_(std::string name, unsigned & value);
      void attribute_(std::string name, double & value);
      void attribute_(std::string name, bool & value);

      void attribute_(std::string name, std::string & value) ;
      void attribute_(std::string name, float & value) ;

      void scalarArray_(Scalar * data, unsigned numFloats);
    };
  }
}
