/// \author Ian Goodfellow
/// \file binOutClassBuilder.h
/// \brief A support class for XmlOutArchive

//Ignore this file unless you are changing the way that serialization works
//This should not be a common operation
//Under most circumstances you should only need to write new serialization
//methods for whatever new class you've written that you want to serialize

#pragma once

namespace sdn
{
  namespace serial
  {
    class BinOutArchive;


    //Builds the XML node for a single class
    class BinOutClassBuilder
    {            
    protected:
      
      friend class BinOutArchive;

      static boost::shared_ptr<BinOutClassBuilder> makeDerived(BinOutArchive * archive, void * addr, int version, std::string id, bool knownToBeSerialized = false, BinOutClassBuilder * subclass = NULL);
      static boost::shared_ptr<BinOutClassBuilder> makeBase(BinOutArchive * archive, void * addr, int version, std::string id, bool knownToBeSerialized = false, BinOutClassBuilder * subclass = NULL);

      void beginDerived(void * addr, int version, std::string id);
      void beginBase(void * addr, int version, std::string id);
      
      //Returns true if this endBase finished this class builder itself (instead of its superclass or a class member)
      bool endBase();
      bool endDerived();

      void endSuper();

      void setBaseType(std::string type);
      void writeHeader();

      void attribute(unsigned & value);
      void attribute(bool & value);
      void attribute(double & value);

      void attribute(float & value);
      void attribute(std::string & value);

      void ScalarArray(Scalar * data, unsigned numFloats);


    protected:      
      bool alreadySerialized_;
      //If true, is a derived class and should see a more derived class declared soon (beginSuper is not passed through for OutArchives)
      bool expectingSuper_;
      //If true, has already started parsing a super class, and should be expecting an endSuper
      bool parsingSuper_;
      //If true, has parsed the whole super class and is just waiting for endSuper
      bool parsedSuper_;
      //If true, has finished the super class portion of the object description
      bool gotSuper_;

      void writeVaddr_(Vaddr v);
      void writeBool_ (bool b);
      void writeInt_(int i);


      BinOutArchive * archive_;
      boost::shared_ptr<BinOutClassBuilder> curChild_;
      BinOutClassBuilder * subclass_;
      bool isDerived_;
      //true iff endBase/endDerived has been called at this level and getNode is ready to call
      bool done_;
      BinOutClassBuilder();
      int version_;
      std::string id_;
    };
  }
}
