//Based on svlConfigManager.cpp by Stephen Gould, in the STAIR Vision Library

#include <string>
using namespace std;
#include <map>
#include "xmlUtils.h"
#include "configManager.h"
#include "logger.h" 
#include "fileUtils.h"
#include <assert.h>

using namespace sdn;
using namespace boost;

// Global variable

bool ConfigManager::alive = true;


// configuration interface -----------------------------------------------------

ConfigParser::ConfigParser()
{

}

ConfigParser::~ConfigParser()
{

}

ConfigListener::ConfigListener(shared_ptr<ConfigParser> p) :  
_parser(p)
{
 /** \todo
  Any SDN_DEBUG statements inside this or ConfigListener constructors
  won't run since the constructors run before the command line parsing
  will respond to the -d flag. Figure out some way of dealing with that.
  */ 
  p->topics(_topics);
  ConfigManager::get()._subscribe(this);
}

ConfigListener::~ConfigListener()
{
    // unregister the module
  if (ConfigManager::alive)
  	ConfigManager::get()._unsubscribe(this);
}

void ConfigListener::readConfig(const char *filename)
{
    SDN_ASSERT(filename != NULL);

    XMLNode root = openXML(filename);
    SDN_ASSERT(!root.isEmpty());


    assert(false); 

    //todo-- go through the file and find all children of the root
    //whose name matches anything in _topics, then call readConfig
    //on them

    /*
    XMLNode node = root.getChildNode(_topics.c_str());
    if (node.isEmpty()) {
      SDN_ERROR("couldn't find configuration for " << _topics
            << " in " << filename);
    } else {
        readConfig(node);
    }
    */
}

void ConfigListener::readConfig(XMLNode & node)
{
	_parser->readConfig(node);
}


// library configuration registry ---------------------------------------------

ConfigManager::ConfigManager()
{
 
  //Automatically read the optional main.xml config file on launch
  string configfile = configPath() +  "/main.xml";
  
  configure(configfile.c_str(),true);//noErrors=true so that having a config file is optional
  
}

ConfigManager::~ConfigManager()
{
	alive = false;
    // do nothing
}

ConfigManager& ConfigManager::get()
{
    static ConfigManager mgr;
    return mgr;
}

void ConfigManager::configure(const char *filename, bool noErrors)
{

    SDN_ASSERT(filename != NULL);
    XMLNode root = openXML(filename);
    if (root.isEmpty()) {
      if (noErrors)
	return;

      SDN_ERROR("couldn't find configuration in " << filename);
      return;
    }

    configure(root);
}

void ConfigManager::configure(XMLNode& root)
{
  if (!root.getName())
    {
      SDN_FATAL("sdn::ConfigManager passed XML file that begins with a nameless tag (should open with an sdnConfig tag) ");
    }

  string name = root.getName();
  
  if (name != "sdnConfig")
    {
      SDN_FATAL("ConfigManager passed XML file that does not begin with an sdnConfig tag");
      return;
    }


    for (int i = 0; i < root.nChildNode(); i++)
      {
        XMLNode node = root.getChildNode(i);
	SDN_ASSERT(node.getName());
	string topicName = string(node.getName());


        ConfigRegistry::iterator it = _registry.find(topicName);
        if (it == _registry.end()) {
	  //no one is subscribed to this topic yet, but this isn't a bug
	  //since main.xml loads before any of the subscriptions can finish
	  
	  it = _createTopic(topicName);
	  
        }
	
	
	vector<ConfigListener *>::iterator j = it->second->subscribers.begin(), last = it->second->subscribers.end();
	
	for (; j != last; ++j)
	  (*j)->readConfig(node);

	it->second->backIssues.push_back(node);
        
      }
}

ConfigManager::ConfigRegistry::iterator ConfigManager::_createTopic(const std::string & name)
{
  ConfigManager::ConfigRegistry::iterator it;

  ConfigTopic * newTopic(new ConfigTopic);
  assert(newTopic != NULL);
  
  _registry.insert(make_pair(name, newTopic));
  
  it = _registry.find(name);
  
  assert(it != _registry.end());

  return it;
}

void ConfigManager::showRegistry() const
{
  assert(false); //todo-- make standardized toString method and use that
}

void ConfigManager::_subscribe(ConfigListener * (m))
{
    SDN_ASSERT(m != NULL);



    const vector<string> & topics = m->topics();

    for (unsigned i = 0; i < topics.size(); i++)
      {
	ConfigRegistry::iterator it = _registry.find(topics[i]);

	if (it != _registry.end())
	  {
	    //Topic already exists

	    //Make sure this is a new subscriber
	    const vector<ConfigListener * > & subscribers = it->second->subscribers;
	    for (unsigned j = 0; j < subscribers.size(); j++)
	      {
		if (subscribers[j] == m)
		  {
		    SDN_WARN("A listener has subscribed to "<<topics[i]<<" twice-- blocking second attempt.");
		    return;
		  }
	      }

	
	    //send all backissues to the new subscriber
	    vector<XMLNode> & backIssues = it->second->backIssues;

	    for (unsigned int j = 0; j < backIssues.size(); j++)
	      m->readConfig(backIssues[j]);
	  }
	else
	  {
	    //Topic doesn't exist yet, create it
	    it = _createTopic(topics[i]);
	  }
	
	//register the new subscriber
	it->second->subscribers.push_back(m);
      }
}

void ConfigManager::_unsubscribe(ConfigListener * (m))
{
  SDN_DEBUG( "unregistering a listener");

  const vector<string> & topics = m->topics();

  for (unsigned i = 0; i < topics.size(); i++)
    {
      ConfigRegistry::iterator it = _registry.find(topics[i]);
      if (it == _registry.end()) {
	SDN_WARN("listener attempted to unsubscribe from non-existent topic with name \"" << topics[i]);
    } else {
	bool found = false;
	for (unsigned j = 0; j < it->second->subscribers.size(); j++)
	  {
	    if (it->second->subscribers[j] == m)
	      {
		it->second->subscribers.erase(it->second->subscribers.begin()+j);
		found = true;
		break;
	      }
	  }
	if (!found)
	{
	  SDN_WARN("listener attempted to unscribe from "<< topics[i] << " despite not being subscribed to it (possible repeat unsubscription)");
	}
      }
    }
}




// ------------- IncludeParser implementation ------------------------------
IncludeParser::IncludeParser()
{

}

IncludeParser::~IncludeParser()
{

}

void IncludeParser::topics(vector<string> & topics)
{
	topics.push_back("include");
}

void IncludeParser::usage(std::ostream & os) const
{
  os << "<include file=\"example.xml\"> : Causes config manager to also load files in \"example.xml\"" << endl;
}

void IncludeParser::readConfig(XMLNode & node)
{
  const char * filename = node.getAttribute("file");

  if (filename)
    {
      ConfigManager::get().configure(filename);
    }
}

static ConfigListener gIncludeListener(boost::shared_ptr<ConfigParser>(new IncludeParser));

