//Author: Ian Goodfellow <ia3n@cs.stanford.edu>
//Some functions are ported from the STAIR Vision Library
#include "logger.h"
#include "fileUtils.h"
#include "strUtils.h"
#include <algorithm>
#include <iostream>
#include <dirent.h>
#include <string.h>
using namespace std;
using namespace sdn;

#if defined(_WIN32)||defined(WIN32)||defined(__WIN32__)
#include "win32/dirent.h"
#include <windows.h>
#else
#include <dirent.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>



std::string sdn::sdnPath()
{
#ifndef SDN_PATH
  Error-- SDN_PATH should be set by the makefile
#endif

    return SDN_MAKE_STRING(SDN_PATH);
}

std::string sdn::configPath()
{
#ifndef CONFIG_PATH
  Error-- CONFIG_PATH should be set by the makefile
#endif
    
    return SDN_MAKE_STRING(CONFIG_PATH);
}

std::vector<std::string> sdn::listSubdirs(const std::string & dir, bool fullPath)
{
  string prefix = "";
  if (fullPath)
    prefix = dir + "/";

  vector<string> rval;
  DIR * dirp;
  struct dirent * entry;

  dirp = opendir(dir.c_str());

  if (dirp)
    {
      entry = readdir(dirp);
      while (entry)
	{
	  if (entry->d_type == DT_LNK)
	    cerr << "listSubdirs doesn't know how to deal with symlinks yet." << endl;
	  
	  if (entry->d_type == DT_DIR) // checks for directories. DT_REG is files. 
	    {
	      if (strcmp(entry->d_name,".")!= 0 && strcmp(entry->d_name,"..")!=0)
		rval.push_back(prefix+string(entry->d_name));
	    }
           entry = readdir(dirp);
	}
    }
  else
    /// \todo-- make this throw an exception
    SDN_WARN("listSubdirs: Could not open directory \""<<dir<<"\"");

  sort(rval.begin(), rval.end());

  return rval;
}

std::vector<std::string> sdn::listFiles(const std::string & dir, std::string  suffix, bool fullPath)
{
  string prefix = "";
  if (fullPath)
    prefix = dir + "/";

  vector<string> rval;
  DIR * dirp;
  struct dirent * entry;

  dirp = opendir(dir.c_str());

  if (dirp)
    {
      entry = readdir(dirp);
      while (entry)
	{
	  if (entry->d_type == DT_LNK)
	    cerr << "listFiles doesn't know how to deal with symlinks yet." << endl;
	  
	  //cout << entry->d_name << " " <<(int) entry->d_type<<endl;

	  if (entry->d_type == DT_REG || entry->d_type==0) // DT_REG doesn't seem to work here, no idea what 0 is but I often get that type 
	    {
	      string name = entry->d_name;
	      //cout << name.substr(name.size()-suffix.size()) << endl;
	      if (name.size() >= suffix.size() && name.substr(name.size()-suffix.size()) == suffix)
		rval.push_back(prefix+string(entry->d_name));
	    }
            entry = readdir(dirp);
	}
    }
  else
    cerr << "listFiles: Could not open directory \""<<dir<<"\"" << endl;

  sort(rval.begin(), rval.end());

  return rval;
}






////// ported from STAIR Vision Library



string sdn::strBaseName(const string &fullPath)
{
    string baseName;
    
    // strip directory name
    string::size_type pos = fullPath.find_last_of("/\\");
    if (pos == string::npos) {
	baseName = fullPath;
    } else {
	baseName = fullPath.substr(pos + 1, fullPath.length() - pos);
    }

    // strip extension
    return strWithoutExt(baseName);
}


string sdn::strExtension(const string &fullPath)
{
    string filename = strFilename(fullPath);
    string::size_type pos = filename.find_last_of(".");
    if (pos != string::npos) {
	return filename.substr(pos + 1, filename.length() - pos);
    }
    
    return string("");
}

string sdn::strReplaceExt(const string &fullPath, const string &ext)
{
  string oldExt = strExtension(fullPath);
  int len = oldExt.length() == 0 ? 0 : oldExt.length() + 1;
  return (fullPath.substr(0, fullPath.length() - len) + ext);
}

string sdn::strFilename(const string &fullPath)
{
    // strip directory name
    string::size_type pos = fullPath.find_last_of("/\\");
    if (pos == string::npos) {
	return fullPath;
    }
    
    return fullPath.substr(pos + 1, fullPath.length() - pos);
}

string sdn::strWithoutExt(const string &fullPath)
{
  string filename = fullPath;

  // strip extension
  string::size_type pos = filename.find_last_of(".");
  if (pos != string::npos) {
    filename = filename.substr(0, pos);
  }
  return filename;
}

bool sdn::mkdir(const char * directoryPath)
{
#if defined(_WIN32)||defined(WIN32)||defined(__WIN32__)
  bool success = CreateDirectory(directoryPath, NULL);
#else
  bool success = ::mkdir(directoryPath, 0777);
#endif
    
  if (!success) {
    SDN_WARN( "failed to create directory " << directoryPath);
  }
  
  return success;
}


std::string sdn::makeValidFilename(const std::string & filename)
{
  string rval = filename;
  for (unsigned i = 0; i < rval.size(); i++)
    {
      if (rval[i] == '/')
	rval[i] = '-';
    }
  return rval;
}
