//Author: Ian Goodfellow <ia3n@cs.stanford.edu>
//File: fileUtils.h
//Description: basic utility functions for
// *loading files relative to the installed library
// *ls-like functionality
// *manipulating filepath strings
// Some functions are ported from the STAIR Vision Library

#pragma once

#include <vector>
//#include <string>

namespace sdn
{
  //Important libsdn paths
///returns the path to root directory of the library
  std::string sdnPath(); 
///returns the path to config directory
  std::string configPath(); 
    

  std::string makeValidFilename(const std::string & filename);

    
  ///Returns sorted list of subdirectories
  std::vector<std::string> listSubdirs(const std::string & dir, bool fullPath=true);
  ///Returns sorted list of files
  std::vector<std::string> listFiles(const std::string & dir, std::string suffix="", bool fullPath = true);
  
  
// create directory
  bool mkdir(const char * path);




  //Filepath string operations ported from Stair Vision Library
/// Returns filepath with directory portion stripped off
  std::string strFilename(const std::string &fullPath); 
/// Returns filepath with extension and directory portion stripped off
  std::string strBaseName(const std::string &fullPath); 
/// Returns extension of a filepath
  std::string strExtension(const std::string &fullPath);
/// Returns filepath with the extension stripped off
  std::string strWithoutExt(const std::string &fullPath);
/// Returns filepath with a different extension
  std::string strReplaceExt(const std::string &fullPath, const std::string &ext);
  
}
