//Author: Ian Goodfellow
//Based on svlLogger.cpp by Stepehen Gould and Ian Goodfellow, from the STAIR Vision Library

#include <ctime>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <cassert>
#include <execinfo.h>
#include <stdio.h>
#include <stdlib.h>
#include "logger.h"

using namespace std;

// logging callbacks
void (*sdn::Logger::showErrorCallback)(const char *message) = NULL;
void (*sdn::Logger::showWarningCallback)(const char *message) = NULL;
void (*sdn::Logger::showMessageCallback)(const char *message) = NULL;


     
/* From the GNU libc manual: Obtain a backtrace and print it to stdout. */
void
print_trace (void)
{
  void *array[10];
  size_t size;
  char **strings;
  size_t i;
  
  size = backtrace (array, 10);
  strings = backtrace_symbols (array, size);
  
  printf ("Obtained %zd stack frames.\n", size);
  
  for (i = 0; i < size; i++)
    printf ("%s\n", strings[i]);
  
  free (strings);
}


ofstream sdn::Logger::_output;
sdn::LogLevel sdn::Logger::_logLevel = sdn::LOGLEVEL_MSG;

sdn::Logger::Logger() 
{
    // do nothing
}

sdn::Logger::~Logger()
{
    // do nothing
}

void sdn::Logger::initialize(const char *filename, bool bOverwrite)
{
    if (_output.is_open()) {
        _output.close();
    }

    if ((filename != NULL) && (strlen(filename) > 0)) {
        _output.open(filename, bOverwrite ? ios_base::out : ios_base::out | ios_base::app);
        assert(!_output.fail());

        time_t t = time(NULL);
        struct tm *lt = localtime(&t);

        _output << "-" << setw(2) << setfill('0') << lt->tm_hour
             << ":" << setw(2) << setfill('0') << lt->tm_min 
             << ":" << setw(2) << setfill('0') << lt->tm_sec
             << " --- log opened --- \n";
    }    
}

void sdn::Logger::initialize(const char *filename,
    bool bOverwrite, sdn::LogLevel level)
{
    _logLevel = level;
    initialize(filename, bOverwrite);
}

void sdn::Logger::log(sdn::LogLevel level, const string& msg)
{
    if (level > _logLevel) return;

    char prefix[4] = "---";
    switch (level) {
      case sdn::LOGLEVEL_FATAL:   prefix[1] = '*'; break;
      case sdn::LOGLEVEL_ERROR:   prefix[1] = 'E'; break;
      case sdn::LOGLEVEL_WARNING: prefix[1] = 'W'; break;
      case sdn::LOGLEVEL_MSG: prefix[1] = '-'; break;
      case sdn::LOGLEVEL_VERBOSE: prefix[1] = '-'; break;
      case sdn::LOGLEVEL_DEBUG:   prefix[1] = 'D'; break;
    }

    if (_output.is_open()) {
        time_t t = time(NULL);
        struct tm *lt = localtime(&t);

        _output << "-" << setw(2) << setfill('0') << lt->tm_hour
             << ":" << setw(2) << setfill('0') << lt->tm_min 
             << ":" << setw(2) << setfill('0') << lt->tm_sec
             << prefix << ' ' << msg << "\n";
    }

    if (level < sdn::LOGLEVEL_WARNING) {
        if (showErrorCallback == NULL) {
            cerr << prefix << ' ' << msg << endl;
        } else {
            showErrorCallback(msg.c_str());
        }
    } else if (level == sdn::LOGLEVEL_WARNING) {
        if (showWarningCallback == NULL) {
            cerr << prefix << ' ' << msg << endl;
        } else {
            showWarningCallback(msg.c_str());
        }
    } else {
        if (showMessageCallback == NULL) {
            cout << prefix << ' ' << msg << endl;
        } else {
            showMessageCallback(msg.c_str());
        }
    }

    if (level == sdn::LOGLEVEL_FATAL) {
      
      print_trace();
      abort();
    }
}
