/// \author Ian Goodfellow
/// \file matrix.h
/// \brief Definitions that control whether the library is single or double precision

#pragma once
#include <Eigen/Core>
#include "../cuda/cudaMatrix.h"
#include "../cuda/cudaVector.h"

/// WARNING: Changing the precision of the library will make it incompatible with all saved archive

namespace sdn
{

#ifndef SDN_USE_CUDA
#define SDN_USE_CUDA 1
#endif

#if SDN_USE_CUDA
#define SDN_DOUBLE_PRECISION 0 // NOTE: this should always be 0
#else

#ifndef SDN_DOUBLE_PRECISION
#define SDN_DOUBLE_PRECISION 1
#endif

#endif

#if SDN_USE_CUDA
        typedef float Scalar;
	typedef CudaVector Vector;
	typedef CudaMatrix Matrix;
#elif SDN_DOUBLE_PRECISION
	typedef double Scalar;
	typedef Eigen::VectorXd Vector;
	typedef Eigen::MatrixXd Matrix;
#else
	typedef float Scalar;
	typedef Eigen::VectorXf Vector;
	typedef Eigen::MatrixXf Matrix;
#endif

}

