/// \author Ian Goodfellow
/// \file parser.h
/** \brief An LL(1) parser that validates the commands given to the
    serialization archives. Unless you are modifying the archive classes
    you don't need to pay any attention to this file.
*/

#pragma once
#include <stack>

namespace sdn
{
  namespace serial
  {

    //Symbols in the serialization language
    enum Symbol
      {
	//Terminals
	FLUSH,
	BEGIN_BASE,
	END_BASE,
	BEGIN_DERIVED,
	END_DERIVED,
	BEGIN_SUPER,
	END_SUPER,
	ATTRIBUTE,
	BEGIN_PTR,
	END_PTR,
	CUSTOM_CONTENT,
	//Non-terminals
	S,
	CLASS_INSTANCES,
	FLUSHES,
	CLASS_INSTANCE,
	BASE_CLASS_INSTANCE,
	DERIVED_CLASS_INSTANCE,
	SUPER_CLASS_INSTANCE,
	INSTANCE_BODY,
	ATTRIBUTE_NT,
	CHILD,
	PTR,
	CONTENT,
	ELEMENTS,
	ELEMENT
      };
    
    // A parser that the Archive class uses to ensure that the sequence of calls to the archive is syntactically correct-- this reduces the need to maintain a lot of parsing state inside the archive
    class Parser
    {
    public:
      Parser();
      void processTerminal(Symbol t);
    protected:
      void debug(Symbol t);
      std::stack<Symbol> stack_;
    };
  }
}
