// Author: Jiquan Ngiam (jngiam@cs.stanford.edu)
/** \file serializableVector.h
    \brief Defines a serializableVector (supports .serialize(targetArchive) ) 
*/

#pragma once

#include "archive.h"
#include <vector>

namespace sdn {

  template<class T>
  class serializableVector : public std::vector<T> {
  public:
    void serialize(serial::Archive & ar) {
      
      SDN_DEBUG("serializableVector::serialize");
      
      int version = 1;
      unsigned int numElems;
      
      version = ar.beginBase(this,version,"sdn::serializableVector");
      
      if (ar.in())
        std::vector<T>::clear();
      else
        numElems = std::vector<T>::size();

      switch(version)
        {
        case 1:
          // Configurable
          ar.attribute("numElems", numElems);
          
          for(unsigned i=0;i<numElems;i++)
            if (ar.in()) {
              // read each element
              T newElem;
              std::vector<T>::push_back(newElem);
              std::vector<T>::back().serialize(ar);
            } else {
              // write out each element
              for(unsigned i=0;i<numElems;i++)
                ((std::vector<T>)(*this))[i].serialize(ar);
            }
          break;
        default:
          SDN_FATAL("Unrecognized sdn::serializableVector version");
        }
        
                
      ar.endBase();

    }
  };


}
