/// \author Ian Goodfellow

#include "varianceEstimator.h"
#include "logger.h"
using namespace sdn;

VarianceEstimator::VarianceEstimator()
{
  n_ = 0;
}

void VarianceEstimator::processSample(const Matrix & x)
{
  //For first sample, we have to initialize the size of the internal matrices
  if (n_ == 0)
    {
      ++n_;
      mean_ = x;
      m2_ = Matrix::Zero(x.rows(),x.cols());     
    }
  else
    //For all samples after first
    {
      ++n_;
      Matrix delta = x - mean_;
      mean_ += delta  / Scalar(n_);
      m2_ += delta.cwise() * (x - mean_);
    }
}

Matrix VarianceEstimator::getMean()
{
  SDN_ASSERT(n_ > 0);
  return mean_;
}

Matrix VarianceEstimator::getVariance()
{
  SDN_ASSERT(n_ > 0);
  return m2_ /Scalar(n_-1);
}
