/// \author Ian Goodfellow

#pragma once

#include "matrix.h"

namespace sdn
{
  /** Estimates the variance (and mean) of each element of a matrix. Uses the numerically
      stable single-pass algorithm described on pg 232 of Seminumerical Algorithms
      by Donald Knuth */
  

  class VarianceEstimator
  {
  public:
    VarianceEstimator();
    
    /**  $x_{i,j}$ is a single sample from a random variable $X_{i,j}$. If you have $n$ samples you should call processSample $n$ times. The VarianceEstimator calculates the variance of x.rows()*x.cols() variables simultaneously. */
    void processSample(const Matrix & x);
    Matrix getMean();
    Matrix getVariance();
  protected:
    unsigned n_;
    Matrix mean_;
    Matrix m2_;
  };



}
