#pragma once

enum CUDA_MATRIX_OPTYPE {
  CWISE_ADDITION,
  CWISE_SUBTRACTION,
  CWISE_MULTIPLICATION,
  CWISE_DIVISION,
  CWISE_POWER,
  CWISE_EXP,
  CWISE_LOG,
  CWISE_ABS,
  CWISE_TANH,
  CWISE_COSH,
  CWISE_SQRT,
  CWISE_CONSTANT,
	CWISE_SETMAX,
	CWISE_COS,
	CWISE_SIN
};

void cudaMatrixTranspose(float* matrixOut, float* matrixIn, int pm, int pn);

int cudaMatrixBlockCopy(float* matrixLhs, float* matrixRhs, int lpm, int lpn,
			int lsrow, int lscol, int rpm, int rpn, int rsrow,
			int rscol, int nrow, int ncol, bool rtrans);

int cudaMatrixCWiseOp(float* matrixOut, float* matrixLhs, float* matrixRhs,
		      int opm, int opn, int lpm, int lpn, int lsrow, int lscol,
		      int rpm, int rpn, int rsrow, int rscol, int nrow,
		      int ncol, bool inPlace, bool rtrans,
		      CUDA_MATRIX_OPTYPE type);

int cudaMatrixCWiseScalarOp(float* matrixOut, float* matrixIn, float scalar,
			    int opm, int opn, int pm, int pn, int srow,
			    int scol, int nrow, int ncol, bool inPlace,
			    CUDA_MATRIX_OPTYPE type);

int cudaMatrixSum(float* result, float* matrixIn, int pm, int pn,
		  int srow, int scol, int nrow, int ncol);

int cudaMatrixTrace(float *result, float *matrixIn, int pm, int pn,
		    int n);

int cudaMatrixMin(float* result, float* matrixIn, int pm, int pn,
		  int srow, int scol, int nrow, int ncol);

int cudaMatrixMax(float* result, float* matrixIn, int pm, int pn,
		  int srow, int scol, int nrow, int ncol);

int cudaMatrixSquaredNorm(float* result, float* matrixIn, int pm, int pn,
			  int srow, int scol, int nrow, int ncol);

int cudaMatrixPartialSum(float* result, float* matrixIn, int pm, int pn,
			 bool colwise, bool trans);

int cudaMatrixPartialNormSquared(float* result, float* matrixIn, int pm, int pn,
			 bool colwise, bool trans);

int cudaMatrixAsDiagonal(float* matrixOut, float* matrixIn, int opm, int pm,
			 int pn, int srow, int scol, int nrow, int ncol);

int cudaMatrixSetZero(float* matrixIn, int n);

int cudaMatrixSet(float* matrixIn, float scalar, int pm, int pn, int m, int n);

int cudaMatrixCopy(float* matrixOut, float* matrixIn, int n);

int cudaMatrixNumberRandomSeeds(int pm, int pn);

int cudaMatrixInitRandom(unsigned **randSeeds, int &nSeeds, int pm, int pn);

int cudaMatrixFillRandomUniform(float *matrixIn, unsigned *randSeeds, int pm, int pn, int m, int n);

int cudaMatrixFillRandomGaussian(float *matrixIn, unsigned *randSeeds, int pm, int pn, int m, int n);
