#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <cublas.h>
//#include "../base/logger.h"

#include "cudaVector.h"

#include <assert.h>
// Decouple from SDN
#ifndef SDN_ASSERT
#define SDN_ASSERT(x) assert(x)
#endif

#ifndef SDN_FATAL
#define SDN_FATAL(x) std::cerr << x << "\n"; exit(-1);
#endif

inline bool checkSuccess(cublasStatus status) {
  return status == CUBLAS_STATUS_SUCCESS;
}

sdn::CudaVector sdn::CudaVector::Zero(int m) {
  return Constant(m, 0.0);
}

sdn::CudaVector sdn::CudaVector::Ones(int m) {
  return Constant(m, 1.0);
}

sdn::CudaVector sdn::CudaVector::Constant(int m, float scalar) {
  SDN_ASSERT(m > 0);
  return CudaVector(m, NULL, true).setConstant(scalar);
}

sdn::CudaVector::CudaVector()
  : CudaMatrix() { }

sdn::CudaVector::CudaVector(int m)
  : CudaMatrix(m, 1) { }

sdn::CudaVector::CudaVector(const CudaVector& other)
  : CudaMatrix(other) { }

sdn::CudaVector::CudaVector(const CudaMatrix& other)
  : CudaMatrix(other) {
  SDN_ASSERT(other.rows() == 1 || other.cols() == 1);
  
  if(other.cols() == 1 && transpose_)
    forceTranspose();

  if(other.rows() == 1) {
    if(transpose_) transpose_ = false;
    else {
      transpose_ = true;
      forceTranspose();
    }
  }
}

sdn::CudaVector::CudaVector(const CudaMatrixBlock& block)
  : CudaMatrix(block) {
  SDN_ASSERT(block.rows() == 1 || block.cols() == 1);

  if(block.cols() == 1 && transpose_)
    forceTranspose();

  if(block.rows() == 1) {
    if(transpose_) transpose_ = false;
    else {
      transpose_ = true;
      forceTranspose();
    }
  }
}

float& sdn::CudaVector::operator()(int i) {
  return CudaMatrix::operator()(i, 0);
}

const float sdn::CudaVector::operator()(int i) const {
  return CudaMatrix::operator()(i, 0);
}

float& sdn::CudaVector::operator[](int i) {
  return CudaMatrix::operator()(i, 0);
}

const float sdn::CudaVector::operator[](int i) const {
  return CudaMatrix::operator()(i, 0);
}

void sdn::CudaVector::resize(int m) {
  CudaMatrix::resize(m, 1);
}

sdn::CudaMatrix::CudaMatrixBlock
sdn::CudaVector::segment(int start, int size) {
  return block(start, 0, size, 1);
}

const sdn::CudaMatrix::CudaMatrixBlock
sdn::CudaVector::segment(int start, int size) const {
  return block(start, 0, size, 1);
}

int sdn::CudaVector::size() const {
  return rows();
}

sdn::CudaMatrix::CudaMatrixBlock
sdn::CudaVector::end(int num) const {
  return CudaMatrixBlock(*this).end(num);
}

sdn::CudaVector& sdn::CudaVector::operator=(const CudaMatrix& rhs) {
  return (*this = CudaMatrixBlock(rhs));
}

sdn::CudaVector& sdn::CudaVector::operator=(const CudaMatrixBlock& rhs) {
  SDN_ASSERT(rhs.rows() == 1 || rhs.cols() == 1);

  CudaMatrix::operator=(rhs);

  if(rhs.cols() == 1 && transpose_)
    forceTranspose();

  if(rhs.rows() == 1) {
    if(transpose_) transpose_ = false;
    else {
      transpose_ = true;
      forceTranspose();
    }
  }

  return *this;
}

sdn::CudaVector::CudaVector(int m, float* vector, bool onCPU)
  : CudaMatrix(m, 1, vector, onCPU, false) { }
