#pragma once
#include "cudaMatrix.h"

// Cublas documentation:
// http://developer.download.nvidia.com/compute/cuda/2_1/toolkit/docs/CUBLAS_Library_2.1.pdf

namespace sdn {
  class CudaVector : public CudaMatrix {
  public:
    // CudaVector static allocators (start in GPU memory)
    static CudaVector Zero(int m);
    static CudaVector Ones(int m);
    static CudaVector Constant(int m, float scalar);

    // CudaVector constructors
    CudaVector();
    CudaVector(int m);
    CudaVector(const CudaVector& other);
    CudaVector(const CudaMatrix& other);
    CudaVector(const CudaMatrixBlock& block);

    // CudaVector element getting/setting
    float& operator()(int i);
    const float operator()(int i) const;
    float& operator[](int i);
    const float operator[](int i) const;

    // CudaVector binary operations
    CudaVector& operator=(const CudaMatrix& rhs);
    CudaVector& operator=(const CudaMatrixBlock& rhs);

    // CudaVector resize
    void resize(int m);

    // CudaVector segment
    CudaMatrixBlock segment(int start, int size);
    const CudaMatrixBlock segment(int start, int size) const;
    
    // CudaVector getters
    int size() const;
    CudaMatrixBlock end(int num) const;

  protected:
    CudaVector(int m, float* vector, bool onCPU);
  };
}
