#pragma once
#include "cudaMatrixUtils.h"

struct entry {
  int index;
  float value;
};

struct indexEntry {
  int index;
  int value;
};

#define BLOCK_DIM (256)
#define HALF_DIM (16) //should be approximately sqrt(BLOCK_DIM) and must divide it evenly

int sparseCudaMatrixCWiseOp(entry* matrixOut, entry* matrixLhs, entry* matrixRhs,
			    int numEntries, int pNumEntries, CUDA_MATRIX_OPTYPE type);

int sparseCudaMatrixCWiseScalarOp(entry* matrixOut, entry* matrixIn, float scalar,
				  int numEntries, int pNumEntries,
				  CUDA_MATRIX_OPTYPE type);

int sparseCudaMatrixSum(float* result, entry* matrixIn, int numEntries, int pNumEntries);

int sparseCudaMatrixMin(float* result, entry* matrixIn, int numEntries, int pNumEntries);

int sparseCudaMatrixMax(float* result, entry* matrixIn, int numEntries, int pNumEntries);

int sparseCudaMatrixSquaredNorm(float* result, entry* matrixIn, int numEntries,
				int pNumEntries);

int sparseCudaMatrixEntryCopy(entry* matrixOut, entry* matrixIn, int pNumEntries);

int sparseCudaMatrixIndexCopy(indexEntry* matrixOut, indexEntry* matrixIn,
			      int pNumEntries);

int sparseCudaMatrixTestEquality(int* result, entry* matrixLhs, entry* matrixRhs,
				 int numEntries, int pNumEntries);

int sparseCudaMatrixVerifyEntries(int* result, entry* matrixLhs, entry* matrixRhs,
				  int numEntries, int pNumEntries);

int sparseCudaMatrixVerifyIndexEntries(int* result, indexEntry* matrixLhs,
				       indexEntry* matrixRhs, int numEntries,
				       int pNumEntries);

int sparseCudaMatrixMultiply(indexEntry* rowIndexOut, entry* rowOut,
			     indexEntry* colIndexOut, entry* colOut,
			     indexEntry* lhsIndexIn, entry* lhsIn,
			     indexEntry* rhsIndexIn, entry* rhsIn,
			     int lhsRows, int rhsCols);

int sparseTimesDenseToDense(float* matrixOut, indexEntry* rowIndexIn, entry* rowIn,
			    float* rhsIn, int lhsRows, int lhsPRows, int m, int n, int pm, int pn);

int sparseTimesDenseToSparse(indexEntry* rowIndexOut, entry* rowOut,
			     indexEntry* colIndexOut, entry* colOut,
			     indexEntry* rowIndexIn, entry* rowIn,
			     float* rhsIn, int lhsRows, int m, int n, int pm, int pn);

//the mother of all methods: 20 parameters
int cudaMatrixTimesCWiseTimesSparse(indexEntry* rowIndexOut, entry* rowOut,
				    indexEntry* colIndexOut, entry* colOut,
				    float* lhsIn, float* rhsIn,
				    indexEntry* rowIndexIn, entry* rowIn,
				    indexEntry* colIndexIn, entry* colIn,
				    int lm, int ln, int lpm, int lpn,
				    int rm, int rn, int rpm, int rpn,
				    int sr, int sc);

// specialized methods for localrf
int onlineOrthogonalization(indexEntry *rowIndex, indexEntry *colIndex, 
			    entry *rowMatrixIn, entry *colMatrixIn, 
			    entry *rowMatrixOut, entry *colMatrixOut, 
			    int sr, int sc, int maxNonZeroPerRow, 
			    int windowSize, int step, int windowsPerRow, 
			    int windowsPerCol, int outputChannels, 
			    int inputChannels);

int weightsTimesData(float* matrixOut, entry* rowIn, float* rhsIn, int outPm, int lhsRows, int nonZeroPerRow, int m, int n, int pm);

// redux methods

int sparseCudaMatrixPartialSum(float *matrixOut, int pdim, int dim, indexEntry *index, entry *matrixIn);
