//Author: Ian Goodfellow

#include "basicImageCollection.h"
#include "../base/logger.h"
#include "../base/matrixUtils.h"
#include <Eigen/Array>

using namespace sdn;
using namespace std;

BasicImageCollection::BasicImageCollection() : _rows(0), _cols(0)
{

}

BasicImageCollection::~BasicImageCollection()
{

}

void BasicImageCollection::setDynamicRange(Scalar minVal, Scalar maxVal)
{
  Scalar oldMinVal = _minVal;
  Scalar oldMaxVal = _maxVal;

  Dataset::setDynamicRange(minVal, maxVal);

  _fixDynamicRange(oldMinVal, oldMaxVal);

}

void BasicImageCollection::_fixDynamicRange(Scalar curMinVal, Scalar curMaxVal)
{
  if (curMinVal == _minVal && curMaxVal == _maxVal)
    return;

  SDN_ASSERT(curMinVal != curMaxVal);

  
  Scalar scale = (_maxVal - _minVal) / (curMaxVal - curMinVal);

  for (unsigned i = 0; i < _images.size(); i++)
  {
/// \todo : make more efficient
	_images[i].cwise() -= curMinVal;
	_images[i] *= scale;
        _images[i].cwise() += _minVal;
  }
}


unsigned int BasicImageCollection::getRandomExampleAsColumn(Matrix & x)
{

  SDN_ASSERT(_images.size());
  unsigned i = rand() % _images.size();
  /// \todo-- use a better random number generator
  /// \todo-- give the experiment control over the seed
  // (Both of these could be handled by the ImageCollection base class)
  
  SDN_ASSERT(x.rows() == _images[i].rows());
  x = _images[i];

  return _labels(i);
}

unsigned int BasicImageCollection::getRandomExample(Matrix & x)
{
  SDN_ASSERT(_images.size());
  unsigned i = rand()% _images.size();
  //todo-- change this to use whatever rng the above method ends up using

  SDN_ASSERT_MSG(x.rows() == (int) _rows,"x.rows()=" << x.rows() << " and _rows=" << _rows);
  SDN_ASSERT_MSG(x.cols() == (int) _cols, "x.cols()=" << x.cols() << " and _cols=" << _cols);
 
  matrixSubColToMatrix(_images[i],0,0,x);  

  return _labels(i);
}

void BasicImageCollection::pyramidDown()
{
  for (unsigned i = 0; i < _images.size(); i++)
    {
      _images[i] = matrixToVector(sdn::pyramidDown(vectorToMatrix(_images[i],_rows,_cols)));      
    }

  _rows /= 2;
  _cols /= 2;
}

std::vector<Matrix>& BasicImageCollection::getImgRef() {
  return _images;
}

Eigen::VectorXi& BasicImageCollection::getLabelRef() {
  return _labels;
}

unsigned int BasicImageCollection::getExampleAsColumn(Matrix &x, unsigned int index)
{
	SDN_ASSERT(index >= 0 && index < _images.size());
	SDN_ASSERT_MSG(x.rows() == _images[index].rows(), "x.rows = " << x.rows() << " images rows = " << _images[index].rows());

	x = _images[index];

	return _labels(index);
}	
