//Author: Ian Goodfellow
//File: basicImageCollection.h
//Description: base class for image collections that are literally just
//collections of images, no temporal information,etc.
//Subclass this to implement loading of different image formats

#pragma once

#include "dataset.h"
#include <vector>


namespace sdn
{
  /// An ImageCollection of fixed-sized images that don't need to be cropped
/** This is useful for something like MNIST. Fixing the example size
allows them to be stored as columns, which speeds up training. */
  class BasicImageCollection : public Dataset
  {	 
  public:
    BasicImageCollection();
    virtual ~BasicImageCollection();
    

    void setDynamicRange(Scalar minVal, Scalar maxVal);

    bool containsLabels() {
      return true;
    }
    unsigned int numLabels() {
      return _num_labels;
    }
    
		unsigned int numExamples() {
			return _images.size();
		}

		unsigned int getExampleAsColumn(Matrix& x, unsigned int index);

    unsigned int getRandomExampleAsColumn(Matrix & x);
    unsigned int getRandomExample(Matrix & x);

    bool supportsExampleSize(unsigned rows, unsigned cols)
    {
      return rows == _rows && cols == _cols;
    }


    //Converts all images to half their regular size
    void pyramidDown();

    unsigned getPreferredRows() { return _rows; }
    unsigned getPreferredCols() { return _cols; }

    std::vector<Matrix>& getImgRef();
    Eigen::VectorXi& getLabelRef();

  protected:
    ///if values in _images are actually in the range [curMinVal,curMaxVal], this scale/translates everything to range [_minVal, _maxVal]
    void _fixDynamicRange(Scalar curMinVal, Scalar curMaxVal);
    
  protected:

    /// These should be stored as column examples
    /**
       BasicImageCollection assumes that the querying Network will
       always use the same patch size as the size of the images.
       Training and testing are thus easily sped up by storing
       the examples as columns. TODO-- make CroppableImageCollection
       subclass
    */
    std::vector<Matrix> _images;
    Eigen::VectorXi _labels;

    unsigned int _num_labels;

    unsigned _rows;
    unsigned _cols;
  };
}
