//Author: Daniel Chia

#include "croppableImageCollection.h"
#include "../base/logger.h"
#include <Eigen/Array>

using namespace sdn;
using namespace std;

CroppableImageCollection::CroppableImageCollection() : _rows(0), _cols(0)
{

}

CroppableImageCollection::~CroppableImageCollection()
{

}

void CroppableImageCollection::setDynamicRange(Scalar minVal, Scalar maxVal)
{
  Scalar oldMinVal = _minVal;
  Scalar oldMaxVal = _maxVal;

  Dataset::setDynamicRange(minVal, maxVal);

  _fixDynamicRange(oldMinVal, oldMaxVal);

}

void CroppableImageCollection::_fixDynamicRange(Scalar curMinVal, Scalar curMaxVal)
{
  if (curMinVal == _minVal && curMaxVal == _maxVal)
    return;

  SDN_ASSERT(curMinVal != curMaxVal);

  
  Scalar scale = (_maxVal - _minVal) / (curMaxVal - curMinVal);

  for (unsigned i = 0; i < _images.size(); i++)
  {
/// \todo : make more efficient
	_images[i].cwise() -= curMinVal;
	_images[i] *= scale;
        _images[i].cwise() += _minVal;
  }
}


unsigned int CroppableImageCollection::getRandomExample(Matrix & x)
{
  SDN_ASSERT_MSG(_requestedRows > 0, "CroppableImageCollection::getRandomExample called without first calling requestExampleSize");
  SDN_ASSERT_MSG(x.rows() == _requestedRows, "Expected request for "<<_requestedRows<<" rows, got request for "<<x.rows());
  SDN_ASSERT_MSG(x.cols() == _requestedCols, "Expected request for "<< _requestedCols<<" cols, got request for "<<x.cols());

  unsigned int index = g_.randUnif(0, _images.size()-1);

  Matrix *p = &_images[index];

  x = p->block(g_.randUnif(0, p->rows() - _requestedRows),
		  	   g_.randUnif(0, p->cols() - _requestedCols),
			   _requestedRows,
			   _requestedCols);

  return _labels[index];
}

