//Author: Daniel Chia
//File: croppableImageCollection.h
//Description: based of basicImageCollection, base class for image collections that are literally just
//collections of images, no temporal information,etc.
//Differs from basicImageCollection in the ability to request cropped patches
//Subclass this to implement loading of different image formats

#pragma once

#include "dataset.h"
#include <vector>
#include "../base/random.h"


namespace sdn
{
  class CroppableImageCollection : public Dataset
  {	 
  public:
    CroppableImageCollection();
    virtual ~CroppableImageCollection();
    

    void setDynamicRange(Scalar minVal, Scalar maxVal);

    bool containsLabels() {
      return true;
    }
    unsigned int numLabels() {
      return _num_labels;
    }
    
    unsigned int getRandomExample(Matrix & x);

    bool supportsExampleSize(unsigned rows, unsigned cols)
    {
      return rows <= _rows && cols <= _cols;
    }


    unsigned getPreferredRows() { return _rows; }
    unsigned getPreferredCols() { return _cols; }



  protected:
    ///if values in _images are actually in the range [curMinVal,curMaxVal], this scale/translates everything to range [_minVal, _maxVal]
    void _fixDynamicRange(Scalar curMinVal, Scalar curMaxVal);

	RandomIntGenerator<> g_;
    
  protected:

    std::vector<Matrix> _images;
		Eigen::VectorXi _labels;

    unsigned int _num_labels;

    unsigned _rows;
    unsigned _cols;
  };
}
