#include "dataset.h"
#include "imageCollectionAMAT.h"
#include "imageCollectionCIFAR.h"
#include "basicImageCollectionCIFAR.h"
#include "pairedVideoCollection.h"
#include "imageDirectories.h"
#include "spectrogramCollectionAMAT.h"
#include <boost/shared_ptr.hpp>
#include <vector>
#include "../base/logger.h"
#include "../base/matrixUtils.h"
#include "../base/xmlUtils.h"
#include "../base/registry.h"
#include "../base/configManager.h"
#include "videoCollection.h"
#include "gratingDataset.h"

using namespace std;
using namespace boost;
using namespace sdn;

sdn::Dataset::Dataset(Scalar minVal, Scalar maxVal) : _minVal(minVal), _maxVal(maxVal), _requestedRows(-1), _requestedCols(-1)
{

}

void sdn::Dataset::load(const char * filepath)
{
  XMLNode root = openXML(filepath);
  load(filepath);
}

void sdn::Dataset::setDynamicRange(Scalar minVal, Scalar maxVal)
{
  _minVal = minVal;
  _maxVal = maxVal;
}


sdn::Dataset::~Dataset()
{

}

sdn::Registry<sdn::Dataset> & sdn::DatasetRegistry()
{
  static sdn::Registry<sdn::Dataset> registry("sdn::Dataset");
  return registry;
}
//Auto-configuration

namespace sdn
{

	class DatasetParser : public ConfigParser
	{
		public:
			DatasetParser() {}
			~DatasetParser() {}
		
			void topics(vector<string> & topics)
			{
				topics.push_back("sdn::Dataset");
			}

			void usage(ostream & os) const
			{
				os << "<sdn::Dataset id=\"name of object\" etc.>" << endl; 
			}

			void readConfig(XMLNode & node)
			{
				DatasetRegistry().registerInstance(node);
			}
	};

  ConfigListener gDatasetListener(shared_ptr<ConfigParser>(new DatasetParser()));
};

void Dataset::requestExampleSize(int rows, int cols)
{
  //Don't use SDN_ASSERT_MSG here because we might set it to compile out,
  //but this isn't debugging code, it is user-interaction code
  if (!supportsExampleSize(rows,cols))
    {
      SDN_FATAL("The dataset you are using does not support the size "<<rows<<"x"<<cols);
    }


  _requestedRows = rows;
  _requestedCols = cols;
}

bool Dataset::containsLabels() {
  return false;
}

unsigned int Dataset::numLabels() {
  return 0;
}

unsigned int Dataset::getRandomExampleAsColumn(Matrix & x)
{
  SDN_ASSERT(x.cols() == 1);
  SDN_ASSERT(x.rows() == _requestedRows * _requestedCols);

  Matrix temp(_requestedRows,_requestedCols);

  int label = getRandomExample(temp);

  matrixToMatrixCol(temp,x);

  return label;
}


bool Dataset::supportsTrajectoryIndexing()
{
  return false;
}

/// Like getRandomExample, but also writes out an index so this example can be found later
unsigned int Dataset::getRandomIndexedExample(Matrix & x, unsigned & index)
{
  SDN_ASSERT_MSG(! supportsTrajectoryIndexing(), "Some subclass of Dataset is falsely claiming to support getRandomIndexedExample (returns true from supportsTrajectoryIndexing)");
  SDN_FATAL("getRandomIndexedExample called on a dataset that does not support trajectory indexing");
  return 0;
}
    
    /// Returns a sequence of related patches. Relatedness will mean different things in different contexts-- it could be patches that appear at the same location in the frame, but with different time coordinates, or it could be gratings that all have the same orientation and frequency but different phase, etc. The central patch will be the patch that was returned by getRandomIndexedExample with index index
std::vector<Matrix> Dataset::getTrajectoryFromIndex(unsigned index)
{
  SDN_ASSERT_MSG(! supportsTrajectoryIndexing(), "Some subclass of Dataset is falsely claiming to support getTrajectoryFromIndex (returns true from supportsTrajectoryIndexing)");
  SDN_FATAL("getTrajectoryFromIndex called on a dataset that does not support trajectory indexing");
  return vector<Matrix> ();
}

unsigned Dataset::getExampleFromIndex(Matrix & x, unsigned index)
{
  SDN_ASSERT_MSG(! supportsTrajectoryIndexing(), "Some subclass of Dataset is falsely claiming to support getExampleFromIdex (returns true from supportsTrajectoryIndexing)");
  SDN_FATAL("getExampleFromIndex called on a dataset that does not support trajectory indexing");
  return false;
}
    
/// Returns true if the the trajectories associated with two indices overlap
bool Dataset::trajectoryOverlap(unsigned index1, unsigned index2)
{
  SDN_ASSERT_MSG(! supportsTrajectoryIndexing(), "Some subclass of Dataset is falsely claiming to support trajectoryOverlap (returns true from supportsTrajectoryIndexing)");
  SDN_FATAL("trajectoryOverlap called on a dataset that does not support trajectory indexing");
  return true;
}

SDN_AUTOREGISTER(Dataset, GratingDataset);
SDN_AUTOREGISTER(Dataset, ImageCollectionAMAT);
SDN_AUTOREGISTER(Dataset, ImageCollectionCIFAR);
SDN_AUTOREGISTER(Dataset, BasicImageCollectionCIFAR);
SDN_AUTOREGISTER(Dataset, PairedVideoCollection);
SDN_AUTOREGISTER(Dataset, VideoCollection);
SDN_AUTOREGISTER(Dataset, ImageDirectories);
SDN_AUTOREGISTER(Dataset, SpectrogramCollectionAMAT);
