/// \author Ian Goodfellow
/// \file grating.h
/// \brief Draws 2D sin gratings

#pragma once

#include "../base/matrix.h"
#include <Eigen/Core>

namespace sdn
{

  class Grating
  {

  public:
    
    Grating(unsigned rows, unsigned cols, double minValue, double maxValue);
    Matrix getImage();
    
    void setFrequency(double f);
    void setOrientation(double t);
    void setPhaseShift(double p);
    double getFrequency() const;
    double getOrientation() const;
    double getPhaseShift() const;

    void setSamplingRate(unsigned xSamplesPerPixel, unsigned ySamplesPerPixel);

    unsigned rows();
    unsigned cols();

  protected:
    
    void render_();
    void drawPixel_(unsigned i, unsigned j);
    double eval_(double x, double y) const;
    
    protected:
    
    Matrix image_;

    //if true, parameters have changed since image was last rendered
    bool dirty_;

    // f (x,y) = baseline_ + amplitude_ * sin( frequency_ * ( x * cos(theta_) + y*sin(theta_)) - phaseShift_  )

    double amplitude_;
    double baseline_;
    double frequency_;
    double theta_;
    double phaseShift_;

    //Right now the following parameters are hardcoded by the constructor, but if you add an interface to control them they should work fine
    double minX_;
    double width_;
    double maxY_;
    double height_;
    double pixelWidth_;
    double pixelHeight_;
    double sampleWidth_;
    double sampleHeight_;
    
    //Antialiasing parameters (hardcoded to 8x8)
    //Antialiasing setup is just to sample on an even grid and average together all samples within the same pixel, no gaussian weighting or anything
    int xSamplesPerPixel_;
    int ySamplesPerPixel_;


  };

}
