#include "imageCollectionAMAT.h"
#include "xmlParser/xmlParser.h"
#include "../base/logger.h"

using namespace sdn;
using namespace std;

ImageCollectionAMAT::ImageCollectionAMAT()
{
  _rows = 28;
  _cols = 28;
}

ImageCollectionAMAT::~ImageCollectionAMAT()
{
}

void ImageCollectionAMAT::load(XMLNode & root)
{
  SDN_ASSERT(root.nChildNode() == 0);
  //todo-- change asserts to exception-based error handling
  SDN_ASSERT(root.getName());
  string name = root.getName();
  SDN_ASSERT(name == "sdn::Dataset");
  SDN_ASSERT(root.getAttribute("id"));
  string id = root.getAttribute("id");
  SDN_ASSERT(id == "sdn::ImageCollectionAMAT");
  SDN_ASSERT(root.getAttribute("path"));
  string path = root.getAttribute("path");
  
  ifstream ifs;
  ifs.open(path.c_str());

  SDN_ASSERT_MSG(!ifs.fail(), "Could not open "<<path);

  int start = -1, numImages = -1;

  SDN_ASSERT(root.getAttribute("set"));
  string set = root.getAttribute("set");

  if (set == "train")
    {
      start = 0;
      numImages = 10000;
    }
  else if (set == "validate")
    {
      start = 10000;
      numImages = 2000;
    }
  else if (set == "test")
    {
      start = 0;
      numImages = 50000;
    }
  else
    SDN_FATAL("Unrecognized AMAT set type "<<set);

  Scalar temp; //used for reading values that are ignored

  //Skip over input preceding our dataset
  for (int i = 0; i < start; i++)
    {
      Scalar temp;
      for (int j = 0; j < 785; j++)
	ifs >> temp;
    }

  assert(!ifs.fail());

  _images.resize(numImages,Matrix::Zero(28*28,1));
  _labels = Eigen::VectorXi::Zero(numImages);

  for (int i = 0; i < numImages; i++)
   {

      //Matrix debug(28,28);

      for (int j = 0; j < 28; j++)
	{
	  for (int k = 0; k < 28; k++)
	    {
	      ifs >> _images[i](j+k*28,0);
              //ifs >> debug(j,k);
	    }
	}

	//matrixSubColToMatrix(_images[i],0,0,debug);
	//saveMatrixAsImage("/tmp/matrix.jpg",debug);    <- this was coming out OK


      Scalar label;
      ifs >> label; //todo-- store label somewhere

      _labels(i) = (int)label;
     
      SDN_ASSERT(!ifs.fail());

    }

  //Read one more value to see if we are at EOF or not
  ifs >> temp;

  if (set=="train")
    SDN_ASSERT(!ifs.fail()); //training set should be followed by validation set
  if (set=="test" || set=="validate")
    SDN_ASSERT(ifs.fail()); //test and validation set should be at EOF

  ifs.close();

  _num_labels = _labels.maxCoeff() + 1;

  _fixDynamicRange(0,1);

  if (root.getAttribute("pyramidDown"))
    {
      int pyramids = atoi(root.getAttribute("pyramidDown"));
      for (unsigned i = 0; i < pyramids; i++)
	{
	  pyramidDown();
	}
    }
}
