#include "imageCollectionCIFAR.h"
#include "xmlParser/xmlParser.h"
#include "../base/logger.h"

using namespace sdn;
using namespace std;

ImageCollectionCIFAR::ImageCollectionCIFAR()
{
  _rows = 32;
  _cols = 32;
}

ImageCollectionCIFAR::~ImageCollectionCIFAR()
{
}

void ImageCollectionCIFAR::load(XMLNode & root)
{
  SDN_ASSERT(root.nChildNode() == 0);
  //todo-- change asserts to exception-based error handling
  SDN_ASSERT(root.getName());
  string name = root.getName();
  SDN_ASSERT(name == "sdn::Dataset");
  SDN_ASSERT(root.getAttribute("id"));
  string id = root.getAttribute("id");
  SDN_ASSERT(id == "sdn::ImageCollectionCIFAR");
  SDN_ASSERT(root.getAttribute("path"));
  string path = root.getAttribute("path");
  
  ifstream ifs;
  ifs.open(path.c_str());

  SDN_ASSERT_MSG(!ifs.fail(), "Could not open "<<path);

  int numImages = -1;

  SDN_ASSERT(root.getAttribute("set"));
  string set = root.getAttribute("set");

  if (set == "train")
    {
      numImages = 50000;
    }
  else if (set == "test")
    {
      numImages = 10000;
    }
  else
    SDN_FATAL("Unrecognized CIFAR set type "<<set);

	SDN_ASSERT(root.getAttribute("seed"));
	unsigned int seed = atoi(root.getAttribute("seed"));

	g_.setSeed(seed);

	Scalar minVal = 0, maxVal = 0;

  _images.resize(numImages,Matrix::Zero(_rows, _cols));
  _labels = Eigen::VectorXi::Zero(numImages);

  for (int i = 0; i < numImages; i++)
  {
		unsigned char temp;

		//read in label

		ifs.read((char *)&temp, 1);
		_labels[i] = temp;

		//read in image

		double tempDouble;

		for (unsigned int j = 0; j < _rows; j++)
			for (unsigned int k = 0; k < _cols; k++)
			{
				ifs.read((char *)&tempDouble, 8);
				_images[i](j,k) = tempDouble;
			}

		Scalar curMin = _images[i].minCoeff();
		Scalar curMax = _images[i].maxCoeff();

		if (i == 0 || curMin < minVal)
			minVal = curMin;
		if (i == 0 || curMax > maxVal)
			maxVal = curMax;
	}

	_fixDynamicRange(minVal, maxVal);


  ifs.close();

  _num_labels = _labels.maxCoeff() + 1;

  _fixDynamicRange(0,1);

}
