#include "imageDirectories.h"
#include "xmlParser/xmlParser.h"
#include "../base/logger.h"
#include "sdn/base.h"
#include <string>
#include <Eigen/Array>
#include <Eigen/Core>

using namespace sdn;
using namespace std;

ImageDirectories::ImageDirectories()
{
	_minRows = 0;
	_minCols = 0;
	_numLabels = 0;
}

ImageDirectories::~ImageDirectories()
{

}

void ImageDirectories::load(XMLNode & root)
{
	SDN_ASSERT(root.nChildNode() == 0);
	//todo-- change asserts to exception-based error handling
	SDN_ASSERT(root.getName());
	string name = root.getName();
	SDN_ASSERT(name == "sdn::Dataset");
	SDN_ASSERT(root.getAttribute("id"));
	string id = root.getAttribute("id");
	SDN_ASSERT(id == "sdn::ImageDirectories");
	SDN_ASSERT(root.getAttribute("path"));
	string path = root.getAttribute("path");
	SDN_ASSERT(root.getAttribute("seed"));
	unsigned int seed = atoi(root.getAttribute("seed"));
	SDN_ASSERT(root.getAttribute("numCategories"));
	_numLabels = atoi(root.getAttribute("numCategories"));	

	_g.setSeed(seed);

	vector<string> subdirs = listSubdirs(path);
	if (subdirs.size() < _numLabels)
	{
		SDN_VERBOSE("Number of image categories is less than specified");
		_numLabels = subdirs.size();
	}
	unsigned int currentImage = 0;
	Scalar minVal = 0;
	Scalar maxVal = 0;
	for (unsigned i = 0; i < _numLabels; i++)
	{
		vector<string> imageSet = listFiles(subdirs[i]);
		_images.resize(_images.size() + imageSet.size());
		_labels.resize(_labels.size() + imageSet.size());
		for (unsigned j = 0; j < imageSet.size(); j++)
		{
			loadMatrixFromImage(imageSet[j].c_str(), _images[currentImage], true);
			_labels[currentImage] = i+1;
			
			if ((_minRows == 0) || (_images[currentImage].rows() < _minRows))
				_minRows = _images[currentImage].rows();
			if ((_minCols == 0) || (_images[currentImage].cols() < _minCols))
				_minCols = _images[currentImage].cols();
			
			Scalar curMin = _images[currentImage].minCoeff();
			Scalar curMax = _images[currentImage].maxCoeff();	
			if ((i == 0 && j == 0) || curMin < minVal)
				minVal = curMin;
			if ((i == 0 && j == 0) || curMax > maxVal)
				maxVal = curMax;
			
			currentImage++;
		}
	}
	_fixDynamicRange(minVal,maxVal);
}

void ImageDirectories::_fixDynamicRange(Scalar curMinVal, Scalar curMaxVal)
{
  if (curMinVal == _minVal && curMaxVal == _maxVal)
    return;

  SDN_ASSERT(curMinVal != curMaxVal);

  
  Scalar scale = (_maxVal - _minVal) / (curMaxVal - curMinVal);

  for (unsigned i = 0; i < _images.size(); i++)
  {
/// \todo : make more efficient
	_images[i].cwise() -= curMinVal;
	_images[i] *= scale;
    _images[i].cwise() += _minVal;
  }
}

unsigned int ImageDirectories::getRandomExample(Matrix &x)
{
	SDN_ASSERT_MSG(_requestedRows > 0, "ImageDirectories::getRandomExample called without first calling requestExampleSize");
	SDN_ASSERT_MSG(x.rows() == _requestedRows, "Expected request for "<<_requestedRows<<" rows, got request for "<<x.rows());
	SDN_ASSERT(x.cols() == _requestedCols);

	unsigned int idx = _g.randUnif(0, _images.size()-1);
	Matrix * p = & _images.at(idx);

	x = p->block( _g.randUnif(0, p->rows()- _requestedRows),
		_g.randUnif(0, p->cols()- _requestedCols),
		_requestedRows,
		_requestedCols);

  return _labels(idx);
}
