//Author: Chris Baldassano
//File: imageDirectories.h
//Descript: loads a Dataset from images labeled by directory

#pragma once

#include "dataset.h"
#include <vector>
#include "../base/random.h"

namespace sdn
{
    /* A Dataset capable of loading images that are
   * separated by class into multiple directories
   * (This is the file format used for the 15 scenes collections and
   * for Caltech-101) */
  class ImageDirectories : public Dataset
  {
  public:
    
    ImageDirectories();
    virtual ~ImageDirectories();

    void load(XMLNode & root);
    unsigned int getRandomExample(Matrix &x);
	
    bool containsLabels() { return true;}
	unsigned int numLabels() { return _numLabels;}
	bool supportsExampleSize(unsigned rows, unsigned cols)
	{ return (rows <= _minRows && cols <= _minCols);}
    unsigned getPreferredRows() { return std::min(14u,_minRows);}
    unsigned getPreferredCols() { return std::min(14u,_minCols);}
    

  protected:
    ///if values in _images are actually in the range [curMinVal,curMaxVal], this scale/translates everything to range [_minVal, _maxVal]
    void _fixDynamicRange(Scalar curMinVal, Scalar curMaxVal);
    
    std::vector<Matrix> _images;
    Eigen::VectorXi _labels;

    unsigned int _numLabels;
    
    RandomIntGenerator<> _g;

	unsigned int _minRows;
	unsigned int _minCols;

  public:
    virtual void setRandomSeed(unsigned int seed) {
      _g.setSeed(seed);
    }
  };

}

//SDN_AUTOREGISTER_H(Dataset, ImageDirectories);
