// Author: Quoc V. Le
// \file invarianceTestDataset.h
// \brief common interface to invariance test datasets 
#pragma once
#include <vector>
#include <stdlib.h>
#include "grating.h"
#include "../base/random.h"

namespace sdn
{
  class InvarianceTestDataset
  {
  protected: 
    virtual void normalizePatch(Matrix &patch){
      /*Scalar mean = patch.sum() / (patch.rows() * patch.cols());
	patch.cwise() -= mean;
	Scalar maxAbs = max(patch.maxCoeff(), fabs(patch.minCoeff()));
	patch /= (maxAbs+ 1e-6);            // now all values between [-1,1]
	patch *= PATCH_RANGE;               // now all values between [-PATCH_RANGE, PATCH_RANGE] 
	//patch *= 1.5;
	*/
    }

  public: 
    
    // get exhaustive, detailed patches (a huge number of patches)
    virtual void getExhaustivePatches(std::vector<Matrix> &allPatches)=0;
    
    // get inexhaustive patches (a small number of patches)
    virtual void getInexhaustivePatches(int inv_type, std::vector<Matrix> &allPatches)=0;
    
    // generate locally varying patches 
    virtual void getLocalPatches(int type, 
				 int index,
				 std::vector<Matrix> &allPatches)=0;
    
    // get the number of test data collections
    virtual int getNumCollection()=0;
    // get the transformation category
    virtual int getCategory(int i)=0;
    // get the number of transformation category
    virtual int getNumCategory()=0;
    // get transformation type
    virtual std::string getType(int i)=0;
    // destructor
    virtual ~InvarianceTestDataset(){};
  };
}


