/// \author Ian Goodfellow <ia3n@cs.stanford.edu>
/// \file pairedVideoCollection.h
/// \brief A dataset class for holding pairs of frames sampled from videos

#pragma once


#include "dataset.h"
#include <map>
#include <vector>
#include "../base/random.h"

namespace sdn
{

  /// A Dataset capapble of storing videos as subsampled pairs
  class PairedVideoCollection : public Dataset
  {
  public:
    
    PairedVideoCollection();
    virtual ~PairedVideoCollection();
    
    void load (XMLNode & root);
    void setDynamicRange(Scalar minVal, Scalar maxVal);
    unsigned int getRandomExample(Matrix & x);

    bool supportsExampleSize(unsigned rows, unsigned cols)
    {
      return rows <= minRows_ && cols <= minCols_;
    }

    unsigned getPreferredRows()
    {
      return std::min(14u,minRows_);
    }

    unsigned getPreferredCols()
    {
      return std::min(14u,minCols_);
    }

  protected:

    /// These should be stored as 2D examples
    std::vector<std::pair<Matrix, Matrix> > _imagePairs;

    RandomIntGenerator<> g_;

    ///if values in _images are actually in the range [curMinVal,curMaxVal], this scale/translates everything to range [_minVal, _maxVal]
    void _fixDynamicRange(Scalar curMinVal, Scalar curMaxVal);

    unsigned minRows_;
    unsigned minCols_;

  };

}

//SDN_AUTOREGISTER_H(Dataset, VideoCollection);

