// Copyright 2009, All Rights Reserved.
// Author: Quoc V. Le (quocle@cs.stanford.edu).
/** \file plotter.cc
    \brief 
*/
#include <cv.h>
#include <Eigen/Core>
#include <Eigen/Array>
#include <fstream>
#include <highgui.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include "plotter.h"
#include <assert.h>
#include "matrix.h"

#define TEMPFILENAME "/tmp/"
using namespace std; 
using namespace sdn;
USING_PART_OF_NAMESPACE_EIGEN // import most common Eigen types 

Plotter::Plotter(){
}
/* main plot function, the idea is to 
   i) load data from a file 
   ii) display the plot using gnuplot
   iii) save the plot using xwd
   iv) convert the plot to jpg file
   the last two steps might not be carried out if save is set to false.
*/
void Plotter::Plot(const char *datafilename, const char *outputfilename, bool save){
  FILE *gnuplotPipe;
  if (save){
    gnuplotPipe = popen("gnuplot ","w");  
  }
  else{
    gnuplotPipe = popen("gnuplot -persist","w");  
  }
  if (gnuplotPipe) {  
    fprintf(gnuplotPipe, "set style data lines\n");  
    if (save){
      /*fprintf(gnuplotPipe, "shell\n");
	fprintf(gnuplotPipe, "xwd -name Gnuplot -silent > /tmp/screen.xwd\n");
	fprintf(gnuplotPipe, "exit\n");*/
      fprintf(gnuplotPipe, "set terminal png\n");
      fprintf(gnuplotPipe, "set output \'%s.png\'\n", outputfilename);
    }
    fprintf(gnuplotPipe, "plot \"%s\" using 1:2 \n", datafilename);      
    fprintf(gnuplotPipe, "quit\n");
    fflush(gnuplotPipe);  
    pclose(gnuplotPipe);  
    /*if (save){
      sprintf(command, " xwdtopnm < /tmp/screen.xwd | ppmtojpeg > %s.jpg\n", outputfilename);
      std::system(command);
      }*/
  }  
  else{
    std::cout <<"gnuplot cannot be found\n" << endl;
  }
}

void Plotter::WriteDataToTextFile(const MatrixXd &x, const MatrixXd &y, const char* datafilename){
  ofstream tempfile;
  tempfile.open(datafilename);
  for (int i=0;i<x.cols();i++){
    tempfile << x[i] << "\t" << y[i] << "\n";
  }
  tempfile.close();
}

void Plotter::WriteDataToTextFile(const MatrixXd &y, const char* datafilename){
  ofstream tempfile;
  tempfile.open(datafilename);
  for (int i=0;i<y.cols();i++){
    tempfile << i << "\t" << y[i] << "\n";
  }
  tempfile.close();
}

void Plotter::WriteDataToTextFile(const std::vector<double> &x, const std::vector<double> &y, 
				  const char* datafilename){
  ofstream tempfile;
  tempfile.open(datafilename);
  for (unsigned int i=0;i<x.size();i++){
    tempfile << x[i] << "\t" << y[i] << "\n";
  }
  tempfile.close();
}

void Plotter::WriteDataToTextFile(const vector<double> &y, const char* datafilename){
  ofstream tempfile;
  tempfile.open(datafilename);
  for (unsigned int i=0;i<y.size();i++){
    tempfile << i << "\t" << y[i] << "\n";
  }
  tempfile.close();
}



void Plotter::PlotAndSave(const MatrixXd &x, const MatrixXd &y, const char *outputfilename){
  assert(x.cols() == y.cols());
  assert(x.rows() == 1);
  assert(y.rows() == 1);
  int i = g_.randUnif(0,10000); 
  std::string s;
  std::stringstream out;
  out << i;
  s = out.str();
  char tempfilename[200];
  sprintf(tempfilename, "/tmp/%d.dat", i);
  WriteDataToTextFile(x, y, tempfilename);
  Plot(tempfilename, outputfilename, true);
} 


void Plotter::PlotAndShow(const MatrixXd &x, const MatrixXd &y){
  assert(x.cols() == y.cols());
  assert(x.rows() == 1);
  assert(y.rows() == 1);

  int i = g_.randUnif(0,10000); 
  std::string s;
  std::stringstream out;
  out << i;
  s = out.str();
  char tempfilename[200];
  sprintf(tempfilename, "/tmp/%d.dat", i);
  WriteDataToTextFile(x, y, tempfilename);
  Plot(tempfilename, "", false);
}


void Plotter::PlotAndSave(const MatrixXd &y, const char *outputfilename){
  
  int i = g_.randUnif(0,10000); 
  std::string s;
  std::stringstream out;
  out << i;
  s = out.str();
  char tempfilename[200];
  sprintf(tempfilename, "/tmp/%d.dat", i);
  WriteDataToTextFile(y, tempfilename);
  Plot(tempfilename, outputfilename, true);
} 


void Plotter::PlotAndShow(const MatrixXd &y){
    
  int i = g_.randUnif(0,10000); 
  std::string s;
  std::stringstream out;
  out << i;
  s = out.str();
  char tempfilename[200];
  sprintf(tempfilename, "/tmp/%d.dat", i);
  WriteDataToTextFile(y, tempfilename);
  Plot(tempfilename, "", false);
}



void Plotter::PlotAndSave(const std::vector<double> &x, const std::vector<double> &y, 
			  const char *outputfilename){
  assert(x.size() == y.size());
  int i = g_.randUnif(0,10000); 
  std::string s;
  std::stringstream out;
  out << i;
  s = out.str();
  char tempfilename[200];
  sprintf(tempfilename, "/tmp/%d.dat", i);
  WriteDataToTextFile(x, y, tempfilename);
  Plot(tempfilename, outputfilename, true);
} 


void Plotter::PlotAndShow(const std::vector<double> &x, const std::vector<double> &y){
  assert(x.size() == y.size());
  int i = g_.randUnif(0,10000); 
  std::string s;
  std::stringstream out;
  out << i;
  s = out.str();
  char tempfilename[200];
  sprintf(tempfilename, "/tmp/%d.dat", i);
  WriteDataToTextFile(x, y, tempfilename);
  Plot(tempfilename, "", false);
}


void Plotter::PlotAndSave(const std::vector<double> &y, const char *outputfilename){
  int i = g_.randUnif(0,10000); 
  std::string s;
  std::stringstream out;
  out << i;
  s = out.str();
  char tempfilename[200];
  sprintf(tempfilename, "/tmp/%d.dat", i);
  WriteDataToTextFile(y, tempfilename);
  Plot(tempfilename, outputfilename, true);
} 


void Plotter::PlotAndShow(const std::vector<double> &y){
  int i = g_.randUnif(0,10000); 
  std::string s;
  std::stringstream out;
  out << i;
  s = out.str();
  char tempfilename[200];
  sprintf(tempfilename, "/tmp/%d.dat", i);
  WriteDataToTextFile(y, tempfilename);
  Plot(tempfilename, "", false);
}
