// Copyright 2009, All Rights Reserved.
// Author: Quoc V. Le (quocle@cs.stanford.edu)
/** \file plotter.h
    \brief gnuplot plotting utilities.
*/
    
#pragma once

#include <cv.h>
#include <Eigen/Core>
#include <Eigen/Array>
#include <string>
#include <vector>
#include "../base/random.h"

/// gnuplot plotting wrapper
/**
 * gnuplot plotting wrapper
 * to use this program you need gnuplot, xwd, xwdtopnm, ppmtojpeg 
 */
namespace sdn
{
/// plots data

class Plotter
{

  
 public:
  Plotter();
  /// plot the graph, save to a file and disappear
  void PlotAndSave(const Eigen::MatrixXd &x, const Eigen::MatrixXd &y, const char* outputfilename);
  /// plot the graph, freeze the window and wait for the user to close the window
  void PlotAndShow(const Eigen::MatrixXd &x, const Eigen::MatrixXd &y);
  /// plot the graph, save to a file and disappear
  void PlotAndSave(const Eigen::MatrixXd &y, const char* outputfilename);
  /// plot the graph, freeze the window and wait for the user to close the window
  void PlotAndShow(const Eigen::MatrixXd &y);

  /// plot the graph, save to a file and disappear
  void PlotAndSave(const std::vector<double> &x, const std::vector<double> &y, const char* outputfilename);
  /// plot the graph, freeze the window and wait for the user to close the window
  void PlotAndShow(const std::vector<double> &x, const std::vector<double> &y);
  /// plot the graph, save to a file and disappear
  void PlotAndSave(const std::vector<double> &y, const char* outputfilename);
  /// plot the graph, freeze the window and wait for the user to close the window
  void PlotAndShow(const std::vector<double> &y);



 private:
  RandomIntGenerator<> g_;
  /// general plot command that has some gritty details
  void Plot(const char *datafilename, const char *outputfilename, bool save);
  /// write the data to a file before calling gnuplot to plot it
  void WriteDataToTextFile(const Eigen::MatrixXd &x, const Eigen::MatrixXd &y, const char* datafilename);
  /// write the data to a file before calling gnuplot to plot it
  /// in this version, only y is present, x is [1..n] where n is the size of y
  void WriteDataToTextFile(const Eigen::MatrixXd &y, const char* datafilename);
  /// write the data to a file before calling gnuplot to plot it
  void WriteDataToTextFile(const std::vector<double> &x, const std::vector<double> &y, const char* datafilename);
  /** write the data to a file before calling gnuplot to plot it
      in this version, only y is present, x is [1..n] where n is the size of y */
  void WriteDataToTextFile(const std::vector<double> &y, const char* datafilename);


};
}
