//Author: Andrew Saxe <asaxe@stanford.edu>
/** \file architectureUpdateCallback.cc
\brief A run time callback for adding layers to a stacked network
*/

#include "architectureUpdateCallback.h"
#include "stackedNetwork.h"

using namespace sdn;
using namespace std;
using namespace boost;

ArchitectureUpdateCallback::ArchitectureUpdateCallback() : TrainCallback(0)
		
{
  iter_ = 1000;
}

ArchitectureUpdateCallback::~ArchitectureUpdateCallback() {}

void ArchitectureUpdateCallback::load(XMLNode & node)
{
  SDN_ASSERT(node.getName());
  SDN_ASSERT(string("sdn::TrainCallback")==node.getName());
  SDN_ASSERT(node.getAttribute("id"));
  SDN_ASSERT(string("sdn::ArchitectureUpdateCallback")==node.getAttribute("id"));

  architecture_ = node.getChildNode("Architecture");  

  if(architecture_.isEmpty()) {
    SDN_WARN("No architecture specified for architectureUpdateCallback."); 
  }
  
  if (node.getAttribute("iter")) {
    iter_ = atoi(node.getAttribute("iter"));
  }

}

void ArchitectureUpdateCallback::callback(TrainState & state) { 

  if(state.iter == iter_) {

    shared_ptr<StackedNetwork> net = dynamic_pointer_cast<StackedNetwork>(state.network);
    SDN_ASSERT_MSG(net != NULL, "The network must be of type StackedNetwork to use the architectureUpdateCallback.");

    net->addLayers(architecture_);
  }
  
  nextIter += iter_; 
}


