/// \author Ian Goodfellow
#include "classificationPRCurve.h"
#include <algorithm>
#include "../base/logger.h"
#include "../gui/plotter.h"

using namespace sdn;
using namespace std;

ClassificationPRCurve::ClassificationPRCurve(const std::vector<double> & posScores, const std::vector<double> & negScores)
{
  tryThresholds(posScores,negScores,points_);
}


void ClassificationPRCurve::tryThresholds(const std::vector<double> & posScores, const std::vector<double> & negScores, std::vector<ClassificationPRCurvePoint> & output)
{


  output.clear();
  vector<double> p = posScores;
  vector<double> n = negScores;
  vector<double> all;


 
  all = n;
  for (unsigned i = 0; i < p.size(); i++)
    all.push_back(p[i]);
  //merge(p.begin(),p.end(), n.begin(),n.end(), all.begin());  // this is seg faulting, don't know why


 
  sort(p.begin(),p.end());
  sort(n.begin(),n.end());
  sort(all.begin(),all.end());


 

  //Index of first positive example.
  unsigned posIdx = 0;
  bool positivesExhausted = false;
  //Index of last negative example
  bool negativesBegun = false;
  unsigned negIdx = 0;


 

  for (unsigned i = 0; i < all.size(); i++)
    {
 
      //Skip repeated values
      if (i > 0 && all.at(i-1) == all.at(i))
	{
	  continue;
	}

      ClassificationPRCurvePoint pt;

      pt.threshold = all.at(i);

      if (!positivesExhausted)
	{
	  while ((!positivesExhausted) && (p.at(posIdx) < all.at(i)))
	    {
	      posIdx++;
	      if (posIdx == p.size())
		{
		  positivesExhausted = true;
		}
	    }
	}
 
      if (positivesExhausted)
	{
	  pt.truePositives = 0;
	  pt.falseNegatives = p.size();
	}
      else
	{
	  pt.truePositives = p.size() - posIdx;
	  pt.falseNegatives = posIdx;
	}
	  
 

      bool hitEnd = false;
      SDN_ASSERT_MSG(negIdx < n.size(), negIdx << " " << n.size());
      SDN_ASSERT_MSG(i < all.size(), i << " " << all.size());
      while (n.at(negIdx) < all.at(i))
	{
	  if (negIdx == n.size() - 1)
	    {
	      hitEnd = true;
	      break;
	    }
	  negIdx++;
	}

 

      if (!hitEnd)
	{
	  if (negIdx == 0)
	    {
	      negativesBegun = false;
	    }
	  else
	    {
	      negativesBegun = true;
	      negIdx--;
	    }
	}

 

      if (negativesBegun)
	{
	  pt.trueNegatives = negIdx + 1;
	  pt.falsePositives = n.size() - pt.trueNegatives;
	}
      else
	{
	  pt.trueNegatives = 0;
	  pt.falsePositives = n.size();
	}

      if (pt.truePositives + pt.falsePositives == 0)
	{
	  pt.precision = 1;
	}
      else
	{
	  pt.precision = double(pt.truePositives) / double(pt.truePositives+pt.falsePositives);
	}

 

      if (pt.falseNegatives + pt.truePositives == 0)
	{
	  pt.recall = 1;
	}
      else
	{
	  pt.recall = double(pt.truePositives)/double(pt.truePositives+pt.falseNegatives);
	}

 



      SDN_ASSERT(0.0 <= pt.precision);
      SDN_ASSERT(pt.precision<= 1.0);
      SDN_ASSERT(0.0 <= pt.recall);
      SDN_ASSERT(pt.recall <= 1.0);


      //Only add point if it isn't a repeat
      if (output.size() == 0 ||
	  pt.truePositives != output.back().truePositives || 
	  pt.falsePositives != output.back().falsePositives ||
	  pt.trueNegatives != output.back().trueNegatives ||
	  pt.falseNegatives != output.back().falseNegatives)
	{
	  output.push_back(pt);
	}


 
    }


 

  output.push_back(ClassificationPRCurvePoint());
  output.back().threshold = all.back()+0.001;
  output.back().truePositives = 0;
  output.back().falsePositives = 0;
  output.back().trueNegatives = n.size();
  output.back().falseNegatives = p.size();
  output.back().precision = 1;
  output.back().recall = 0;

}

double ClassificationPRCurve::maxF(double beta)
{
  SDN_ASSERT(points_.size());
  double best = points_.at(0).fScore(beta);

  for (unsigned i = 1; i < points_.size(); i++)
    {
      if (points_.at(i).fScore(beta) > best)
	{
	  best = points_.at(i).fScore(beta);
	}
    }
  return best;
}

struct xypair
{
  double x;
  double y;

  xypair(double x, double y) : x(x), y(y) {}
  bool operator<(const xypair & other) const { if (x == other.x) return y < other.y; return x < other.x; }
};


double ClassificationPRCurve::area()
{
  double area = 0;

  vector<xypair> coords;
  
  for (unsigned i = 0; i < points_.size(); i++)
    coords.push_back(xypair(points_[i].recall, points_[i].precision));

  sort(coords.begin(),coords.end());

  //Remove points under the curve
  vector<xypair>::iterator iter = coords.begin();

  while (iter != coords.end())
    {

      if ((iter+1) != coords.end() && iter->x == (iter+1)->x)
	{
	  iter = coords.erase(iter);
	}
      else
	{
	  ++iter;
	}
    }



  //Do the actual calculation
  SDN_ASSERT(coords.size() >= 2);
  for (unsigned i = 0; i + 1 < coords.size(); i++)
    {
      double rc = coords[i].x;
      double rn = coords[i+1].x;
      double pc = coords[i].y;
      double pn = coords[i+1].y;


      SDN_ASSERT(rc <= rn);
     
      double width = rn - rc;
      double rectHeight = min(pc,pn);
      double triHeight = max(pc,pn) - rectHeight;
      
      area += width * (rectHeight + 0.5 * triHeight);
    }

  return area;
}

void ClassificationPRCurve::show()
{
  vector<double> x;
  vector<double> y;

  for (unsigned i = 0; i < points_.size(); i++)
    {
      x.push_back(points_.at(i).recall);
      y.push_back(points_.at(i).precision);
    }

  Plotter p;
  p.PlotAndShow(x,y);
}
