/// \author Ian Goodfellow

#pragma once

#include <vector>

namespace sdn
{

class ClassificationPRCurvePoint
{
 public:
  unsigned truePositives;
  unsigned falsePositives;
  unsigned trueNegatives;
  unsigned falseNegatives;
  //A positive is anything >= the threshold
  double threshold;
  double precision;
  double recall;

  double fScore(double beta = 1.0) { return (1.0+beta*beta)*(precision*recall)/(beta*beta*precision+recall); }
};

  /// Calculates PR curves for a 2-class classification problem (as opposed to a detection problem, which is more complicated because the detector may not report any score at all for some true positives)
class ClassificationPRCurve
{

 public:

  ClassificationPRCurve(const std::vector<double> & posScores, const std::vector<double> & negScores);  

  double maxF(double beta = 1.0);
  double area();
  void show();

  unsigned numPoints() { return points_.size(); }
  ClassificationPRCurvePoint getPoint(unsigned i) { return points_[i]; }

 protected:
  void tryThresholds(const std::vector<double> & posScores, const std::vector<double> & negScores, std::vector<ClassificationPRCurvePoint> & output);

 protected:
  //Guaranteed to be sorted by threshold
  std::vector<ClassificationPRCurvePoint> points_;



};



}
