/// \author Andrew Saxe <asaxe@stanford.edu>
// \file endTrainingCallback.h
// \brief A train runtime callback for ending training

#pragma once

#include "trainRuntimeManager.h"
#include <vector>
#include <string>

namespace sdn
{


class EndTrainingCallback : public TrainCallback {
  public:
  EndTrainingCallback() : TrainCallback(0) {
  }

  virtual ~EndTrainingCallback() {}

  void callback(TrainState& state) { 
    SDN_ASSERT(state.iter == nextIter); 
    
    if(state.iter == iter_)
      state.network->endTraining();

    nextIter += iter_;

  }

  void load(XMLNode & node)
  {
    SDN_ASSERT(node.getName());
    SDN_ASSERT(std::string("sdn::TrainCallback")==node.getName());
    SDN_ASSERT(node.getAttribute("id"));
    SDN_ASSERT(std::string("sdn::EndTrainingCallback")==node.getAttribute("id"));
    const char * iterStr = node.getAttribute("iter");
    if (iterStr)
      iter_ = atoi(iterStr);
  }

  protected:
    int iter_;
    
};

}


//SDN_AUTOREGISTER_H(TrainCallback, EndTrainingCallback);
