/// \author Ian Goodfellow

#include "graphicalTransformer.h"
#include "../base/pointer.h"
#include "../base/logger.h"
#include "../base/matrixUtils.h"

using namespace sdn;
using namespace boost;

/**
   Consider pixel (i,j) of the output to span the rectangle with real x coordinates in range [j,j+1) and y coordinates in
   range [i,i+1)
*/



GraphicalTransformer::GraphicalTransformer()
{
	outputRows_ = 1;
	outputCols_ = 1;
	inputRows_ = 1;
	inputCols_ = 1;
	weights_ = ptr(new Matrix(Matrix::Zero(outputRows_ * outputCols_, inputRows_*inputCols_)));
}

GraphicalTransformer::~GraphicalTransformer()
{
	
}

shared_ptr<const Matrix> GraphicalTransformer::getWeights()
{
	return weights_;
}

unsigned GraphicalTransformer::getInputRows() const
{
	return inputRows_;
}

unsigned GraphicalTransformer::getInputCols() const
{
	return inputCols_;
}

void GraphicalTransformer::resize_(unsigned inputRows, unsigned inputCols, unsigned outputRows, unsigned outputCols)
{
	inputRows_ = inputRows;
	inputCols_ = inputCols;
	outputRows_ = outputRows;
	outputCols_ = outputCols;
	weights_->resize(outputRows_ * outputCols_, inputRows_ * inputCols_);
}

Scalar & GraphicalTransformer::getWeight_(unsigned inputRow, unsigned inputCol, unsigned outputRow, unsigned outputCol)
{
  SDN_ASSERT(inputRow < inputRows_);
  SDN_ASSERT(inputCol < inputCols_);
  SDN_ASSERT(outputRow < outputRows_);
  SDN_ASSERT(outputCol < outputCols_);
  return (*weights_)(vectorCoord(outputRow,outputCol,outputRows_,outputCols_),
		     vectorCoord(inputRow,inputCol,inputRows_,inputCols_));
}
