/// \author Ian Goodfellow

#pragma once

#include <boost/shared_ptr.hpp>
#include "../base/matrix.h"

namespace sdn
{
	/** Abstract class for providing linear transformations to a 		GraphicalTransformLayer. */
	class GraphicalTransformer
	{
		public:
			GraphicalTransformer();
			virtual ~GraphicalTransformer();

			///Returns a pointer to the weights that implement the transformation. This pointer is always valid, and the content it points to will update with subsequent calls to the GraphicalTransformer
			boost::shared_ptr<const Matrix> getWeights();

			///Sets the size of the Network the GraphicalTransformer's output will serve as input to
			virtual void setOutputSize(unsigned rows, unsigned cols) = 0;

			///Gets the size of the input the GraphicalTransformer requires
			unsigned getInputRows() const;
			unsigned getInputCols() const;

			///Advances to the next "positive example" (changes the transformation parameters)
			virtual void next() = 0;

		protected:
			void resize_(unsigned inputRows, unsigned inputCols, unsigned outputRows, unsigned outputCols);
			Scalar & getWeight_(unsigned inputRow, unsigned inputCol, unsigned outputRow, unsigned outputCol);
			
		protected:
			boost::shared_ptr<Matrix> weights_;
			unsigned inputRows_;
			unsigned inputCols_;
			unsigned outputRows_;
			unsigned outputCols_;
	};
}
