/// \author Ian Goodfellow
/// \file honglakWrapper.h
/// \brief A C++ wrapper for Honglak Lee's matlab code

#pragma once

#include "network.h"
#include "engine.h"

namespace sdn
{


  class HonglakWrapper : public Network
  {

  public :
    
    HonglakWrapper();
    virtual ~HonglakWrapper();
    
    long long train(long long nIter, bool allowHalting);
    
    bool doneTraining() const;
    
    
    void processExampleAsColumn(const Matrix & x);

    
    void display() { SDN_FATAL("Not implemented"); }
    
    bool isTwoDimensional() { return true; }
    
    void setInputSize(unsigned int rows, unsigned int cols)
    {
      SDN_FATAL("Can't resize that which has already been trained");
    }

    void forwardPropToHighestLayer(const sdn::Matrix &);
    void processExampleToHighestLayer(const sdn::Matrix &);


    boost::shared_ptr<StorageNode> getTestingNode() { return output_; }


  protected:

    bool equals_(const Network & other)
    {
      SDN_FATAL("Honglak networks have no equal");
      return false;
    }

    boost::shared_ptr<StorageNode> output_;
    boost::shared_ptr<Engine> e_;

    mxArray * x_arr_;

  };

}
