#pragma once
#include "boost/shared_ptr.hpp"
#include "sdn/dataset.h"
#include <ext/numeric>         // for iota

#define POSITIVE 1
#define NEGATIVE -1
#define DEFAULT_FIRING_RATE 0.99
#define DEFAULT_NEURON_FRACTION 0.2
namespace sdn{

  /** Functor for ascending indices sorting i.e., sort the indices based on the corresponding 
   *  values.  
   */
  template<typename T>
    struct indirect_less_than {
      T *array;
      indirect_less_than(T *value) : array(value) {}
      bool operator() (const int &a, const int &b) {return array[a] < array[b];}
    };
  
  
  /** Functor for descending indices sorting i.e., sort the indices based on the corresponding 
   *  values.  
   */
  template<typename T>
    struct indirect_greater_than {
      T *array;
      indirect_greater_than(T *value) : array(value) {}
      bool operator() (const int &a, const int &b) {return array[a] > array[b];}
    };
  
  
  /* compute the threshold by quantile and count the global activations */
  void computeThresholdAndGlobalCount(std::vector<Scalar> activations,
				      const Scalar quantile,
				      Scalar &threshold,
				      Scalar &globalCount);

  
  void computeThresholdScore(const Matrix networkLocalCountsPositive, 
			     const Matrix middleCountsPositive,
			     const Matrix networkGlobalCountsPositive,
			     const Matrix networkLocalCountsNegative, 
			     const Matrix middleCountsNegative,
			     const Matrix networkGlobalCountsNegative,
			     Scalar neuronFraction,
			     Scalar &networkScore);
  
  void computeGlobalStatistics(boost::shared_ptr<Network> network,
			       InvarianceTestDataset &itd, 
			       int sign,
			       Scalar quantile,
			       Matrix &thresholds,
			       Matrix &globalCounts);
  void computeLocalStatistics(boost::shared_ptr<Network> network,
			      int inv_type, 
			      int sign,
			      InvarianceTestDataset &itd, 
			      Matrix thresholds, 
			      Matrix &localCounts,
			      Matrix &middleCounts);

  // nips metrics, fast but can be bad
  void firingRateTest(boost::shared_ptr<Network> network,
		      InvarianceTestDataset &itd,
		      Scalar quantile,
		      Scalar neuronFraction,
		      Scalar &networkScoreOut,
		      Scalar &templateScoreOut,
		      Scalar &maxpoolingScoreOut,
		      std::vector<Scalar> &scorePerCategory);
}
