#include <fstream>
#include <vector>
#include <string>

#include "../base/matrix.h"

namespace sdn {
  enum COST_FUNCTION {
    CORRELATION_COST,
    SPARSE_COST,
    LOGCOSH_DIFF_COST
  };

  enum ACTIVATION_FUNCTION {
    SQUARE_SQRT_ACT,
    LOGCOSH_LINEAR_ACT
  };

  enum FORMAT {
    NORMAL,
    LIBSVM
  };

  enum REGULARIZATION {
    NONE,
    L1,
    L2
  };

  enum WEIGHT_MATRIX {
    W_MAT, 
    V_MAT,
    T_MAT
  };
  
  enum DATASET {
    SLOWROT,
    CIFAR,
    CIFAR_LABELS,
    CIFAR_TEST,
    CIFAR_TEST_LABELS,
    NORB,
    NORB_LABELS,
    NORB_TEST,
    NORB_TEST_LABELS
  };

  class InvariantNetwork {
  public:
    InvariantNetwork(int neighborhoodSize, int W_rows);

    void initRandWeights();
    void initWeight(std::ifstream& data, WEIGHT_MATRIX M);
    void readInput(std::ifstream& data, DATASET D);

    void train(Scalar learnRate, int iterations, int batchSize,
	       Scalar tolerance, bool train_w, bool train_v, bool train_t,
	       Scalar reg_v, Scalar reg_w, COST_FUNCTION cost,
	       ACTIVATION_FUNCTION act);
    void trainSoftmax(Scalar learnRate, int iterations, int batchSize,
		      Scalar tolerance, int inputsPerExample, int classes,
		      ACTIVATION_FUNCTION act,REGULARIZATION reg, 
		      Scalar lambda);
    // void mergeRGBFeatures(std::string filePrefix);
    void saveWeights(std::string filePrefix);
    void saveT(std::string filePrefix);
    void saveFeatures(std::string filePrefix, int batchSize, 
		      int inputsPerExample, 
		      ACTIVATION_FUNCTION act_f, 
		      FORMAT f, bool test);
    void setInputsOnCPU(bool onCPU);
   
    Matrix getW();
    Matrix getV();
    Matrix getT();
    
    void testAccuracy(int batchSize, int inputsPerExample,
		      ACTIVATION_FUNCTION act);

  protected:
    Matrix neighborhoodMatrix(int xdim, int ydim, int size);
    Matrix neighborhoodTimes(Matrix m);
    Matrix neighborhoodRows;

    void feedForward();
    void concatOutputs(int inputsPerExample);
    void feedForwardSoftmax();
    Scalar backProp(Scalar oldError);
    Scalar computeCost();
    Scalar computeLL(int batchIndex, int batchSize);
    void computeDerivatives(Matrix& deriv);
    Scalar computeWholeBatchCost(int oldIndex, int batches, int batchSize);
    void performProjections();
    bool hasNegativeV();
    void getA1(int batchSize, int batchIndex);

    int N, Wr, ns, xdim, ydim;
    std::vector<int> labels;
    bool update_v, update_w, update_t;
    bool inputs_on_cpu;
    Scalar alpha, alpha_init, lambdaV, lambdaW;
    Matrix W, V, T;
    Matrix inputs, multilabels;
    Matrix a1, z2, a2, z3, a3, a3t, z4;
    COST_FUNCTION cost_f;
    ACTIVATION_FUNCTION act_f;
    Scalar C;
  };
}
