#include "linearVisualization.h"
#include "../base/matrixUtils.h"
#include <boost/pointer_cast.hpp>
#include "stackedNetwork.h"
#include <sstream>
#include <iostream>
#include "rbm.h"
using namespace std;
using namespace sdn;
using namespace boost;

/// weights[i] is the set of weights for layer i, with different hidden units in rows
static vector<VisImageSet> linearProjection(vector<Matrix> & weights, string weightType, unsigned networkHeight, unsigned networkWidth)
{
  vector<VisImageSet> rval(weights.size());

  for (unsigned i = 0; i < rval.size(); i++)
    {
      stringstream ss;
      ss << "Layer "<<(i+1)<<" Linear Projection "<<weightType;
      rval[i].title = ss.str();
    }

  if (weights.size())
    {
      rval[0].images.resize(weights[0].rows());
      for (int i = 0; i < weights[0].rows(); i++)
	{
	  rval[0].images[i] = vectorToMatrix(weights[0].row(i), networkHeight, networkWidth);
	}
    }

  for (unsigned i = 1; i < rval.size(); i++)
    {
      rval[i].images.resize(weights[i].rows());
      cout << "Layer "<<(i+1)<<" "<<rval[i].images.size()<<endl;
      for (int j = 0; j < weights[i].rows(); j++)
	{
	  rval[i].images[j] = Matrix::Zero(networkHeight,networkWidth);
	  for (int k = 0; k < weights[i].cols(); k++)
	    {
	      rval[i].images[j] += weights[i](j,k) * rval[i-1].images[k];
	    }
	}
    }

  return rval;
}

static vector<VisImageSet> makePairs(vector<VisImageSet> es,
					vector<VisImageSet> ds,
					string visualizationType)
{
  SDN_ASSERT(es.size() == ds.size());

  vector<VisImageSet> rval(es.size());

  for (unsigned i = 0; i < rval.size(); i++)
    {
      stringstream ss;
      ss << "Layer "<<(i+1)<<" "<<visualizationType<<" Encoder/decoder pairs";
      rval[i].title = ss.str();
      rval[i].groupSize = 2;

      SDN_ASSERT_MSG(es[i].images.size() == ds[i].images.size(),i<< " " << es[i].images.size()<<" "<<ds[i].images.size());

      cout << "A" << endl;

      rval[i].images.resize(2*es[i].images.size());

      cout << "B" << endl;

      for (unsigned j = 0; j < es[i].images.size(); j++)
	{
	  SDN_ASSERT(i<rval.size());
	  SDN_ASSERT(2*j+1 < rval.at(i).images.size());
       
	  rval.at(i).images.at(2*j) = es.at(i).images.at(j);
	  rval.at(i).images.at(2*j+1) = ds.at(i).images.at(j);
	}

      cout << "C" << endl;
    }

  return rval;
}

vector<VisImageSet> LinearVisualization::makeVisualizations(boost::shared_ptr<Network> network)
{
  vector<VisImageSet> rval;
  unsigned rows = network->getInputRows();
  unsigned cols = network->getInputCols();

  shared_ptr<StackedNetwork> sn = dynamic_pointer_cast<StackedNetwork>(network);

  if (sn)
    {
      vector<Matrix> e = sn->getEncoderWeights();
      vector<Matrix> d = sn->getDecoderWeights();

      vector<VisImageSet> es = ::linearProjection(e,"Encoder Weights",rows,cols);
      vector<VisImageSet> ds = ::linearProjection(d,"Decoder Weights",rows,cols);

      vector<VisImageSet> pairs = makePairs(es,ds,"Linear Projection");

      for (unsigned i = 0; i < es.size(); i++)
	{
	  rval.push_back(es[i]);
	}

      for (unsigned j = 0; j < ds.size(); j++)
	{
	  rval.push_back(ds[j]);
	}

      for (unsigned k = 0; k < pairs.size(); k++)
	{
	  rval.push_back(pairs[k]);
	}
    }
  else
    {
      shared_ptr<RBM> rbm = dynamic_pointer_cast<RBM>(network);

      vector<Matrix> temp(1,rbm->getWeights())   ;   

      vector<VisImageSet> temp2 (::linearProjection(temp, "RBM weights",rows,cols));
      for (unsigned j = 0; j < temp2.size(); j++)
	rval.push_back(temp2[j]);

      vector<Matrix> temp3(1,rbm->getFastWeights())   ;   

      vector<VisImageSet> temp4 (::linearProjection(temp3, "RBM fast weights",rows,cols));
      for (unsigned j = 0; j < temp4.size(); j++)
	rval.push_back(temp4[j]);

    }



  return rval;
}
