#include <Eigen/Core>
#include <Eigen/Array>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <stdlib.h>
#include "sdn/dataset.h"
#include "maxpoolingTemplateMatching.h"

USING_PART_OF_NAMESPACE_EIGEN // import most common Eigen types 
using namespace std; 
using namespace sdn;

MaxpoolingTemplateMatching::MaxpoolingTemplateMatching(){
}

void MaxpoolingTemplateMatching::setTemplate(const vector<Matrix> t){
  _templates = t;
  _max_norm2 = -999999;
  _template_norm2s.clear();
  for (unsigned int i=0;i<_templates.size();i++){
    Scalar _template_norm2 = (_templates[i].cwise() * _templates[i]).sum();
    _max_norm2 = max(_max_norm2, _template_norm2);
    _template_norm2s.push_back(_template_norm2);
  }  
}

void MaxpoolingTemplateMatching::forwardProp(const Matrix x){
  _value = -9999999;
  assert(_templates[0].rows() == x.rows() && _templates[0].cols() == x.cols());

  for (unsigned int i=0;i<_templates.size();i++){
    //Scalar val = ((_templates[i].cwise() * x).sum()) / _max_norm2;//_template_norm2s[i] ;
    //_value = max(_value, val); // TANH
    Matrix temp = _templates[i] -  x; 
    Matrix dotproduct_temp = temp.cwise() * temp;
    Scalar norm_temp = dotproduct_temp.sum();  
    Scalar val = exp( - norm_temp / _template_norm2s[i] ); // RBF
    _value = max(_value, val);
    
  }    

}

Scalar MaxpoolingTemplateMatching::getActivation(){
  return min(max(_value,Scalar(-1.0)),Scalar(1.0));
}

