/// \author Ian Goodfellow
#include "simpleClassifier.h"
#include "mnistTestCallback.h"
#include <sdn/base.h>
#include <sdn/old.h>
#include <sdn/dataset.h>
#include <fstream>
#include <iostream>
#include <Eigen/Array>
using namespace boost;
using namespace sdn;
using namespace std;

/// Uses the hidden units in the deepest layer of the network
// specified by argv[1] to as features for a logistic regression
// model
// trains on the first NUM_TRAIN_EXAMPLES
// mnist digits (scaled down to 14x14) 
// tests on the next NUM_TEST_EXAMPLES
// reports means squared error between true labels and predicted probabilities
// currently just classifies whether each digit is an "8"


void getExamples(shared_ptr<Network> n, shared_ptr<StorageNode> h,
		 Matrix & X, Vector & y, vector<unsigned> & yv, shared_ptr<Dataset> d,
		 unsigned numExamples)
{
  X.resize(numExamples,h->getActivities().rows());
  y.resize(numExamples);
  yv.resize(numExamples);

  Matrix input = Matrix::Zero(n->getInputRows(),n->getInputCols());

  for (unsigned i = 0; i < numExamples; i++)
    {
      yv[i] = d->getRandomExample(input);
      y(i) = (yv[i] == 8);
      n->processExampleToHighestLayer(input);
      X.row(i) = h->getActivities().transpose();
    }

  cout << "A piece of x: " << X.block(0,0,4,4) << endl;
}


double MnistTestCallback::mnistTest(shared_ptr<Network> n)
{
  shared_ptr<StackedNetwork> sn = dynamic_pointer_cast<StackedNetwork>(n);
  SDN_ASSERT(sn);

  shared_ptr<Dataset> d = DatasetRegistry().make("mnist_test_half");
  SDN_ASSERT(d);


  //Logistic l;

  //l.setPosClassWeight(1.0);

  SimpleClassifier c;

  shared_ptr<StorageNode> h = sn->getStorageNode(sn->numLayers() /2 );
  
  SDN_ASSERT(h);

  #define NUM_TRAIN_EXAMPLES 500
  #define NUM_TEST_EXAMPLES 500

  Matrix Xtr, Xte;
  Vector Ytr, Yte;
  Vector Yhat;
  vector<unsigned> Ytrv, Ytev;


  getExamples(n,h,Xtr,Ytr,Ytrv,d,NUM_TRAIN_EXAMPLES);
  getExamples(n,h,Xte,Yte,Ytev,d,NUM_TRAIN_EXAMPLES);

  for (unsigned i = 0; i < Xtr.cols(); i++)
    {
      double m = matrixMean(Xtr.col(i));
      Xtr.col(i).cwise() -= m;
      Xte.col(i).cwise() -= m;

      double v = sqrt(matrixVariance(Xtr.col(i)));
      if (v > 0)
	{
	  Xtr.col(i) /= v;
	  Xte.col(i) /= v;
	}
      else
	{
	  SDN_ERROR("Column "<<i<<" is constant");
	}
    }


  cout << "A piece of X after whitening: " << Xtr.block(0,0,4,4) << endl;


  Matrix cov(Matrix::Zero(Xtr.cols(),Xtr.cols()));
  cout << "Calculating cov" << endl;
  matrixCovariance(Xtr, cov);
  cout << "Done" << endl;
  cout << "cov: "<< cov.block(0,0,4,4) << endl;


  c.train(Xtr, Ytrv);
  //l.getProbabilitiesFromMatrices(Xte,Yhat);


  SDN_DEBUG("mnistTest about to return");

  return c.error(Xte,Ytev);
}
