///\author Ian Goodfellow
/// Uses the hidden units in the deepest layer of the network
// specified by argv[1] to as features for a logistic regression
// model
// trains on the first NUM_TRAIN_EXAMPLES
// mnist digits (scaled down to 14x14) 
// tests on the next NUM_TEST_EXAMPLES
// reports test set misclassification rate

#pragma once

#include "trainRuntimeManager.h"
#include <vector>
#include "sdn/gui.h"
#include "sdn/base.h"
#include <string>
#include "invarianceTest.h"
#include "sdn/dataset.h"
#include <fstream>

namespace sdn
{
  

  class MnistTestCallback : public TrainCallback {

  public:

    MnistTestCallback() : TrainCallback(0) {}
      
  virtual ~MnistTestCallback() {}

  void callback(TrainState& state) { 


    if(saveFile_.is_open()) {
      double result=  mnistTest(state.network);
      SDN_MSG("mnist test error: "<<result);
      
      saveFile_ << state.iter << "\t" <<result  << std::endl;
      saveFile_.flush();
    }
    else
      {
	SDN_MSG("Skipping because file is not open");
      }
    
    nextIter += step_;
    
  }

  double mnistTest(boost::shared_ptr<Network> n);

  void load(XMLNode & node)
  {
    SDN_ASSERT(node.getName());
    SDN_ASSERT(std::string("sdn::TrainCallback")==node.getName());
    SDN_ASSERT(node.getAttribute("id"));
    SDN_ASSERT(std::string("sdn::MnistTestCallback")==node.getAttribute("id"));
    const char * stepStr = node.getAttribute("step");
    if (stepStr)
      step_ = atoi(stepStr);

    const char * filename = node.getAttribute("filename");
    if (filename)
      {
	saveFile_.open(filename);
	SDN_ASSERT(saveFile_.is_open());
      }
    
  }

  protected:
    int step_;
    std::ofstream saveFile_;
    
  };

}


//SDN_AUTOREGISTER_H(TrainCallback, GratingTestQuantileThresholdCallback);
