/// \author Quoc V. Le
// \file naturalFiringRateTest.h
// \brief A train runtime callback for natural firing rate test 

#pragma once

#include "trainRuntimeManager.h"
#include <vector>
#include "sdn/gui.h"
#include "sdn/base.h"
#include "sdn/base.h"
#include <string>
#include "invarianceTest.h"
#include "sdn/dataset.h"
#include <time.h>

namespace sdn
{
  class NaturalFiringRateTest : public TrainCallback {
  private:
    NaturalTestDataset *itd;
    bool initializeNatural;
    std::vector<Scalar> networkScores;
    std::vector<Scalar> tmScores;
    std::vector<Scalar> mtmScores;
  public:
    // constructor
    NaturalFiringRateTest() : TrainCallback(0) {
      initializeNatural = false;
      firingRate_ = DEFAULT_FIRING_RATE;
      neuronFraction_ = DEFAULT_NEURON_FRACTION;
    }
      
    virtual ~NaturalFiringRateTest() {}
    
    void callback(TrainState& state) { 
      //double begin=clock();
      SDN_DEBUG("Running weights grating test callback");
      if (!initializeNatural){
	itd = new NaturalTestDataset(state.network->getInputCols(), state.network->getInputRows());
	initializeNatural = true;
      }
      assert(state.iter == nextIter); 
      
      Scalar networkScoreOut;
      Scalar templateScoreOut;
      Scalar maxpoolingScoreOut;
      std::vector<Scalar> scorePerCategory;
      firingRateTest(state.network,
		     *itd,
		     firingRate_,
		     neuronFraction_,
		     networkScoreOut,
		     templateScoreOut,
		     maxpoolingScoreOut,
		     scorePerCategory);
      
      networkScores.push_back(networkScoreOut);
      tmScores.push_back(templateScoreOut);
      mtmScores.push_back(maxpoolingScoreOut);
      
      std::cout << "natural test (new measure), " ;
      std::cout << "network scores so far ";
      for (unsigned int i=0;i<networkScores.size();i++){
	std::cout << " " << networkScores[i] ;
      }
      std::cout << std::endl;
      
      if(saveFile_.is_open()) {
	saveFile_ << state.iter << "\t" << networkScores.back() << "\t" << tmScores.back() << "\t" << mtmScores.back();
	for (unsigned int i=0;i<scorePerCategory.size();i++){
	  saveFile_ << "\t" << scorePerCategory[i];
	}
	saveFile_ << std::endl;
	saveFile_.flush();
      }
      
      nextIter += step_;   
    }
    
    void load(XMLNode & node)
    {
      SDN_ASSERT(node.getName());
      SDN_ASSERT(std::string("sdn::TrainCallback")==node.getName());
      SDN_ASSERT(node.getAttribute("id"));
      SDN_ASSERT(std::string("sdn::NaturalFiringRateTest")==node.getAttribute("id"));
      const char * stepStr = node.getAttribute("step");
      if (stepStr)
	step_ = atoi(stepStr);
      const char * filename = node.getAttribute("filename");
      if (filename)
	saveFile_.open(filename);
      const char * inputFiringRate = node.getAttribute("firing_rate");
      if (inputFiringRate)
	firingRate_ = atof(inputFiringRate);
      const char * inputNeuronFraction = node.getAttribute("neuron_fraction");
      if (inputNeuronFraction)
	neuronFraction_ = atof(inputNeuronFraction);
    }

  protected:
    int step_;
    std::ofstream saveFile_;
    float firingRate_;
    float neuronFraction_;
  };
}

