//Author: Andrew Saxe <asaxe@stanford.edu>
/** \file paramsCallback.cc
\brief A run time callback for adding layers to a stacked network
*/

#include "paramsCallback.h"

#include "stackedNetwork.h"

using namespace sdn;
using namespace std;
using namespace boost;

ParamsCallback::ParamsCallback() : TrainCallback(0)
		
{
  iter_ = 1000;
}

ParamsCallback::~ParamsCallback() {}

void ParamsCallback::load(XMLNode & node)
{
  SDN_ASSERT(node.getName());
  SDN_ASSERT(string("sdn::TrainCallback")==node.getName());
  SDN_ASSERT(node.getAttribute("id"));
  SDN_ASSERT(string("sdn::ParamsCallback")==node.getAttribute("id"));

  params_ = node.getChildNode("Params");  

  if(params_.isEmpty()) {
    SDN_WARN("No parameters specified for ParamsCallback."); 
  }
  
  if (node.getAttribute("iter")) {
    iter_ = atoi(node.getAttribute("iter"));
  } else {
    SDN_FATAL("You must specify an iter attribute for the ParamsCallback.");
  }

}

void ParamsCallback::callback(TrainState & state) { 

  if(state.iter == iter_) {

    shared_ptr<StackedNetwork> net = dynamic_pointer_cast<StackedNetwork>(state.network);
    SDN_ASSERT_MSG(net != NULL, "The network must be of type StackedNetwork to use the ParamsCallback.");

    net->setParams(params_);
  }
  
  nextIter += iter_; 
}


