//Author: Ian Goodfellow <ia3n@cs.stanford.edu>
/** \file patchVizCallback.h
\brief A train runtime callback for visualizing patches and
      their reconstructions
*/

#include "patchVizCallback.h"

using namespace sdn;
using namespace std;
using namespace boost;

PatchVizCallback::PatchVizCallback() : TrainCallback(0)
		
{
  _step = 1000;
  _showImage = true;
  _saveFile = false;
  normalize_contrast_ = false;
  _filepath = "";
}

PatchVizCallback::~PatchVizCallback() {}

void PatchVizCallback::load(XMLNode & node)
{
  SDN_ASSERT(node.getName());
  SDN_ASSERT(string("sdn::TrainCallback")==node.getName());
  SDN_ASSERT(node.getAttribute("id"));
  SDN_ASSERT(string("sdn::PatchVizCallback")==node.getAttribute("id"));
  if (node.getAttribute("showImage"))
    {
      _showImage = atoi(node.getAttribute("showImage"));
      
    }
  if (node.getAttribute("saveFile"))
    {
      _saveFile = atoi(node.getAttribute("saveFile"));
    }
  if (node.getAttribute("filepath"))
    {
      _filepath = node.getAttribute("filepath");
    }
  if (node.getAttribute("step"))
    {
      _step = atoi(node.getAttribute("step"));
    }
  if (node.getAttribute("normalize_contrast"))
    {
      normalize_contrast_ = atoi(node.getAttribute("normalize_contrast"));
    }
}

void PatchVizCallback::callback(TrainState & state) { 

  if (_viz == NULL || state.network->getInputRows() != _viz->getPatchHeight() || state.network->getInputCols() != _viz->getPatchWidth())
    {
      _viz = shared_ptr<PatchDisplay>(new PatchDisplay("Reconstructions (input recon pairs)", state.network->getInputRows() , state.network->getInputCols() ,2,6,10));
      /// \todo Find out the true dynamic range of the network
      _viz->setLimits(-1,1);
      if (_showImage)
	_viz->enableWindow();
    }


  nextIter += _step; 

 
  Matrix patch = Matrix::Zero(state.network->getInputRows(),state.network->getInputCols());
  state.dataset->getRandomExample(patch);

  if(normalize_contrast_) {
    //SDN_MSG("Pre normalizing contrast recon: [" << patch.minCoeff() << ", " << patch.maxCoeff() << "] mean: " << patch.sum()/(patch.rows()*patch.cols()));
    if(abs(patch.maxCoeff()-patch.minCoeff()) > 0.01) {
      
      patch = ((patch.cwise()-patch.minCoeff())/(patch.maxCoeff()-patch.minCoeff())*2.0).cwise()-1;
      patch *= 0.8;

      //SDN_MSG("Post normalizing contrast recon: [" << patch.minCoeff() << ", " << patch.maxCoeff() << "] mean: " << patch.sum()/(patch.rows()*patch.cols()));
    }
  }

  _viz->addPatch(patch);

  if (state.network->supportsReconstruction())
    {
      //SDN_MSG("Pre proc ex normalizing contrast recon: [" << patch.minCoeff() << ", " << patch.maxCoeff() << "] mean: " << patch.sum()/(patch.rows()*patch.cols()));
      state.network->processExample(patch);
      //SDN_MSG("Pre get recon normalizing contrast recon: [" << patch.minCoeff() << ", " << patch.maxCoeff() << "] mean: " << patch.sum()/(patch.rows()*patch.cols()));
      state.network->getReconstruction(patch);
      //SDN_MSG("Recon mean n such: " << patch.sum()/(patch.rows()*patch.cols()) << " min,max: " << patch.minCoeff() << ", " << patch.maxCoeff()); 
      _viz->addPatch(patch);

      if (_showImage)
	_viz->wait(25);
    }
  
  if (_saveFile)
    _viz->saveImage(_filepath.c_str());

}


