/// \author Ian Goodfellow <ia3n@cs.stanford.edu>

#pragma once


#include <vector>
#include "../base/matrixUtils.h"
#include "network.h"

namespace sdn
{

  /// An RBM trained with FPCD
  class RBM : public Network
  {

  public:

    RBM();
    virtual ~RBM();

    void showFantasy();

    long long train(long long nIter, bool allowHalting);


    bool isTwoDimensional();

	const Matrix& getHighestLayerActivities();


    virtual void processExampleAsColumn(const Matrix & x)
    {
      fireUp_(x,h_);
      fireDown_(h_,_input);
    }

    virtual void processExampleToHighestLayer(const Matrix & x)
    {
      processExampleAsColumn(x);
    }

    virtual void forwardPropToHighestLayer(const Matrix & x)
    {
      processExampleAsColumn(x);
    }

    virtual void display()
    {
      SDN_FATAL("Not implemented yet");
      
    }

    void serialize(serial::Archive & ar);


    const Matrix & getWeights() const { return weights_; }

    const Matrix & getFastWeights() const { return fastWeights_; }

    bool supportsReconstruction() const { return true; }

    void getReconstruction(Matrix & x)
    {
      x = matrixToVector(x);
      fireUp_(x,h_);
      fireDown_(h_,x);
      x = vectorToMatrix(x,getInputRows(),getInputCols());
    }

    void registerDataset(boost::shared_ptr<Dataset> dataset)
    {
      dataset->setDynamicRange(0,1);
      Network::registerDataset(dataset);
    }


  protected:

    void binarize_(Matrix & m);

    bool equals_(const Network & other)
    {
      SDN_FATAL("not implemented yet");
      return false;
    }

    void setInputSize_(unsigned rows, unsigned cols);
    void initialize_();

    //Assumes h is logistic, does not sample
    void fireUp_(const Matrix & v, Matrix & h, bool useFastWeights = false);
    //Assumes v is linear-gaussian, does not sample
    void fireDown_(const Matrix & h, Matrix & v, bool useFastWeights = false);

  protected:

    unsigned numHidden_;
    unsigned batchSize_;

    Matrix vizBiases_;
    Matrix hidBiases_;
    Matrix weights_;

    Matrix vizBiasGradient_;
    Matrix hidBiasGradient_;
    Matrix weightsGradient_;

    double learningRate_;

    Matrix fastVizBiases_;
    Matrix fastHidBiases_;
    Matrix fastWeights_;
    double fastLearningRate_;

    //note-- not multiplied by learning rate or batch size
    double fastWeightDecay_;

    
    Matrix h_;

    unsigned numFantasy_;
    std::vector<Matrix> vFantasy_;
    std::vector<Matrix> hFantasy_;

    Matrix meanActivation_;
    double activationAveragingTimeConstant_;

    double targetMeanActivation_;
    double sparsityPenalty_;

    double weightDecay_;
  };

}
