#include "reconstructionVisualization.h"
#include "../base/matrixUtils.h"
#include <boost/pointer_cast.hpp>
#include "stackedNetwork.h"
#include <sstream>
#include <iostream>
#include "../base/registry.h"
#include "../base/attributeParser.h"
using namespace std;
using namespace sdn;
using namespace boost;

vector<VisImageSet> ReconstructionVisualization::makeVisualizations(boost::shared_ptr<Network> network)
{
  SDN_ASSERT(network->supportsReconstruction());
  vector<VisImageSet> rval;
  unsigned rows = network->getInputRows();
  unsigned cols = network->getInputCols();

  shared_ptr<StackedNetwork> sn = dynamic_pointer_cast<StackedNetwork>(network);

  VisImageSet vis;
  vis.groupSize = 2;
  vis.title = "Reconstructions";


  dataset_->requestExampleSize(network->getInputRows(),network->getInputCols());

  for (unsigned i = 0; i < numPatches_; i++)
  {
	Matrix patch(Matrix::Zero(rows,cols));
	dataset_->getRandomExample(patch);
	vis.images.push_back(patch);
	network->processExample(patch);
	network->getReconstruction(patch);
	vis.images.push_back(patch);
  }

  rval.push_back(vis);

  return rval;
}


void ReconstructionVisualization::load(XMLNode & node)
{
	SDN_ASSERT(node.getName());
	SDN_ASSERT(string(node.getName())=="sdn::Visualization");
	AttributeParser p;
	p.addString("id");
	p.addInt("numPatches",200);
	p.addString("dataset");

	p.parse(node);

	SDN_ASSERT(p.getString("id").value == "sdn::ReconstructionVisualization");
	numPatches_ = p.getInt("numPatches").value;
	dataset_ = DatasetRegistry().make(p.getString("dataset").value);
}


