/// \author Ian Goodfellow

#include "activationFunction.h"
#include "simpleClassifier.h"
#include "../base/matrixUtils.h"
#include "../base/pointer.h"
#include "../base/stlUtils.h"
using namespace sdn;
using namespace std;
using namespace boost;

SimpleClassifier::SimpleClassifier() : layer_(shared_ptr<ActivationFunction>(new SoftmaxActivationFunction()))
{
  layer_.link(owned_ptr(&input_),owned_ptr(&output_));
}

void SimpleClassifier::train(const Matrix & X, const std::vector<unsigned> & y)
{
  input_.resize(X.cols());
  output_.resize(maxElem(y)+1);
  Vector target(output_.size());
  layer_.resync();

  double alpha = 0.1;

  unsigned i = 0;
  for (i = 0; i < 10000; i++)
    {
      if (i % 100 == 0)
	{
	  SDN_MSG("Iter "<<i);
	  SDN_MSG("Error: "<<error(X,y));
	}

      for (int j = 0; j < X.rows(); j++)
	{
	  int idx = rand() % X.rows();
	  Matrix input = X.row(idx).transpose();
	  dynamic_cast<Layer *>(&layer_)->forwardProp(input);
	  target.setConstant(0);
	  target(y[idx]) = 1.0;
	  layer_.presentTarget(target);
	  layer_.backProp();
	  layer_.gradientDescentUpdate(alpha);
	}

      alpha *= 0.999;
    }
}

double SimpleClassifier::error(const Matrix & X, const vector<unsigned> & y)
{
  unsigned misclassified = 0;
  for (int i = 0; i < X.rows(); i++)
    {
      if (getClass(X.row(i)) != y[i])
	misclassified++;
    }
  return double(misclassified)/double(X.rows());
}

void SimpleClassifier::getProbabilities(const Vector & x,  Vector & y)
{
  dynamic_cast<Layer *>(&layer_)->forwardProp(x);  //TODO-- why is this cast necessary? why isn't StandardLayer inheriting forwardProp(matrix)?
  y = output_.getActivities();
}

unsigned SimpleClassifier::getClass(const Vector & x)
{
  Vector y;
  getProbabilities(x,y);
  return maxIndex(y);
}

void SimpleClassifier::getMatrixProbabilities(const Matrix & X, Matrix & Y)
{
  SDN_ASSERT(X.rows() == Y.rows());

  for (int i = 0; i < X.rows(); i++)
    {
      Vector x(X.row(i)), y(Y.row(i));
      getProbabilities(x,y);
      Y.row(i) = y;
    }
}
  
void SimpleClassifier::getClasses(const Matrix & X, std::vector<unsigned> & y)
{
  y.resize(X.rows());

  for (int i = 0; i < X.rows(); i++)
    {
      y[i] = getClass(X.row(i));
    }
}
