///Author: Andrew Saxe <asaxe@stanford.edu>
/// \file sparsityVisualizationCallback.h
/// \brief A train runtime callback for plotting sparsity

#pragma once

#include <boost/smart_ptr.hpp>
#include <string>

#include "trainRuntimeManager.h"
#include <vector>
#include "sdn/gui.h"
#include "stackedNetwork.h"
#include <fstream>

namespace sdn
{
 
  /// \todo allow it to be displayed in real time not just saved as png
  /// \todo give user control over whether it is displayed or saved

class SparsityVisualizationCallback : public TrainCallback {
  public:
  SparsityVisualizationCallback() : TrainCallback(1) {}

  virtual ~SparsityVisualizationCallback() {}

  void callback(TrainState& state) {
    SDN_DEBUG("Running sparsity visualization callback");
    
 
    SDN_ASSERT(state.iter == nextIter); 
    
    boost::shared_ptr<StackedNetwork> net = boost::dynamic_pointer_cast<StackedNetwork>(state.network);
    SDN_ASSERT_MSG(net, "Network must be of type sdn::StackedNetwork to use sparsityVisualizationCallback.");

    sparsity.push_back(net->getAverageSparsity());
    iterations.push_back(state.iter);

    if(saveFile_.is_open()) {
      saveFile_ << iterations.back() << "\t" << sparsity.back() << std::endl;
      saveFile_.flush();
    }
  

    if(iterations.size() > 1)
      plotter.PlotAndSave(iterations, sparsity, "sparsity_plot");

    nextIter += step_;
    
    SDN_DEBUG("Done running sparsity visualization callback");
  }

  void load(XMLNode & node)
  {
    SDN_ASSERT(node.getName());
    SDN_ASSERT(std::string("sdn::TrainCallback")==node.getName());
    SDN_ASSERT(node.getAttribute("id"));
    SDN_ASSERT(std::string("sdn::SparsityVisualizationCallback")==node.getAttribute("id"));
    const char * stepStr = node.getAttribute("step");
    if (stepStr)
      step_ = atoi(stepStr);

    const char * filename = node.getAttribute("filename");
    if (filename)
      saveFile_.open(filename);
  }

  Plotter plotter;

  std::vector<double> sparsity;
  std::vector<double> iterations;
  
  protected:
  int step_;
  std::ofstream saveFile_;
};

}

//SDN_AUTOREGISTER_H(TrainCallback, SparsityVisualizationCallback);
