// Copyright 2009, All Rights Reserved.
// Authors: Andrew Saxe (asaxe@stanford.edu)
//          Ian Goodfellow (ia3n@cs.stanford.edu)
/** \file storageNode.h
    \brief  Declares the interface of a storage layer that holds inputs
    and outputs.
*/

#pragma once

#include <boost/smart_ptr.hpp>
#include "../base/matrix.h"

namespace sdn {

class ActivationFunction;
namespace serial
{
	class Archive;
}

class StorageNode {
 public:

  StorageNode(int layer_size = 1);
  unsigned size();
  int numExamples();

  void resize(int newSize);


  

//uncomment for debugging hacks
//todo-- undefine NOT_CONST and update denoising to work via an activation function? 
#define NOT_CONST
#ifndef NOT_CONST
  const
#endif
  Matrix& getActivities();
  const Matrix& getDerivatives();
  void setActivities(const Matrix & activities);

  /// Sets the activities to f(x)
  /** (If f(x) is not used elsewhere, it would be inefficient
       to compute it in a different matrix, only to copy it into
       the storage layer immediately afterward) */
  void computeActivities(boost::shared_ptr<ActivationFunction> f,
       const Matrix & x);
 
  void setDerivatives(const Matrix & derivatives);


  void serialize(serial::Archive & ar);
  
  static std::string getClassName() { return "sdn::StorageNode"; }
 
  bool equals(const StorageNode & other);
 
 protected:
  void init(int size);

 protected:
  unsigned layer_size_;
  Matrix activities_;
  Matrix derivatives_;
 

};




}





