/// \author Ian Goodfellow

#include "testCallback.h"
#include "test.h"
#include "../base/registry.h"
#include <fstream>

using namespace sdn;
using namespace boost;
using namespace std;

void TestCallback::callback(TrainState& state) { 
  
  if(saveFile_.is_open()) {
    vector<double> result= test_->run(state.network,state.dataset);

    
    saveFile_ << state.iter;
    for (unsigned i = 0; i < result.size(); i++)
      saveFile_ << "\t" <<result[i]  ;
    saveFile_<< std::endl;
    saveFile_.flush();
  }
  else
    {
      SDN_MSG("Skipping because file is not open");
    }
	
  nextIter += step_;
  
}

void TestCallback::load(XMLNode & node)
{
  SDN_ASSERT(node.getName());
  SDN_ASSERT(std::string("sdn::TrainCallback")==node.getName());
  SDN_ASSERT(node.getAttribute("id"));
  SDN_ASSERT(std::string("sdn::TestCallback")==node.getAttribute("id"));
  const char * stepStr = node.getAttribute("step");
  if (stepStr)
    {
    step_ = atoi(stepStr);
    }
  else
    {
      step_ = 10000;
    }
  
  SDN_DEBUG("Running test every "<<step_<<" iterations");


  const char * filename = node.getAttribute("filename");
  if (filename)
    {
      saveFile_.open(filename);
      SDN_ASSERT(saveFile_.is_open());
    }

  if (node.nChildNode() != 1)
    {
      SDN_FATAL("TestCallback XMLNode should have a single child containing the Test to run");
    }

  XMLNode testNode = node.getChildNode(0);
  test_ = TestRegistry().load(testNode);

  if (! test_)
    {
      SDN_FATAL("TestCallback failed to load a Test to run");
    }
}
