#include "testCallback.h"
#include "mnistTestCallback.h"
#include "architectureUpdateCallback.h"
#include "endTrainingCallback.h"
#include "paramsCallback.h"
#include "patchVizCallback.h"
#include "sparsityVisualizationCallback.h"
#include "trainingErrorVisualizationCallback.h"
#include "trainRuntimeManager.h"
#include "weightsVisualizationCallback.h"
#include "costFunction.h"
#include "saveFileCallback.h"
#include "../base/registry.h"
//#include "naturalTestCallback.h"
//#include "gratingTestCallback.h"
//#include "gratingTestVarianceRatioCallback.h"
//#include "naturalTestVarianceRatioCallback.h"
//#include "gratingTestQuantileThresholdCallback.h"
//#include "naturalTestQuantileThresholdCallback.h"
//#include "naturalTestQuantileThresholdTopCallback.h"
//#include "gratingTestQuantileThresholdTopCallback.h"
//#include "naturalTestQuantileThresholdSignCallback.h"
//#include "gratingTestQuantileThresholdSignCallback.h"
//#include "gratingTestSlownessCallback.h"
//#include "naturalTestSlownessCallback.h"
#include "naturalFiringRateTest.h"
#include "gratingFiringRateTest.h"


using namespace sdn;
using namespace boost;
using namespace std;

namespace sdn
{
  class TrainRuntimeManagerConfigParser : public ConfigParser
  {
  protected:

    TrainRuntimeManager & _mgr;

  public:
    TrainRuntimeManagerConfigParser(TrainRuntimeManager & parent) : _mgr(parent)
    {

    }

    virtual ~TrainRuntimeManagerConfigParser()
    {

    }

    void topics(vector<string> & topics)
    {
      topics.push_back("UseTrainCallbacks");
    }

    void usage(ostream & os) const
    {
      os << "<UseTrainCallbacks> [... saved train callbacks to load ... ] </UseTrainCallbacks>" << endl;
    }

    void readConfig(XMLNode & node)
    {
      const char * name_c = node.getName();
      SDN_ASSERT(name_c);

      string name(name_c);

      SDN_ASSERT(name == "UseTrainCallbacks");

      for (int i = 0; i < node.nChildNode(); i++)
	{
	  
	 
	   
	  XMLNode tempNode = node.getChildNode(i);
	  
	  shared_ptr<TrainCallback> temp = 
	    TrainCallbackRegistry().load(tempNode);
	  if (!temp)
	    {
	      SDN_WARN("<UseTrainCallbacks> tag could not load its "<<i<<"th child.");
	    }
	  else
	    {
	      _mgr.registerCallback(temp);
	    }
	}
    }
  };

}

TrainRuntimeManager::TrainRuntimeManager(boost::shared_ptr<Network> n, boost::shared_ptr<Dataset> d) : _state(n,d), _listener(shared_ptr<ConfigParser>(new TrainRuntimeManagerConfigParser(*this)))
{
  
}


void TrainRuntimeManager::run()
{
  //#define PROFILE
  #ifdef PROFILE
    _state.network->train(100);
    return;
   #endif

  _state.iter = 0; 
  _state.startTime = time(NULL); //todo-- find a better way of getting time

  //Catch anything at time 0
  _runCallbacks();

  bool bTermOK = _callbacksAllowTermination();

  while (true)
  {
	long long nIter = _itersToNextCallback();

        SDN_ASSERT(nIter > 0 || nIter == -1); //we should have handled everything at the current iteration or earlier

	if (nIter == -1)
	{

		SDN_VERBOSE("No callbacks, running network until it terminates");
		SDN_ASSERT(_callbacks.size() == 0);//assumption that callbacks are only triggered by iteration number
		SDN_ASSERT(bTermOK);
		_state.network->train(-1);
		break;
        }
	else
	{
	  
	 
		long long completedIters = _state.network->train(nIter, bTermOK);
		_state.iter += completedIters;		

		if (!bTermOK)
		{
			//if we didn't allow the network to stop early, make sure it did the requested number of iterations
			SDN_ASSERT(completedIters == nIter)
		}
		else
		{
			if (_state.network->doneTraining())
				break;
			else
				SDN_ASSERT(completedIters == nIter); //if the network is not done, it should have done the requested iterations
		}

		
		_runCallbacks();

		///\todo make sure call back can save the network
		/*DenoisingAutoencoder* net = static_cast<DenoisingAutoencoder*>(_state.network.get());
		  std::ofstream ofs("filename");
		  boost::archive::text_oarchive oa(ofs);
		  oa << *net; */
		
		//std::ifstream ifs("filename");
		//boost::archive::text_iarchive ia(ifs);
		// read class state from archive
		//ia >> *net;
		
			
	}
  }
}

void TrainRuntimeManager::_runCallbacks()
{
	for (unsigned i = 0; i < _callbacks.size(); i++)
	{
		if (_callbacks[i]->nextIter == _state.iter)
		{
			_callbacks[i]->callback(_state);
		}	
	}
}

bool TrainRuntimeManager::_callbacksAllowTermination()
{
	for (unsigned i = 0; i < _callbacks.size(); i++)
	{
		if (!(_callbacks[i]->allowTermination()))
			return false;
	}
	return true;
}

long long TrainRuntimeManager::_itersToNextCallback()
{
	if (_callbacks.size())
	{
		long long minCallback = _callbacks[0]->nextIter;

		for (unsigned i = 1; i < _callbacks.size(); i++)
			if (_callbacks[i]->nextIter < minCallback)
				minCallback = _callbacks[i]->nextIter;

		SDN_ASSERT(minCallback > _state.iter);
	
		return minCallback - _state.iter;
	}

	return -1;
}

void TrainRuntimeManager::registerCallback(shared_ptr<TrainCallback> p)
{
	SDN_ASSERT(p);
	_callbacks.push_back(p);
	SDN_DEBUG("Registering callback to start at "<<p->nextIter);
}

TrainCallback::TrainCallback(long long firstIter) : nextIter(firstIter)
{
}

TrainCallback::~TrainCallback()
{
}

bool TrainCallback::allowTermination()
{
	return true;
}

void TrainCallback::load(XMLNode & node)
{

}

Registry<TrainCallback> & sdn::TrainCallbackRegistry()
{
  static Registry<TrainCallback> registry("sdn::TrainCallback");
  return registry;
}

SDN_AUTOREGISTER(TrainCallback, SaveFileCallback);
SDN_AUTOREGISTER(TrainCallback, ArchitectureUpdateCallback);
SDN_AUTOREGISTER(TrainCallback, EndTrainingCallback);
SDN_AUTOREGISTER(TrainCallback, ParamsCallback);
SDN_AUTOREGISTER(TrainCallback, PatchVizCallback);
SDN_AUTOREGISTER(TrainCallback, SparsityVisualizationCallback);
SDN_AUTOREGISTER(TrainCallback, TrainingErrorVisualizationCallback);
SDN_AUTOREGISTER(TrainCallback, WeightsVisualizationCallback);
SDN_AUTOREGISTER(TrainCallback, TestCallback);

SDN_AUTOREGISTER(TrainCallback, NaturalFiringRateTest);
SDN_AUTOREGISTER(TrainCallback, GratingFiringRateTest);
