//Author: Ian Goodfellow
//File: train.h
//Description: Classes used to implement the training runtime environment

#pragma once


#include <boost/smart_ptr.hpp>
#include <string>

#include "network.h"
#include "../dataset/dataset.h"
#include "../base/configManager.h"

namespace sdn
{

/// State of the training runtime exposed to TrainCallback objects
  class TrainState //todo-- use accessors / friendship to make iter and startTime read only for callbacks, read/write for runtime manager
  {
  public:
    TrainState( boost::shared_ptr<sdn::Network> n,
		boost::shared_ptr<sdn::Dataset> i) : network(n), dataset(i){}
      
      
      boost::shared_ptr<sdn::Network> network;
      boost::shared_ptr<sdn::Dataset> dataset;
      long long startTime;
      long long iter;
  };
  
/// Callbacks for the TrainRuntimeManager
  class TrainCallback
  {
  public:
    
    ///for now, callbacks just say when the next iteration they'd like to be called is. later, we might allow them to be called based on other events
    int nextIter; 
    
    
  public:
    /// constructor takes argument for the first iteration on which the callback should run
    TrainCallback(long long firstIter);
    
    virtual ~TrainCallback();

    /// return true if this callback is fine with allowing training to finish (default is to return true)
    virtual bool allowTermination(); 
    
    /// the callback itself. this should set nextIter
    virtual void callback(TrainState & state) = 0;

    /// default implementation is no-op
    virtual void load(XMLNode & node);
  };

 sdn::Registry<TrainCallback> & TrainCallbackRegistry();

  
/// Runtime environment for training Network objects 
  class TrainRuntimeManager
  {
  public:
    
    ///initializes, but does not launch, the training runtime manager
    TrainRuntimeManager(boost::shared_ptr<sdn::Network> n, boost::shared_ptr<sdn::Dataset> d);
    
    ///adds a callback to the runtime manager
    void registerCallback(boost::shared_ptr<TrainCallback> p);
    
    ///launches the runtime environment
    void run();
  protected:
    
    TrainState _state;
    std::vector<boost::shared_ptr<TrainCallback> > _callbacks;
    ConfigListener _listener;

  protected:
    
    ///run all callbacks whose iteration has arrived
    void _runCallbacks(); 
    
    ///returns true iff all callbacks say it is OK to quit
    bool _callbacksAllowTermination(); 
    
    ///returns number of iterations until next callback should run
    long long _itersToNextCallback(); 
  };
}
