//Author: Andrew Saxe <asaxe@stanford.edu>
// \file trainingErrorVisualizationCallback.h
// \brief A train runtime callback for plotting training error

#pragma once

#include "trainRuntimeManager.h"
#include <vector>
#include "sdn/gui.h"
#include "stackedNetwork.h"
#include <fstream>


namespace sdn
{
 
  /// \todo allow it to be displayed in real time not just saved as png
  /// \todo give user control over whether it is displayed or saved

class TrainingErrorVisualizationCallback : public TrainCallback {
  public:
  TrainingErrorVisualizationCallback() : TrainCallback(1) {}

  virtual ~TrainingErrorVisualizationCallback() {}

  void callback(TrainState& state) {
    SDN_DEBUG("Running training error visualization callback");
    SDN_ASSERT(state.iter == nextIter); 
    
    errors.push_back(state.network->getAverageError());
    iterations.push_back(state.iter);

    if(saveFile_.is_open()) {
      saveFile_ << iterations.back() << "\t" << errors.back() << std::endl;
      saveFile_.flush();
    }
    
    if(iterations.size() > 1)
      plotter.PlotAndSave(iterations, errors, "training_error_plot");
    
    nextIter += step_;
    
    SDN_DEBUG("Done running training error visualization callback");
  }
  
  void load(XMLNode & node)
  {
    SDN_ASSERT(node.getName());
    SDN_ASSERT(std::string("sdn::TrainCallback")==node.getName());
    SDN_ASSERT(node.getAttribute("id"));
    SDN_ASSERT(std::string("sdn::TrainingErrorVisualizationCallback")==node.getAttribute("id"));
    const char * stepStr = node.getAttribute("step");
    if (stepStr)
      step_ = atoi(stepStr);

    const char * filename = node.getAttribute("filename");
    if (filename)
      saveFile_.open(filename);

  }

  Plotter plotter;

  std::vector<double> errors;
  std::vector<double> iterations;
  
  protected:
  int step_;
  std::ofstream saveFile_;
};

}

//SDN_AUTOREGISTER_H(TrainCallback, TrainingErrorVisualizationCallback);
