/// \author Ian Goodfellow

#pragma once

#include "graphicalTransformer.h"
#include "../base/random.h"

namespace sdn
{
	/** A GraphicalTransformer that performs translation in one direction */
	class TranslationTransformer : public GraphicalTransformer
	{
		public:
			TranslationTransformer();
			virtual ~TranslationTransformer();

			///Sets the direction of positive translation (direction viewer will translate) in radians relative to x axis (with positive y being up in image space, not up in row number)
			void setDirection(double theta);

			///Sets the range of offsets randomly selected from each time next() is called (uniform distribution)
			void setOffsetRange(double minOffset, double maxOffset);

			void next();

			void setOffset(double offset);
			void setOutputSize(unsigned rows, unsigned cols);

		protected:
			void recalculateSize_();
			void sampleOffset_();
			void recalculateWeights_();
			void calculateRangeAndOrigin_(unsigned outputSize, double stepSize, double & origin, unsigned samplingRate, unsigned & inputSize);
			void recalculateWeight_(unsigned i, unsigned j);//assumes weights have been zeroed

		protected:
		
			double offset_;
			double minOffset_;
			double maxOffset_;
			double dirX_;
			double dirY_;
/** Amount that should be added to x coordinates in the input space to make sure upper left of input is 0. Will always be an integer, but saved as a double to reduce number of casts. */
			double xOrigin_;
			double yOrigin_;

			/* Antialiasing parameters */
			unsigned xSamples_;
			unsigned ySamples_;

			// what's a pirate's favorite variable?
			RandomRealGenerator<double> rrg_;
	};
}
