/// \author Ian Goodfellow
/// file \weightVisualization.h
#pragma once

#include "../base/matrix.h"
#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>

class XMLNode;

namespace sdn
{
  
///Contains a set of weight images
  struct VisImageSet
  {
    /// The title of the set-- e.g., "Layer 1 Encoding Weights" or "Layer 2 Encoder/Decoder Pairs"
    std::string title;
    /// A separate image visualizing the function computed by each hidden unit
    std::vector<Matrix> images;

    /// Each groupSize consecutive images should be shown together (i.e., use groupSize 2 for encoder/decoder pairs)
    int groupSize;

    bool showUnnormalized;
    bool showNormalized;

    VisImageSet() : groupSize(1), showUnnormalized(true), showNormalized(true)
    {
    }
  };


  class Network;

  ///An abstract base class for computing weight visualizations
  class Visualization
  {
    
  public:
    
    virtual std::vector<VisImageSet> makeVisualizations(boost::shared_ptr<Network> network) = 0;
    

    virtual void load(XMLNode & node) {}
    
  };
  
  template <typename T> class Registry;
  Registry<Visualization> & VisualizationRegistry();

}
