/// \author Andrew Saxe <asaxe@stanford.edu>
/// \author Ian Goodfellow <ia3n@stanford.edu> (ConfigManager integration)
// \file trainingErrorVisualizationCallback.h
// \brief A train runtime callback for plotting training error

#pragma once

#include "trainRuntimeManager.h"
#include <vector>
#include <string>

namespace sdn
{


class WeightsVisualizationCallback : public TrainCallback {
  public:
  WeightsVisualizationCallback() : TrainCallback(0) {
  }

  virtual ~WeightsVisualizationCallback() {}

  void callback(TrainState& state) { 
    SDN_DEBUG("Running weights visualization callback");

    assert(state.iter == nextIter); 
    
    state.network->display();

    nextIter += step_;

  }

  void load(XMLNode & node)
  {
    SDN_ASSERT(node.getName());
    SDN_ASSERT(std::string("sdn::TrainCallback")==node.getName());
    SDN_ASSERT(node.getAttribute("id"));
    SDN_ASSERT(std::string("sdn::WeightsVisualizationCallback")==node.getAttribute("id"));
    const char * stepStr = node.getAttribute("step");
    if (stepStr)
      step_ = atoi(stepStr);
  }

  protected:
    int step_;
    
};

}


//SDN_AUTOREGISTER_H(TrainCallback, WeightsVisualizationCallback);
