//Author: Daniel Chia <danchia@stanford.edu>
//File: crossValidationBlock.h
//Description: implements cross validation on a dataset for a supervised network

#pragma once

#include "sdn/base.h"
#include "sdn/stable.h"
#include "../../../dataset/basicImageCollection.h"
#include "boost/shared_ptr.hpp"

namespace sdn {

	class CrossValidationBlock: public AbstractBlock {
	
		public:

			virtual void loadXML(XMLNode& node);
			
			virtual void initTransientParameters();
			virtual void initPersistentParameters();
			
			virtual void serialize(serial::Archive &ar);

			void evaluateNetwork(boost::shared_ptr<AbstractSupervisedNetwork> network);
			double getError();

			CrossValidationBlock();
			~CrossValidationBlock();

			// Persistent Parameters

			// Configurable Parameters
			std::string datasetName;
			int inputHeight;
			int inputWidth;
			bool normalizeInput;

			// Transient Parameters
			boost::shared_ptr<BasicImageCollection> dataset;
			sdn::Matrix inputExample;
			sdn::Matrix prediction;
			int numCorrect;
			NormalizeInputBlock normInput;

	};
}
