//Author: Daniel Chia <danchia@stanford.edu>
//File: danchiaConvertNetworkInScript.cc
//Description: 

#include "danchiaConvertNetworkInScript.h"
#include "sdn/base.h"
#include "sdn/dataset.h"
#include "sdn/registry.h"

using namespace std;

namespace sdn {

  class DanchiaConvertNetworkInScriptConfigParser : public ConfigParser {

  protected:
    DanchiaConvertNetworkInScript & parentScript;

  public:
  
    DanchiaConvertNetworkInScriptConfigParser(DanchiaConvertNetworkInScript & parent) : parentScript(parent) {}
    ~DanchiaConvertNetworkInScriptConfigParser() {}
    
    void topics(std::vector<std::string> & topics) {
      // Only Have ONE topic
      // If not, loadXML in your Script will be called multiple times
      // This might be a concern
      topics.push_back("sdn::DanchiaConvertNetworkInScript");
    }

    void readConfig(XMLNode & node) {
      parentScript.loadXML(node);
    }
    
    void usage(std::ostream & os) const {
      os << "<sdn::DanchiaConvertNetworkInScript ...... script=\"<Scalar>\" network=\"<Scalar>\" \" />\n";
    }
  };
  
  // -------------------------------------------------------------------

  DanchiaConvertNetworkInScript::DanchiaConvertNetworkInScript() {
    loadUsingConfigManager();
  }
  
  DanchiaConvertNetworkInScript::~DanchiaConvertNetworkInScript() {
  }
  
  void DanchiaConvertNetworkInScript::loadXML(XMLNode& node) {
    AttributeParser p;
    
    p.addString("networkFile");
		p.addString("saveDir");
    
    p.parse(node);
    
    // Set Values
    networkFile = p.getString("networkFile");
 		saveDir = p.getString("saveDir"); 

 		// Load network


		std::ifstream fin;

		fin.open(networkFile.c_str(), std::ios::binary);
		SDN_ASSERT_MSG(fin.good(), "Unable to open " << networkFile);

		sdn::serial::BinInArchive ar(fin);
		boost::shared_ptr<AbstractNetwork> net = sdn::AbstractNetworkRegistry().load(ar);
		fin.close();

		SDN_ASSERT_MSG(net, "Unable to load network from file");

		network = boost::dynamic_pointer_cast<StandardAutoEncoderNetwork> (net);

		SDN_ASSERT_MSG(net, net->className() << "is not a StandardAutoEncoderNetwork network");
		SDN_VERBOSE("Loaded " << networkFile << " as network");

    
  }

  void DanchiaConvertNetworkInScript::loadUsingConfigManager() {

    // The following line will load the XML config
    ConfigListener listener
      (boost::shared_ptr<ConfigParser>
        (new DanchiaConvertNetworkInScriptConfigParser(*this)));

    // We have this here instead of loadXML because loadXML might get
    // called multiple times by the configParser
    initPersistentParameters();
    initTransientParameters();
  }
  
  void DanchiaConvertNetworkInScript::initTransientParameters() {
  }
  
  void DanchiaConvertNetworkInScript::initPersistentParameters() {
   }
  
  void DanchiaConvertNetworkInScript::serialize(serial::Archive & ar) {
    SDN_FATAL(className() << "::" << __FUNCTION__ << " not implemented");
  }

  
  void DanchiaConvertNetworkInScript::executeOneIteration() {
    
    SDN_ASSERT(manager);

		ofstream of;

/*		// output network config

		of.open("networkConfig.dat");

		of << network->targetActivation << endl;
		of << network->activationAveragingConstant << endl;
		of << network->learingRateSparsity << endl;
		of << network->hiddenLayerSize << endl;

		// affine encoder

		of << network->affineEncoder.learningRate << endl;
		of << network->affineEncoder.weightDecay << endl;
		of << network->affineEncoder.outputSize << endl;
		of << network->affineEncoder.inputSize << endl;

		// affine decoder

		of << network->affineDecoder.learningRate << endl;
		of << network->affineDecoder.weightDecay << endl;
		of << network->affineDecoder.outputSize << endl;
		of << network->affineDecoder.inputSize << endl;

		of.close(); */

		// affine encoder weights

		string temp = saveDir + "encoder.dat";
		of.open(temp.c_str());
		of << network->affineEncoder.weights;
		of << network->affineEncoder.bias;
		of.close();

		// affine decoder weights

		temp = saveDir + "decoder.dat";
		of.open(temp.c_str());
		of << network->affineDecoder.weights;
		of << network->affineDecoder.bias;
		of.close();

  }
  

}
