//Author: Daniel Chia
//File: danchiaConvertNetworkOutScript.cc
//Description: 

#include "danchiaConvertNetworkOutScript.h"
#include "sdn/base.h"
#include "sdn/dataset.h"
#include "sdn/registry.h"

namespace sdn {

  class DanchiaConvertNetworkOutScriptConfigParser : public ConfigParser {

  protected:
    DanchiaConvertNetworkOutScript & parentScript;

  public:
  
    DanchiaConvertNetworkOutScriptConfigParser(DanchiaConvertNetworkOutScript & parent) : parentScript(parent) {}
    ~DanchiaConvertNetworkOutScriptConfigParser() {}
    
    void topics(std::vector<std::string> & topics) {
      // Only Have ONE topic
      // If not, loadXML in your Script will be called multiple times
      // This might be a concern
      topics.push_back("sdn::DanchiaConvertNetworkOutScript");
    }

    void readConfig(XMLNode& node) {
      parentScript.loadXML(node);
    }
    
    void usage(std::ostream & os) const {
      os << "<sdn::DanchiaConvertNetworkOutScript ...... script=\"<Scalar>\" network=\"<Scalar>\" \" />\n";
    }
  };
  
  // -------------------------------------------------------------------

  DanchiaConvertNetworkOutScript::DanchiaConvertNetworkOutScript() {
    loadUsingConfigManager();
  }
  
  DanchiaConvertNetworkOutScript::~DanchiaConvertNetworkOutScript() {
  }
  
  void DanchiaConvertNetworkOutScript::loadXML(XMLNode& node) {
    
    AttributeParser p;
    
    p.addString("saveDir");
   	p.addString("network"); 
    p.parse(node);
    
    // Note that values are set *directly* into the network
		saveDir = p.getString("saveDir");
		networkName = p.getString("network");
    
    // We need to create the network here because the XML parameters
    // for it need to be read directly to the network    
    network = boost::dynamic_pointer_cast<StandardAutoEncoderNetwork> (AbstractNetworkRegistry().make(networkName));
    if (!network) {
      SDN_ERROR("Could not load network with name " << networkName);
      SDN_ERROR("Is " << networkName << " a StandardAutoEncoderNetwork?");
      ERROR_OUT_LIST_REGISTRY_NAMES(AbstractNetwork);
      SDN_FATAL("Quitting");
    }

    // Deep Loading
    XMLNode networkNode = node.getChildNode("NetworkParameters");
    SDN_ASSERT_MSG(!networkNode.isEmpty(), "You need to specify NetworkParameters !")
    network->loadXML(networkNode); 

  }

  void DanchiaConvertNetworkOutScript::loadUsingConfigManager() {

    // The following line will load the XML config
    ConfigListener listener
      (boost::shared_ptr<ConfigParser>
        (new DanchiaConvertNetworkOutScriptConfigParser(*this)));

    // We have this here instead of loadXML because loadXML might get
    // called multiple times by the configParser
    initPersistentParameters();
    initTransientParameters();

  }
  
  void DanchiaConvertNetworkOutScript::initTransientParameters() {


  }
  
  void DanchiaConvertNetworkOutScript::initPersistentParameters() {
    network->initTransientParameters();
    network->initPersistentParameters();
  }
  
  void DanchiaConvertNetworkOutScript::serialize(serial::Archive & ar) {
    SDN_FATAL("DanchiaConvertNetworkOutScript is not serializable yet");
  }

  
  void DanchiaConvertNetworkOutScript::executeOneIteration() {
    
    SDN_ASSERT(manager);
    
		boost::shared_ptr<StandardAutoEncoderNetwork> net = boost::dynamic_pointer_cast<StandardAutoEncoderNetwork> (network);

		std::ifstream inFile;

		// encoder

		inFile.open(quickString("%sencoder.dat", saveDir.c_str()).c_str());
		readMatrix(inFile, net->affineEncoder.weights);
		readMatrix(inFile, net->affineEncoder.bias);	
		inFile.close();

		//decoder

		inFile.open(quickString("%sdecoder.dat", saveDir.c_str()).c_str());
		readMatrix(inFile, net->affineDecoder.weights);
		readMatrix(inFile, net->affineDecoder.bias);
		inFile.close();

		net->quickSaveBinary(saveDir + "network.dat");
  }

	void DanchiaConvertNetworkOutScript::readMatrix(std::ifstream &stream, Matrix &m)
	{
		for (unsigned int i = 0; i < m.rows(); i++)
			for (unsigned int j = 0; j < m.cols(); j++)
				stream >> m(i,j);
	}
}
