//Author: Daniel Chia <danchia@stanford.edu>
//File: danchiaSupervisedScript.h
//Description: 

#pragma once

#include "boost/shared_ptr.hpp"
#include "sdn/stable.h"

namespace sdn {

  // Forward Declarations
  class Dataset;

  /// DanchiaSupervisedScript class
  /** 
  */
  class DanchiaSupervisedScript : public AbstractScript,
                                          public ProgressReportCallbackInterface,
                                          public ReconstructionErrorCallbackInterface,
                                          public SupervisedErrorCallbackInterface,
                                          public SaveModelCallbackInterface {

  public:
  

    virtual void loadXML(XMLNode& node);

    virtual void initTransientParameters(); 
    virtual void initPersistentParameters();

    virtual void serialize(serial::Archive & ar);

    virtual void loadUsingConfigManager();

    virtual void executeOneIteration();
    
    DanchiaSupervisedScript();
    ~DanchiaSupervisedScript();

    // Configurable       
    std::string encodingNetworkFile;
    std::string supervisedNetworkName;
    std::string datasetName;
    int inputHeight;
    int inputWidth;
    int unsupervisedTrainingIterations;
    bool fineTuneUnsupervisedNetwork;

    // Persistent Parameters
    boost::shared_ptr<AbstractAutoEncoderNetwork> encodingNetwork;
    boost::shared_ptr<AbstractSupervisedNetwork> supervisedNetwork;
    
    // Transient
    boost::shared_ptr<Dataset> dataset;
    sdn::Matrix inputExample;
    sdn::Matrix targetOutput;
    
    // Fixed Parameters
    NormalizeInputBlock normInput;
    
    // Callback Interfaces --------------------------------------------
    virtual void reportProgress(std::ostream& o);

    virtual const boost::shared_ptr<CostFunction> getSupervisedCostFunction() ;
    virtual const sdn::Matrix& getTarget() ;
    virtual const sdn::Matrix& getPrediction() ;

    virtual const boost::shared_ptr<CostFunction> getReconstructionCostFunction() ;
    virtual const sdn::Matrix& getReconstruction() ;
    virtual const sdn::Matrix& getOriginal() ;

    // ---- Save Model Callback ----
    virtual const void saveModelInDirectory(std::string dir);
  };
  
  
}
