//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: jngiamHelpers.cc
//Description: Helpers

#include "jngiamHelpers.h"

namespace sdn {

  Matrix ComputeColwiseWeightedKurtBlock::computeColwiseWeightedKurt(const Matrix& m, const Matrix& w) {
    
    
    /* Compute Kurtosis for each Column *
     * Naive Sample Kurtosis */
    
    /* Assume that each column is an input example */
    
    /* Each column has w.rows() rows, we use this as a weight/probability dist */
    if (unitX.cols() != m.cols())
      unitX  = Matrix::Zero(1, m.cols()); 
    if (unitX2.cols() != m.cols())
      unitX2 = Matrix::Zero(1, unitX.cols());
    if (unitX4.cols() != m.cols())
      unitX4 = Matrix::Zero(1, unitX.cols());
    
    if (wNorm.rows() != w.rows())
      wNorm = Matrix::Zero(w.rows(), 1);
    
    // Can optimize this
    /*
    for ( unsigned i =0 ;i <w.rows() ; i++) {
      wNorm(i,0) = (w.row(i) * w.row(i).transpose())[0];
    }*/
    wNorm = (w.cwise() * w).rowwise().sum();
        
    // Replace with rowwise or colwise?
    for (unsigned j=0;j<m.rows();j++) {
      unitX += wNorm(j,0) * m.row(j);
    }
    
    unitX *= 1.0/(wNorm.sum());
    for (unsigned j=0;j<m.rows();j++) {
      Matrix diffSquared = (m.row(j) - unitX.row(0)).cwise().square();
      unitX2 += wNorm(j,0)*diffSquared;
      unitX4 += wNorm(j,0)*diffSquared.cwise().square();
    }
    
    unitX2 *= 1.0/(wNorm.sum());
    unitX4 *= 1.0/(wNorm.sum());
    
    kurt = (unitX4.cwise() / (unitX2.cwise() * unitX2)).cwise() - 3.0; // normalized kurtosis
    // kurt = unitX4 - (3*(var.cwise() * var)); // non normalized kurtosis, definition from Hyvarinen
    // kurt = unitX4; // just the 4th order
    
    return kurt;
    
  }

  Matrix computeColwiseKurtosis(const Matrix& m) {
    
    Matrix ones, mean, cm2, vcm2, cm4, vcm4, kurt;
    
    /* Compute Kurtosis for each Column *
     * Naive Sample Kurtosis */
    
    if (ones.rows() != m.rows())
      ones = Matrix::Ones(m.rows(), 1);
    
    // Mean
    mean =  ones * (1.0/double(m.rows()) * m.colwise().sum());

    // 2nd Central Moment
    vcm2 = (m - mean).cwise().square();
    cm2 = (1.0/double(m.rows())) * vcm2.colwise().sum();
    
    // 4th Central Moment
    vcm4 = vcm2.cwise().square();
    cm4 = (1.0/double(m.rows())) * vcm4.colwise().sum();

    // Normalized Kurtosis
    kurt = (cm4.cwise() / (cm2.cwise() * cm2)).cwise() - 3.0;
    
    return kurt;
  }


}
