//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: testAffineBlock.h
//Description: 

#pragma once

#include "sdn/stable.h"
#include <Eigen/Array>

namespace sdn {

  /// TestAffineBlock class
  /** Expects input to be a matrix of column vector only
   *  each column vector is one example
  */
  class TestAffineBlock : public ComposableBlock {
  
  public:
  
    virtual void setConfigurableParameters(sdn::Scalar defLearningRate,
                                           sdn::Scalar defWeightDecay,
                                           int defInputSize,
                                           int defOutputSize);
    virtual void loadXML(XMLNode& node);
    
    virtual void initTransientParameters() ;
    virtual void initPersistentParameters();
    
    virtual void serialize(serial::Archive & ar);
    
    TestAffineBlock();
    ~TestAffineBlock();

    // Persistent Parameters
    sdn::Matrix weights;                // Each row is a feature detector
    sdn::Matrix bias;
    
    // Configurable Parameters
    sdn::Scalar learningRate;
    sdn::Scalar weightDecay;
    int outputSize;
    int inputSize;
    bool lockBias;
    bool shareBias;
    sdn::Scalar fixedWeightNorm;
    
    // Transient Parameters
    sdn::Matrix weightsGradient;
    sdn::Matrix biasGradient;
    sdn::Matrix weightNorms;
    sdn::Matrix ones; 
    
    // Operations
    virtual void forwardProp(const sdn::Matrix& input,
                             sdn::Matrix& output);
   
    virtual void backProp(const sdn::Matrix& input,
                          const sdn::Matrix& outputErrorDerivatives,
                          sdn::Matrix& inputErrorDerivatives);
    
    virtual void gradientUpdate();

    // Additional Update
    virtual void forceGradientUpdateBias(sdn::Scalar lr);
    
  };
  
}
