//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: JngiamAutoEncoderNetwork.h
//Description: Uses AutoEncoderBlocks to create a Network

#pragma once

#include "sdn/base.h"
#include "sdn/stable.h"
#include "../blocks/jngiamHelpers.h"
#include "../blocks/testAffineBlock.h"

namespace sdn {

  /// JngiamAutoEncoderNetwork class
  /** 
  */
  class JngiamAutoEncoderNetwork : public AbstractAutoEncoderNetwork {
  public:

    virtual void setConfigurableParameters(sdn::Scalar defTargetActivation,
                                           sdn::Scalar defLearningRateSparsity,
                                           unsigned int defHiddenLayerSize,
                                           sdn::Scalar  defAveragingConstant);
    virtual void loadXML(XMLNode& node) ;
    
    virtual void initTransientParameters();
    virtual void initPersistentParameters();
    
    virtual void serialize(serial::Archive & ar) ;
    
    virtual void unsupervisedTrain(const sdn::Matrix& example);
    virtual const boost::shared_ptr<CostFunction> getCostFunction();
    
    virtual const sdn::Matrix& encode(const sdn::Matrix& input);
    virtual const sdn::Matrix& decode(const sdn::Matrix& hidden);

    virtual void encodeBackProp(const sdn::Matrix& input,
              const sdn::Matrix& hiddenErrorDerivatives,
        sdn::Matrix& inputErrorDerivatives);

    virtual void decodeBackProp(const sdn::Matrix& hidden,
              const sdn::Matrix& outputErrorDerivatives,
        sdn::Matrix& hiddenErrorDerivatives);

    virtual void encodeGradientUpdate();
    virtual void decodeGradientUpdate();
    
    virtual const sdn::Matrix& getHiddenRepresentation();
    virtual const sdn::Matrix& getReconstruction();
    
    virtual void setupGoodInitialParameters();
    
    JngiamAutoEncoderNetwork();
    ~JngiamAutoEncoderNetwork();

    // Persistent Parameters
    TestAffineBlock affineEncoder, affineDecoder;
    TanhBlock   sigmEncoder, sigmDecoder;
    sdn::Scalar outputScale;
    
    // Configurable Parameters
    sdn::Scalar  targetActivation;
    sdn::Scalar  initialBias;
    sdn::Scalar  activationAveragingConstant; 
    sdn::Scalar  kurtAveragingConstant;
    sdn::Scalar  learningRateSparsity; 
    sdn::Scalar  learningRateKurtosis; 
    sdn::Scalar  biasShift;
    sdn::Scalar  acceptableReconstructionError;
    bool         optimizeUnitKurtosis;
    bool         noDecTanh;
    sdn::Scalar  normalizeOutputToL2Norm;
    unsigned int hiddenLayerSize;
    
    bool         learnOutputScaling;
    sdn::Scalar  outputScaleGradientLearningRate;
    // Transient Parameters
    sdn::Scalar outputScaleGradient;

    ExponentiallyWeightedMeanBlock activationTracker;
    ExponentiallyWeightedMeanBlock normalizedKurtTracker;
    ExponentiallyWeightedMeanBlock unnormalizedKurtTracker;
    ExponentiallyWeightedMeanBlock errorTracker;
    
    // ComputeColwiseWeightedKurtBlock computeKurt;
    sdn::Matrix hiddenRepresentation, affineRepEnc, affineRepDec;
    sdn::Matrix inputGradient, hiddenGradient, outputGradient;
    sdn::Matrix reconstruction;
    
    // Kurtosis Helpers, Leaving it here to aid the optimization
    sdn::Matrix ones, mean, cm2, vcm2, cm4, vcm4, vcm3, cm3, vcm1, cm1, kurt, dervK1, dervK2, dervK;
    sdn::Matrix ones2, preBias, weightsDerv;

    // Fixed Parameters, could technically load this 
    boost::shared_ptr<CostFunction> costFunc;
    
  };

}
  
