//Author: Jiquan <jngiam@cs.stanford.edu>
//File: autoTuneScript.h
//Description: 

#pragma once

#include "boost/shared_ptr.hpp"
#include "sdn/stable.h"
#include "../networks/autoTuneNetwork.h"

namespace sdn {

  // Forward Declarations
  class Dataset;

  /// AutoTuneScript class
  /** 
  */
  class AutoTuneScript : public AbstractScript, 
                         public ProgressReportCallbackInterface,
                         public SaveModelCallbackInterface,
                         public VizWeightsCallbackInterface {

  public:
  

    virtual void loadXML(XMLNode& node);

    virtual void initTransientParameters(); 
    virtual void initPersistentParameters();

    virtual void serialize(serial::Archive & ar);

    virtual void loadUsingConfigManager();

    virtual void executeOneIteration();
    
    AutoTuneScript();
    ~AutoTuneScript();

    // Configurable       
    std::string useWhiteningMatrixFile;
    std::string useMeanMatrixFile;
    
    std::string trainDatasetName;
    std::string validateDatasetName;
    int inputHeight;
    int inputWidth;
    int nExamples;
    
    int hyperTuneStep;

    // Configurable-Persistent Parameters
    AutoTuneNetwork network;
    
    // Allow to train over an existing autoencoder network
    boost::shared_ptr<AbstractAutoEncoderNetwork> autoEncNetwork;
    
    // Transient
    boost::shared_ptr<Dataset> validateDataset;
    boost::shared_ptr<Dataset> trainDataset;
    sdn::Matrix inputExample;
    sdn::Matrix targetOutput;
    sdn::Matrix oneInputExample, whiteInputExample;
    sdn::Matrix encodedExample;
    sdn::Matrix tempVizMatrix;
    sdn::Matrix whiteMat, meanMat, invWhite, temp;
    
    
    // Fixed Parameters
    NormalizeInputBlock normInput;
    
    // ---- ProgressReportCallback ----
    virtual void reportProgress(std::ostream& o) ;
    
    // ---- VisualizationCallback ----
    virtual const sdn::Matrix& getEncoderWeights();
    virtual const int getWeightPatchHeight() ;
    virtual const sdn::Matrix& getDecoderWeights() ;

    
    // ---- SaveModelCallbackCallbackInterface ----
    virtual const void saveModelInDirectory(std::string dir);
  
  };
  
  
}
