//Author: Jiquan <jngiam@cs.stanford.edu>
//File: jngiamScript.h
//Description: 

#pragma once

#include "boost/shared_ptr.hpp"
#include "sdn/stable.h"
#include "../networks/jngiamAutoEncoderNetwork.h"

namespace sdn {

  // Forward Declarations
  class Dataset;

  /// JngiamScript class
  /** 
  */
  class JngiamScript : public AbstractScript, 
                       public ReconstructionErrorCallbackInterface,
                       public ProgressReportCallbackInterface,
                       public SaveModelCallbackInterface,
                       public VizWeightsCallbackInterface {

  public:
  

    virtual void loadXML(XMLNode& node);

    virtual void initTransientParameters(); 
    virtual void initPersistentParameters();

    virtual void serialize(serial::Archive & ar);

    virtual void loadUsingConfigManager();

    virtual void executeOneIteration();
    
    JngiamScript();
    ~JngiamScript();

    // Configurable       
    std::string networkName;
    std::string datasetName;
    int inputHeight;
    int inputWidth;
    int nExamples;
    bool useL2Normalization;
    bool noScaleNormalization;

    // Configurable-Persistent Parameters
    boost::shared_ptr<JngiamAutoEncoderNetwork> network;
    
    // Allow to train over an existing autoencoder network
    boost::shared_ptr<AbstractAutoEncoderNetwork> autoEncNetwork;
    
    // Transient
    boost::shared_ptr<Dataset> dataset;
    sdn::Matrix inputExample;
    sdn::Matrix oneInputExample;
    sdn::Matrix encodedExample;
    sdn::Matrix tempVizMatrix;
    
    // Fixed Parameters
    NormalizeInputBlock normInput;
    
    // ---- Reconstruction Callback ----
    virtual const boost::shared_ptr<CostFunction> getReconstructionCostFunction() ;
    virtual const sdn::Matrix& getReconstruction() ;
    virtual const sdn::Matrix& getOriginal() ;
    
    
    // ---- ProgressReportCallback ----
    virtual void reportProgress(std::ostream& o) ;
    
    // ---- VisualizationCallback ----
    virtual const sdn::Matrix& getEncoderWeights();
    virtual const int getWeightPatchHeight() ;
    virtual const sdn::Matrix& getDecoderWeights() ;

    
    // ---- SaveModelCallbackCallbackInterface ----
    virtual const void saveModelInDirectory(std::string dir);
  
  };
  
  
}
