//Author: NAME <name@stanford.edu>
//File: jngiamScript1.cc
//Description: 

#include "jngiamScript1.h"
#include "sdn/base.h"
#include "sdn/dataset.h"
#include "sdn/registry.h"

namespace sdn {

  class JngiamScript1ConfigParser : public ConfigParser {

  protected:
    JngiamScript1 & parentScript;

  public:
  
    JngiamScript1ConfigParser(JngiamScript1 & parent) : parentScript(parent) {}
    ~JngiamScript1ConfigParser() {}
    
    void topics(std::vector<std::string> & topics) {
      // Only Have ONE topic
      // If not, loadXML in your Script will be called multiple times
      // This might be a concern
      topics.push_back("sdn::JngiamScript1");
    }

    void readConfig(XMLNode& node) {
      parentScript.loadXML(node);
    }
    
    void usage(std::ostream & os) const {
      os << "<sdn::JngiamScript1 ...... dataset=\"<string>\"  />\n";
    }
  };
  
  // -------------------------------------------------------------------

  JngiamScript1::JngiamScript1() {
    loadUsingConfigManager();
  }
  
  JngiamScript1::~JngiamScript1() {
  }
  
  void JngiamScript1::loadXML(XMLNode& node) {
    
    AttributeParser p;
    
    p.addString("dataset");
    p.addInt("inputHeight");
    p.addInt("inputWidth");
    
    p.parse(node);
    
    // Note that values are set *directly* into the network
    datasetName = p.getString("dataset");
    inputHeight = p.getInt("inputHeight");
    inputWidth  = p.getInt("inputWidth");

  }

  void JngiamScript1::loadUsingConfigManager() {

    // The following line will load the XML config
    ConfigListener listener
      (boost::shared_ptr<ConfigParser>
        (new JngiamScript1ConfigParser(*this)));

    // We have this here instead of loadXML because loadXML might get
    // called multiple times by the configParser
    initPersistentParameters();
    initTransientParameters();

  }
  
  void JngiamScript1::initTransientParameters() {

    dataset =  DatasetRegistry().make(datasetName);
    if (!dataset) {
      SDN_ERROR("Could not load dataset with name " << datasetName);
      ERROR_OUT_LIST_REGISTRY_NAMES(Dataset);
      SDN_FATAL("Quitting");
    }
    dataset->requestExampleSize(inputHeight, inputWidth);

    // We are requesting column vector examples
    inputExample = sdn::Matrix::Zero(inputHeight*inputWidth, 1);

    sumInputAbs = sdn::Matrix::Zero(inputHeight*inputWidth, 1);
    sumInputL2 = sdn::Matrix::Zero(inputHeight*inputWidth, 1);
    sumNoScale = sdn::Matrix::Zero(inputHeight*inputWidth, 1);
    
    l2AveErrorTracker.setConfigurableParameters(0.001,
                                                TRACKER_AUTOSET,
                                                SIZE_AUTOSET);
    absAveErrorTracker.setConfigurableParameters(0.001,
                                                TRACKER_AUTOSET,
                                                SIZE_AUTOSET);
                                                
    zeroErrorTracker.setConfigurableParameters(0.001,
                                               TRACKER_AUTOSET,
                                               SIZE_AUTOSET);
    noScaleErrorTracker.setConfigurableParameters(0.001,
                                               TRACKER_AUTOSET,
                                               SIZE_AUTOSET);

    pxsum = 0;
    absSum = l2Sum = seen = 0.0;
    outlier = 0;
    mmax = 0;
    mmin = 0;
  }
  
  void JngiamScript1::initPersistentParameters() {
  }
  
  void JngiamScript1::serialize(serial::Archive & ar) {
    SDN_FATAL("JngiamScript1 is not serializable yet");
  }

  
  void JngiamScript1::executeOneIteration() {
    
    SDN_ASSERT(manager);
    
    int label = dataset->getRandomExampleAsColumn(inputExample);
    
    // do fancy stuff here
    // normInput.normalizePatch(inputExample);
    
    Matrix patch = inputExample;
    double mean = patch.sum() / (patch.rows() * patch.cols());
    patch.cwise() -= mean;

    double maxAbs = (patch.maxCoeff() > fabs(patch.minCoeff()) ? patch.maxCoeff() : fabs(patch.minCoeff()));
    
    double absMult = 0.8 / (maxAbs+ 1e-6);
    double l2NormMult = 1.0 / sqrt(patch.cwise().square().sum());

    seen++;
    
    mmax = MAX(mmax, patch.maxCoeff());
    mmin = MIN(mmin, patch.minCoeff());
    
    /*
    std::cout << "patch max: " << patch.maxCoeff() << " min " << patch.minCoeff() << "\n";
    std::cout << "all   max: " << mmax << " min " << mmin << "\n";

    std::cout << "abs\t" << absMult << "\t"
              << "l2n\t" << l2NormMult << "\t"
              << "ratio\t" << absMult / l2NormMult << "\n";
    
    Matrix patch1 = patch * l2NormMult;
    maxAbs = (patch1.maxCoeff() > fabs(patch1.minCoeff()) ? patch1.maxCoeff() : fabs(patch1.minCoeff()));
    std::cout << "max after l2: " << maxAbs << "\n";
    
    Matrix patch2 = patch * absMult;
    std::cout << "l2 after abs: " << sqrt(patch.cwise().square().sum()) << "\n";
    */
    if (absMult > 20) {
      outlier++;
    } else {
      absSum += absMult;
      l2Sum += l2NormMult;
      std::cout << "abs ave: " << absSum/seen << "  l2 ave " << l2Sum/seen  << " ratio:  " << absSum/l2Sum<<  " outliertario " << outlier/seen <<"\n";
    }
    
    
    sdn::Matrix absNormalizedImage = absMult   *patch;
    sdn::Matrix  l2NormalizedImage = l2NormMult*patch;
    
    sumInputAbs += absNormalizedImage;
    sumInputL2  += l2NormalizedImage;
    sumNoScale  += patch;
    
    // Compute average image
    sdn::Matrix aveAbs = 1.0/seen * sumInputAbs;
    sdn::Matrix aveL2  = 1.0/seen * sumInputL2;
    sdn::Matrix aveNS  = 1.0/seen * sumNoScale;
    pxsum += l2NormalizedImage.cwise().abs().sum()/l2NormalizedImage.size();
    
    // Compute error and add to moving average
    absAveErrorTracker.addValues(costFunc.output(absNormalizedImage, aveAbs));
    l2AveErrorTracker.addValues(costFunc.output(l2NormalizedImage, aveL2));
    noScaleErrorTracker.addValues(costFunc.output(patch, aveNS));
    zeroErrorTracker.addValues(costFunc.output(l2NormalizedImage, Matrix::Zero(l2NormalizedImage.rows(),l2NormalizedImage.cols())));
    
    std::cout << "absAveErrorTracker: " << absAveErrorTracker.getMean() << " l2AveErrorTracker " << l2AveErrorTracker.getMean() << " ";
    std::cout << " zero-L2-errorTracker: " << zeroErrorTracker.getMean() 
              << " noScaleErrorTracker: "  << noScaleErrorTracker.getMean()
              << " pxave " << pxsum/seen << "\n";

    // std::cout <<  "this error: "<< costFunc.output(l2NormalizedImage, l2NormalizedImage) << "\n";
    
  }
 
}


















