//Author: Daniel Chia, Zhenghao Chen, Pang Wei Koh 
//File: invarianceTestBlock.h
//Description: Port of old/invarianceTest.cc to work in redesign

#pragma once

#include "sdn/base.h"
#include "../../../stable/defaultParameters.h"
#include "../../../stable/abstractBlock.h"
#include "../../../stable/abstractNetwork.h"
#include "../../../stable/networks/standardAutoEncoderNetwork.h"
#include "sdn/dataset.h"
#include "../../../gui/patchDisplay.h"
#include <string>


#define POSITIVE 1
#define NEGATIVE -1
#define DEFAULT_FIRING_RATE 0.99
#define DEFAULT_NEURON_FRACTION 0.2

namespace sdn {

  class InvarianceTestBlock : public AbstractBlock {

  public:
  
    virtual void loadXML(XMLNode& node) ;
    virtual void initTransientParameters() {};
    virtual void initPersistentParameters() {};
    virtual void serialize(serial::Archive & ar) ;
    virtual void invarianceTest(boost::shared_ptr<AbstractAutoEncoderNetwork> networkInput, int patch_Width, int patch_Height);

	 void initializeGradient(int patch_Width, int patch_Height, int inv_type);
	 void computeGradient(Matrix& weightsGradient, Matrix& biasGradient);

    InvarianceTestBlock() {};
    virtual ~InvarianceTestBlock() {};

    /*
     Takes a network object and natural test dataset, returns the invariance score of the network
     */

    // ---- Specific functions for invariance calculations ----

    /** Functor for ascending indices sorting i.e., sort the indices based on the corresponding 
     *  values.  
     */

    template<typename T>
    	struct indirect_less_than {
    		T *array;
    		indirect_less_than(T *value) : array(value) {}
    		bool operator() (const int &a, const int &b) {return array[a] < array[b];}
    	};


    /** Functor for descending indices sorting i.e., sort the indices based on the corresponding 
     *  values.  
     */
    template<typename T>
    	struct indirect_greater_than {
    		T *array;
    		indirect_greater_than(T *value) : array(value) {}
    		bool operator() (const int &a, const int &b) {return array[a] > array[b];}
    	};


    /* compute the threshold by quantile and count the global activations */
    void computeThresholdAndGlobalCount(std::vector<Scalar> activations,
    		Scalar &threshold,
    		Scalar &globalCount);

    void computeThresholdScore(const Matrix networkLocalCountsPositive, 
    		const Matrix middleCountsPositive,
    		const Matrix networkGlobalCountsPositive,
    		const Matrix networkLocalCountsNegative, 
    		const Matrix middleCountsNegative,
    		const Matrix networkGlobalCountsNegative,
    		Scalar &networkScore);

    void computeGlobalStatistics(
    		int sign,
    		Matrix &thresholds,
    		Matrix &globalCounts);

    void computeLocalStatistics(
    		int inv_type, 
    		int sign,
    		Matrix thresholds, 
    		Matrix &localCounts,
    		Matrix &middleCounts);

    // nips metrics, fast but can be bad
    void firingRateTest();

    // ----------------------------


    boost::shared_ptr<AbstractAutoEncoderNetwork> network; 
	 boost::shared_ptr<StandardAutoEncoderNetwork> invNetwork;
    NaturalTestDataset* itd;

    Scalar quantile;
    Scalar neuronFraction;

    int patchWidth;
    int patchHeight;
	 int nHiddenRows;
	 int nHiddenCols;
	 int patchSize;
	 int gradientInvType;

	 std::vector<std::vector<int> > trajectoryIndices;
	 int trajectoryFrameCount;

	 Matrix middlePositive;

    Scalar networkScoreOut;
    Scalar templateScoreOut;
    Scalar maxpoolingScoreOut;
    std::vector<Scalar> scorePerCategory;
    bool displayTrajectory;
    std::string trajectorySavePath;

  };
}
