//Author: Chen Zhenghao Koh Pang Wei Daniel Chia
//File: InvarianceNetwork.cc
//Description: Network designed to produce invariance to test invariance test basically only need encode

#include "../../../stable/blocks/sharedBlocks.h"
#include "invarianceNetwork.h"
#include "sdn/base.h"
#include <Eigen/Array>
#include <iostream>

using namespace std;

namespace sdn {
  
  const sdn::Matrix& InvarianceNetwork::encode(const sdn::Matrix& input) {
    
    affineEncoder.forwardProp(input, affineRepEnc);
    sigmEncoder.forwardProp(affineRepEnc, hiddenRepresentation);
    //squares activations
    squareEncoder.forwardProp(hiddenRepresentation, squaredHiddenRepresentation);
    affineEncoder2.forwardProp(squaredHiddenRepresentation, affineRepEnc2);
    sigmEncoder2.forwardProp(affineRepEnc2,finalActivation);
    
    
    return finalActivation;
  }

  const sdn::Matrix& InvarianceNetwork::decode(const sdn::Matrix& hidden) {
    
    SDN_ASSERT(affineEncoder.inputSize > 0);
    
    cout << "decode has not been implemented for this network yet" << endl;
    return hidden;

  }

  void InvarianceNetwork::encodeBackProp(const sdn::Matrix& input,
            const sdn::Matrix& hiddenErrorDerivatives,
            sdn::Matrix& inputErrorDerivatives) {
    sdn::Matrix tempGradient;
    sdn::Matrix tempGradient2;
    sigmEncoder2.backProp(affineRepEnc2, hiddenErrorDerivatives, tempGradient);
    affineEncoder2.backProp(squaredHiddenRepresentation, tempGradient, tempGradient2);
    squareEncoder.backProp(hiddenRepresentation,tempGradient2, tempGradient);
    sigmEncoder.backProp(affineRepEnc, tempGradient, tempGradient2);
    affineEncoder.backProp(input, tempGradient2, inputErrorDerivatives);
  }

  void InvarianceNetwork::decodeBackProp(const sdn::Matrix& hidden,
            const sdn::Matrix& outputErrorDerivatives,
            sdn::Matrix& hiddenErrorDerivatives) {
    sdn::Matrix tempGradient;
    cout << "decode is not implemented yet" << endl;
 
  }
  /*
  void InvarianceNetwork::encodeGradientUpdate() {
    affineEncoder.gradientUpdate();
  }

  void InvarianceNetwork::decodeGradientUpdate() {
    affineDecoder.gradientUpdate();
  }

  const sdn::Matrix& InvarianceNetwork::getHiddenRepresentation() {
    return hiddenRepresentation;
  }
  
  const sdn::Matrix& InvarianceNetwork::getReconstruction() {
    return reconstruction;
  } 
  
  void InvarianceNetwork::setupGoodInitialParameters() {
    
    if (affineEncoder.inputSize > 0) {
      RandomizeWeightMatrix().randomize(affineEncoder.weights, affineEncoder.inputSize);
    }
      
    if (hiddenLayerSize > 0) {
      RandomizeWeightMatrix().randomize(affineDecoder.weights, hiddenLayerSize);
      affineEncoder.bias = sigmEncoder.getBiasInitialParameters(targetActivation, hiddenLayerSize);
    }
    
  }
  
  // --------------------------------------------------------------------------

  void InvarianceNetwork::setConfigurableParameters(sdn::Scalar defTargetActivation,
                                                             sdn::Scalar defLearningRateSparsity,
                                                             unsigned int defHiddenLayerSize,
                                                             sdn::Scalar defAveragingConstant) {

    // Shallow Loading

    targetActivation = defTargetActivation;
    hiddenLayerSize = defHiddenLayerSize;
    learningRateSparsity = defLearningRateSparsity;
    activationAveragingConstant = defAveragingConstant;
    
    
    initPersistentParameters();
    initTransientParameters();
  }
  */
  void InvarianceNetwork::loadXML(XMLNode& node) {
/*
    AttributeParser p;
    
    //p.addDouble("targetActivation");
    //p.addDouble("activationAveragingConstant");
    //p.addDouble("learningRateSparsity");
    p.addInt("hiddenLayerSize");
    p.addInt("hiddenLayer2Size");
    
    p.parse(node);
    
    //targetActivation = p.getDouble("targetActivation");
    //activationAveragingConstant = p.getDouble("activationAveragingConstant");
    //learningRateSparsity = p.getDouble("learningRateSparsity");
    hiddenLayerSize = p.getInt("hiddenLayerSize");
    hiddenLayer2Size = p.getInt("hiddenLayer2Size");
    // Deep Loading
    //XMLNode affineEncoderNode = node.getChildNode("Encoder");
    ///if (!affineEncoderNode.isEmpty())
    //  affineEncoder.loadXML(affineEncoderNode);

    //XMLNode affineDecoderNode = node.getChildNode("Decoder");
    //if (!affineDecoderNode.isEmpty())
    //  affineDecoder.loadXML(affineDecoderNode);
*/
    initPersistentParameters();
    initTransientParameters();

  }
  
  void InvarianceNetwork::initTransientParameters() {
    //if (hiddenLayerSize > 0) {
    
      hiddenLayerSize = affineEncoder.weights.cols();
      hiddenLayer2Size = affineEncoder2.weights.cols();

      hiddenRepresentation = sdn::Matrix::Zero(hiddenLayerSize, 1);
      hiddenGradient = sdn::Matrix::Zero(hiddenLayerSize, 1);
      affineRepEnc =  sdn::Matrix::Zero(hiddenLayerSize, 1);
      affineRepEnc2 = sdn::Matrix::Zero(hiddenLayer2Size, 1);
      squaredHiddenRepresentation = sdn::Matrix::Zero(hiddenLayerSize,1);
    //}

    //activationTracker.setConfigurableParameters(activationAveragingConstant,
    //                                            targetActivation,
    //                                            SIZE_AUTOSET);
    
  }
  
  void InvarianceNetwork::initPersistentParameters() {
    
    affineEncoder.initPersistentParameters();
    //affineDecoder.initPersistentParameters();
    
    sigmEncoder.initPersistentParameters();
    //sigmDecoder.initPersistentParameters();

    squareEncoder.initPersistentParameters();
    affineEncoder2.initPersistentParameters();
    sigmEncoder2.initPersistentParameters();
    

    //setupGoodInitialParameters();

  }

  const boost::shared_ptr<CostFunction> InvarianceNetwork::getCostFunction() {
    return costFunc;
  }

  void InvarianceNetwork::serialize(serial::Archive & ar)  {
    SDN_DEBUG(className() << "::" << __FUNCTION__);
    int version = 1000;

    version = ar.beginBase(this,version,className());

    switch(version)
      {
      case 1000:
      
        // Persistent
	cout << "checking the size of affine****" << endl;
	cout << "affine 1 is " << affineEncoder.weights.rows() << " by " << affineEncoder.weights.cols() << endl;
        affineEncoder.serialize(ar);
        //affineDecoder.serialize(ar);
        sigmEncoder.serialize(ar);
        //sigmDecoder.serialize(ar);
	squareEncoder.serialize(ar);
	cout << "affine 2 is " << affineEncoder2.weights.rows() << " by " << affineEncoder2.weights.cols() << endl;
	affineEncoder2.serialize(ar);
	sigmEncoder2.serialize(ar);
        
        // Configurable
        ar.attribute("hiddenLayerSize", hiddenLayerSize);
	ar.attribute("hiddenLayer2Size", hiddenLayer2Size);

    
        break;
        
      default:
        SDN_FATAL("Unrecognized " << className() << " version");
      }
    ar.endBase();
      
    // Input Case: Initialize Transient
    if (ar.in()) {
      initTransientParameters();
    }
        
  }


  InvarianceNetwork::InvarianceNetwork() {
    costFunc = boost::shared_ptr<CostFunction> (new SumOfSquaresCostFunction());
  }
  
  InvarianceNetwork::~InvarianceNetwork() {
  }

  
}
