
//Author: Chen Zhenghao, Koh Pang Wei, Daniel Chia
//File: invarianceNetwork.h
//Description: 

#pragma once

#include "sdn/base.h"
#include "../../../stable/abstractNetwork.h"
#include "../../../stable/blocks/sharedBlocks.h"
#include "../../../stable/blocks/standardBlocks.h"
#include "../../../stable/defaultParameters.h"
#include "../../../stable/math/costFunction.h"
#include "../../../stable/math/normPenalty.h"
#include <iostream>
using namespace std;

namespace sdn {

  /// StandardAutoEncoderNetwork class
  /** 
  */
  class InvarianceNetwork : public AbstractAutoEncoderNetwork {
  public:

//    virtual void setConfigurableParameters(sdn::Scalar defTargetActivation,
//                                           sdn::Scalar defLearningRateSparsity,
//                                           unsigned int defHiddenLayerSize,
//                                           sdn::Scalar  defAveragingConstant);
    virtual void loadXML(XMLNode& node) ;
    
    virtual void initTransientParameters();
    virtual void initPersistentParameters();
    
    virtual void serialize(serial::Archive & ar) ;
    
    virtual void unsupervisedTrain(const sdn::Matrix& example){};
    virtual const boost::shared_ptr<CostFunction> getCostFunction();
    
    virtual const sdn::Matrix& encode(const sdn::Matrix& input);
    virtual const sdn::Matrix& decode(const sdn::Matrix& hidden);

    virtual void encodeBackProp(const sdn::Matrix& input,
              const sdn::Matrix& hiddenErrorDerivatives,
        sdn::Matrix& inputErrorDerivatives);

    virtual void decodeBackProp(const sdn::Matrix& hidden,
              const sdn::Matrix& outputErrorDerivatives,
        sdn::Matrix& hiddenErrorDerivatives);

    virtual void encodeGradientUpdate(){};
    virtual void decodeGradientUpdate(){};
    
    virtual const sdn::Matrix& getHiddenRepresentation(){};
    virtual const sdn::Matrix& getReconstruction(){};
    
//    virtual void setupGoodInitialParameters();
    
    InvarianceNetwork();
    ~InvarianceNetwork();

    // Persistent Parameters
    AffineBlock affineEncoder, affineDecoder, affineEncoder2;
    TanhBlock   sigmEncoder, sigmDecoder, sigmEncoder2;
    SquareBlock squareEncoder;
    
    // Configurable Parameters
    sdn::Scalar  targetActivation;
    sdn::Scalar  activationAveragingConstant; 
    sdn::Scalar  learningRateSparsity; 
    unsigned int hiddenLayerSize;
    unsigned int hiddenLayer2Size;
    

    virtual void loadAffineBlock1(const sdn::Matrix& x){
	affineEncoder.inputSize = x.cols();
	affineEncoder.outputSize = x.rows();
	affineEncoder.weights = x;
	Matrix b (6545,1);
	b.Zero(6545,1);
	affineEncoder.bias = b;
	cout << "in load affine block size of x is " << affineEncoder.weights.rows() << " " << affineEncoder.weights.cols() << endl;

    }
    virtual void loadAffineBlock2(const sdn::Matrix& x){
	affineEncoder2.inputSize = x.cols();
	affineEncoder2.outputSize = x.rows();
	affineEncoder2.weights = x;
	Matrix b (595,1);
	b.Zero(595,1);
	affineEncoder.bias = b;
	cout << "in load affine block size of x is " << affineEncoder2.weights.rows() << " " << affineEncoder2.weights.cols() << endl;

    }

    // Transient Parameters
    ExponentiallyWeightedMeanBlock activationTracker;
    sdn::Matrix hiddenRepresentation, affineRepEnc, affineRepDec, affineRepEnc2, squaredHiddenRepresentation, finalActivation;
    sdn::Matrix inputGradient, hiddenGradient, outputGradient;
    sdn::Matrix reconstruction;
    
    // Fixed Parameters, could technically load this 
    boost::shared_ptr<CostFunction> costFunc;
    
  };

}
  
