//Author: Daniel Chia, Zhenghao Chen, Pang Wei Koh
//File: invarianceCallbacks.cc
//Description: Implements invariance metric

#include <ext/numeric>
#include "invarianceCallbacks.h"
#include "../../../../stable/math/costFunction.h"
#include "../../../../stable/helpers.h"
#include "../../../../stable/scriptRuntimeManager.h"
#include "../../../../registry/scriptRegistry.h"
#include "sdn/base.h"
#include <iostream>

using namespace std;
namespace sdn {

	void InvarianceCallback::loadXML(XMLNode & node) {

		AttributeParser p;
		cout << "in here" << endl;
		p.addInt("reportStep");
		p.addInt("firstIter", 0);
		p.addInt("patchWidth");
		p.addInt("patchHeight");
		
		p.parse(node);
		
		reportStep = p.getInt("reportStep");
		runFirstIter = p.getInt("firstIter");
		patchWidth = p.getInt("patchWidth");
		patchHeight = p.getInt("patchHeight");

		XMLNode invarianceTestNode = node.getChildNode("InvarianceTestBlock");
    		SDN_ASSERT_MSG(!invarianceTestNode.isEmpty(), "You need to specify <InvarianceTestBlock> !")
		invTestBlock.loadXML (invarianceTestNode);
		

	}

	long int InvarianceCallback::itersToNextCall() {
		return reportStep;
	}

	long int InvarianceCallback::firstIter() {
		return runFirstIter;
	}

	void InvarianceCallback::execute(ScriptRuntimeManager& scriptManager,
			AbstractScript& script,
			unsigned int iteration) {

		SDN_ASSERT(rScript = dynamic_cast<InvarianceCallbackInterface*> (&script));

		initTransientParameters();
		if (iteration % reportStep == 0) {
			invTestBlock.invarianceTest(network,patchWidth, patchHeight);
		}
	}

	void InvarianceCallback::initTransientParameters() {
		network = rScript->getNetwork();
		//patchWidth = rScript->patchWidth;
		//patchHeight = rScript->patchHeight;
	}

	void InvarianceCallback::initPersistentParameters() {
	}

	void InvarianceCallback::serialize(serial::Archive & ar) {
		SDN_FATAL(className() << " not serializable yet");
	}

}
