//Author: Daniel Chia, Zhenghao Chen, Pang Wei Koh 
//File: invarianceCallbacks.h
//Description: Port of old/invarianceTest.cc to work in redesign

#pragma once

#include <ext/numeric>         // for iota
#include "../../../../stable/blocks/sharedBlocks.h"
#include "../../blocks/invarianceTestBlock.h"
#include "../../../../stable/abstractScript.h"
#include "sdn/dataset.h"
#include <boost/shared_ptr.hpp>

#define POSITIVE 1
#define NEGATIVE -1
//#define DEFAULT_FIRING_RATE 0.99
//#define DEFAULT_NEURON_FRACTION 0.2

namespace sdn {


	/// InvarianceCallback interface and class
	/** To use this callback, have your script implement this interface
	 */

	class InvarianceCallbackInterface {
		public:
			virtual const boost::shared_ptr<sdn::AbstractAutoEncoderNetwork> getNetwork() = 0 ;
			
			int patchHeight;
			int patchWidth;

			virtual ~InvarianceCallbackInterface() {}
	};

	class InvarianceCallback : public AbstractScriptCallback {

		public:

			virtual void loadXML(XMLNode& node) ;

			virtual void initTransientParameters() ;
			virtual void initPersistentParameters() ;

			virtual void serialize(serial::Archive & ar) ;

			virtual void execute(ScriptRuntimeManager& scriptManager,
					AbstractScript& script,
					unsigned int iteration) ;

			virtual long int itersToNextCall();
			virtual long int firstIter();

			InvarianceCallback() {}
			virtual ~InvarianceCallback() {}

			InvarianceTestBlock invTestBlock;

			// Configurable
			long int reportStep;
			long int runFirstIter;			
			
			int patchWidth;
			int patchHeight;

			// Transient
			boost::shared_ptr<sdn::AbstractAutoEncoderNetwork> network; 
			InvarianceCallbackInterface* rScript;

	};

}
