//Author: Daniel Chia
//File: danchiaScript.h
//Description: 

#pragma once

#include "boost/shared_ptr.hpp"
#include "sdn/stable.h"

namespace sdn {

  // Forward Declarations
  class Dataset;

  /// DanchiaScript class
  /** 
  */
  class DanchiaScript : public AbstractScript, 
                        public ReconstructionErrorCallbackInterface,
											 	public ProgressReportCallbackInterface,
                       	public SaveModelCallbackInterface,
                       	public VizWeightsCallbackInterface  {

  public:
  

    virtual void loadXML(XMLNode& node);

    virtual void initTransientParameters(); 
    virtual void initPersistentParameters();

    virtual void serialize(serial::Archive & ar);

    virtual void loadUsingConfigManager();

    virtual void executeOneIteration();
    
    DanchiaScript();
    ~DanchiaScript();

    // Configurable       
    std::string networkName;
    std::string datasetName;
    int inputHeight;
    int inputWidth;

    // Configurable-Persistent Parameters
    boost::shared_ptr<StandardAutoEncoderNetwork> network;
    
    // Transient
    boost::shared_ptr<Dataset> dataset;
    sdn::Matrix inputExample;
    
    // Fixed Parameters
    NormalizeInputBlock normInput;
    
    // ---- Reconstruction Callback ----
    virtual const boost::shared_ptr<CostFunction> getReconstructionCostFunction() ;
    virtual const sdn::Matrix& getReconstruction() ;
    virtual const sdn::Matrix& getOriginal() ;

     // ---- ProgressReportCallback ----
    virtual void reportProgress(std::ostream& o) ;
    
    // ---- VisualizationCallback ----
    virtual const sdn::Matrix& getEncoderWeights();
    virtual const int getWeightPatchHeight() ;
    virtual const sdn::Matrix& getDecoderWeights() ;

    
    // ---- SaveModelCallbackCallbackInterface ----
    virtual const void saveModelInDirectory(std::string dir);
 
  };
  
  
}
