//Author: Daniel Chia, Zhenghao Chen, Pang Wei Koh
//File: invarianceScript.cc
//Description: Blank script; provides interface for invariance callback

#include "invarianceScript.h"
#include "sdn/base.h"
#include "sdn/dataset.h"
#include "sdn/registry.h"
#include <iostream>
using namespace std;

namespace sdn {

  class InvarianceScriptConfigParser : public ConfigParser {

  protected:
    InvarianceScript & parentScript;

  public:
  
    InvarianceScriptConfigParser(InvarianceScript & parent) : parentScript(parent) {}
    ~InvarianceScriptConfigParser() {}
    
    void topics(std::vector<std::string> & topics) {
      // Only Have ONE topic
      // If not, loadXML in your Script will be called multiple times
      // This might be a concern
      topics.push_back("sdn::InvarianceScript");
    }

    void readConfig(XMLNode& node) {
      parentScript.loadXML(node);
    }
    
    void usage(std::ostream & os) const {
      os << "<sdn::InvarianceScript ...... script=\"<Scalar>\" network=\"<Scalar>\" \" />\n";
    }
  };
  
  // -------------------------------------------------------------------

  InvarianceScript::InvarianceScript() {
    loadUsingConfigManager();
  }
  
  InvarianceScript::~InvarianceScript() {
  }
  
  void InvarianceScript::loadXML(XMLNode& node) {
    
    AttributeParser p;
    
    p.addString("networkFile");
	 p.addInt("patchHeight");
	 p.addInt("patchWidth");

//	 p.addString("dataset");
//  p.addInt("inputHeight");
//  p.addInt("inputWidth");
    
    p.parse(node);
    
    // Note that values are set *directly* into the network
    networkFile = p.getString("networkFile");
	 patchHeight = p.getInt("patchHeight");
	 patchWidth = p.getInt("patchWidth");

//    datasetName = p.getString("dataset");
//    inputHeight = p.getInt("inputHeight");
//    inputWidth  = p.getInt("inputWidth");
    
    // ---------------------------------------------------
    
    // Load AutoEncoder Part
    
		if (networkFile != "") {
			std::ifstream fin;

			fin.open(networkFile.c_str(), std::ios::binary);
			
			SDN_ASSERT_MSG(fin.good(), "Unable to open " << networkFile);
			
			sdn::serial::BinInArchive ar(fin);
			boost::shared_ptr<sdn::AbstractNetwork> net = sdn::AbstractNetworkRegistry().load(ar);
			fin.close();

			SDN_ASSERT_MSG(net, "Unable to load network from file: " << networkFile);

			network = boost::dynamic_pointer_cast<sdn::AbstractAutoEncoderNetwork> (net);
//			network = boost::dynamic_pointer_cast<sdn::InvarianceNetwork> (net);

			SDN_ASSERT_MSG(network, network->className() << " is not an autoencoder network!");

			SDN_MSG("Loaded " << network->className() << " as encoder");
		}
		
    // ---------------------------------------------------



    // We need to create the network here because the XML parameters
    // for it need to be read directly to the network
		/*    
    network = boost::dynamic_pointer_cast<StandardAutoEncoderNetwork> (AbstractNetworkRegistry().make(networkName));
    if (!network) {
      SDN_ERROR("Could not load network with name " << networkName);
      SDN_ERROR("Is " << networkName << " a StandardAutoEncoderNetwork?");
      ERROR_OUT_LIST_REGISTRY_NAMES(AbstractNetwork);
      SDN_FATAL("Quitting");
    }
*/

    // Deep Loading
		/*
    XMLNode networkNode = node.getChildNode("NetworkParameters");
    SDN_ASSERT_MSG(!networkNode.isEmpty(), "You need to specify NetworkParameters !")
    network->loadXML(networkNode); 
	 */

  }

  void InvarianceScript::loadUsingConfigManager() {

    // The following line will load the XML config
    ConfigListener listener
      (boost::shared_ptr<ConfigParser>
        (new InvarianceScriptConfigParser(*this)));

    // We have this here instead of loadXML because loadXML might get
    // called multiple times by the configParser
    initPersistentParameters();
    initTransientParameters();

  }
  
  void InvarianceScript::initTransientParameters() {
/*
	cout << "Starting generation" << endl;
	ifstream fin;
	Matrix w1 (2205,32*32);
        double temp;
	if (fin.is_open())
		for (int i = 0;i < 2205;i++){
			for (int j = 0; j < 32*32; j++){			
			fin >> temp;
			w1(i,j) = temp;
//			cout << temp << " ";
			}
		}
//	cout << "this is the first matrix (transposed)" << endl;
//	for (int i = 0;i<32;i++){
//		for (int j = 0;j<32;j++){
//			cout << w1(j,i)<<" "; 		
//		}	
//		cout << endl;
//	}
//	cout << w1(1,1) << endl;
	cout << "before load affine" << endl;
	inetwork.loadAffineBlock1(w1);
	cout << "after load affine" << endl;
	Matrix w2(315,2205);
//	w2.Zero(595,6545);
	int count = 0;
	for (int i=0;i<315;i++){
		for (int j=0;j<7;j++){
			w2(i,count) = 1;	
			count++;
		}
	}
	cout << "before load affine" << endl;
	inetwork.loadAffineBlock2(w2);
	cout << "before saving" << endl;
	inetwork.quickSaveBinary("inetwork.dat");
	cout << "generation done" << endl;
*/
/*
    dataset =  DatasetRegistry().make(datasetName);
    if (!dataset) {
      SDN_ERROR("Could not load dataset with name " << datasetName);
      ERROR_OUT_LIST_REGISTRY_NAMES(Dataset);
      SDN_FATAL("Quitting");
    }
    dataset->requestExampleSize(inputHeight, inputWidth);

    // We are requesting column vector examples
    inputExample = sdn::Matrix::Zero(inputHeight*inputWidth, 1);
  
*/
  }
  
  void InvarianceScript::initPersistentParameters() {
 //   network->initTransientParameters();
 //   network->initPersistentParameters();
  }
  
  void InvarianceScript::serialize(serial::Archive & ar) {
    SDN_FATAL("InvarianceScript is not serializable yet");
  }

  
  void InvarianceScript::executeOneIteration() {
    
    SDN_ASSERT(manager);
    
  //  int label = dataset->getRandomExampleAsColumn(inputExample);
    
    // do fancy stuff here
    //normInput.normalizePatch(inputExample);
    //network->unsupervisedTrain(inputExample);
    
  }
  
  // ---- Invariance Callback ----

  const boost::shared_ptr<sdn::AbstractAutoEncoderNetwork> InvarianceScript::getNetwork() {
	  return network;
  }
  
}
