//Author: Daniel Chia, Zhenghao Chen, Pang Wei Koh
//File: invarianceTweakScript.h
//Description: 

#pragma once

#include "boost/shared_ptr.hpp"
#include "sdn/stable.h"
#include "./callbacks/invarianceCallbacks.h"
#include "../networks/invarianceTweakingStandardNetwork.h"

namespace sdn {

  // Forward Declarations
  class Dataset;

  /// InvarianceTweakScript class
  /** 
  */


  class InvarianceTweakScript : public AbstractScript,
										  public SaveModelCallbackInterface,
										  public VizWeightsCallbackInterface
	{

		public:


			virtual void loadXML(XMLNode& node);

			virtual void initTransientParameters(); 
			virtual void initPersistentParameters();

			virtual void serialize(serial::Archive & ar);

			virtual void loadUsingConfigManager();

			virtual void executeOneIteration();

			InvarianceTweakScript();
			virtual ~InvarianceTweakScript();

			// Configurable       
			std::string networkFile;
			int patchHeight;
			int patchWidth;
			int invType;

			// Configurable-Persistent Parameters

			// Transient
			boost::shared_ptr<StandardAutoEncoderNetwork> origNetwork;
			boost::shared_ptr<InvarianceTweakingStandardNetwork> network;

			// Fixed Parameters

			// ---- Invariance Callback ----

			virtual const boost::shared_ptr<sdn::AbstractAutoEncoderNetwork> getNetwork();

			// ---- VisualizationCallback ----
			virtual const sdn::Matrix& getEncoderWeights();
			virtual const int getWeightPatchHeight() ;
			virtual const sdn::Matrix& getDecoderWeights() ;


			// ---- SaveModelCallbackCallbackInterface ----
			virtual const void saveModelInDirectory(std::string dir);


	};


}
