//Author: NAME <name@stanford.edu>
//File: templateScript.h
//Description: 

#pragma once

#include "boost/shared_ptr.hpp"
#include "sdn/stable.h"

namespace sdn {

  // Forward Declarations
  class Dataset;

  /// TemplateScript class
  /** 
  */
  class TemplateScript : public AbstractScript, 
                         public ReconstructionErrorCallbackInterface {

  public:
  

    virtual void loadXML(XMLNode& node);

    virtual void initTransientParameters(); 
    virtual void initPersistentParameters();

    virtual void serialize(serial::Archive & ar);

    virtual void loadUsingConfigManager();

    virtual void executeOneIteration();
    
    TemplateScript();
    ~TemplateScript();

    // Configurable       
    std::string networkName;
    std::string datasetName;
    int inputHeight;
    int inputWidth;

    // Configurable-Persistent Parameters
    boost::shared_ptr<AbstractAutoEncoderNetwork> network;
    
    // Transient
    boost::shared_ptr<Dataset> dataset;
    sdn::Matrix inputExample;
    
    // Fixed Parameters
    NormalizeInputBlock normInput;
    
    // ---- Reconstruction Callback ----
    virtual const boost::shared_ptr<CostFunction> getReconstructionCostFunction() ;
    virtual const sdn::Matrix& getReconstruction() ;
    virtual const sdn::Matrix& getOriginal() ;
  
  };
  
  
}
