//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: networkRegistry.cc
//Description: Sets up the registry of composable blocks

#include "networkRegistry.h"
#include "sdn/base.h"

// Stable Networks

#include "../stable/networks/composableAutoEncoderNetwork.h"
#include "../stable/networks/standardAutoEncoderNetwork.h"
#include "../stable/networks/stackedAutoEncoderNetwork.h"
#include "../stable/networks/composableSupervisedNetwork.h"

// Project Networks

#include "../projects/jngiam/networks/jngiamAutoEncoderNetwork.h"
#include "../projects/jngiam/networks/autoTuneNetwork.h"
#include "../projects/pangwei/networks/invarianceTweakingStandardNetwork.h"
#include "../projects/pangwei/networks/invarianceNetwork.h"

namespace sdn {

  Registry<AbstractNetwork>& AbstractNetworkRegistry() {
    static Registry<AbstractNetwork> registry("sdn::AbstractNetwork");
    return registry;
  }

 
}


// AutoEncoder
SDN_AUTOREGISTER(AbstractNetwork, ComposableAutoEncoderNetwork);
SDN_AUTOREGISTER(AbstractNetwork, StandardAutoEncoderNetwork);
SDN_AUTOREGISTER(AbstractNetwork, StackedAutoEncoderNetwork);

// Supervised Stuff
SDN_AUTOREGISTER(AbstractNetwork, ComposableSupervisedNetwork);

// Project Stuff

// jngiam
SDN_AUTOREGISTER(AbstractNetwork, JngiamAutoEncoderNetwork);
SDN_AUTOREGISTER(AbstractNetwork, AutoTuneNetwork);

// pangwei
SDN_AUTOREGISTER(AbstractNetwork, InvarianceTweakingStandardNetwork);
SDN_AUTOREGISTER(AbstractNetwork, InvarianceNetwork);

