//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: scriptRegistry.cc
//Description: Sets up the registry of scripts

#include "scriptRegistry.h"
#include "sdn/base.h"

// Stable Scripts
#include "../stable/scripts/callbacks/sharedCallbacks.h"
#include "../stable/scripts/callbacks/vizCallbacks.h"
#include "../stable/scripts/codeCompareScript.h"
#include "../stable/scripts/standardAutoEncoderScript.h"
#include "../stable/scripts/unsupervisedTrainingScript.h"
#include "../stable/scripts/supervisedWithPreTrainingScript.h"
#include "../stable/scripts/trainStackedAutoEncodersScript.h"


// Project Scripts
#include "../projects/template/scripts/templateScript.h"
#include "../projects/jngiam/scripts/jngiamScript.h"
#include "../projects/danchia/scripts/danchiaScript.h"
#include "../projects/danchia/scripts/danchiaSupervisedScript.h"
#include "../projects/danchia/scripts/danchiaCrossValidationScript.h"
#include "../projects/danchia/scripts/danchiaConvertNetworkOutScript.h"
#include "../projects/danchia/scripts/danchiaConvertNetworkInScript.h"
#include "../projects/jngiam/scripts/jngiamScript1.h"
#include "../projects/jngiam/scripts/autoTuneScript.h"
#include "../projects/pangwei/scripts/invarianceScript.h"
#include "../projects/pangwei/scripts/callbacks/invarianceCallbacks.h"
#include "../projects/pangwei/scripts/invarianceTweakScript.h"

namespace sdn {

  Registry<Script>& ScriptRegistry() {
    static Registry<Script> registry("sdn::Script");
    return registry;
  }

  Registry<ScriptCallback>& ScriptCallbackRegistry() {
    static Registry<ScriptCallback> registry("sdn::ScriptCallback");
    return registry;
  }

 
}


// Stable Scripts
SDN_AUTOREGISTER(Script, StandardAutoEncoderScript);
SDN_AUTOREGISTER(Script, TrainStackedAutoEncodersScript);
SDN_AUTOREGISTER(Script, UnsupervisedTrainingScript);
SDN_AUTOREGISTER(Script, SupervisedWithPreTrainingScript);
SDN_AUTOREGISTER(Script, CodeCompareScript);

// Stable Callbacks

SDN_AUTOREGISTER(ScriptCallback, ReconstructionErrorCallback);
SDN_AUTOREGISTER(ScriptCallback, SupervisedErrorCallback);
SDN_AUTOREGISTER(ScriptCallback, ProgressReportCallback);
SDN_AUTOREGISTER(ScriptCallback, SaveModelCallback);
SDN_AUTOREGISTER(ScriptCallback, DisableOtherCallback);
SDN_AUTOREGISTER(ScriptCallback, CodeCompareScriptCompareDifferenceCallback);
SDN_AUTOREGISTER(ScriptCallback, VizWeightsCallback);

// Project Scripts
SDN_AUTOREGISTER(Script, TemplateScript);
SDN_AUTOREGISTER(Script, JngiamScript);
SDN_AUTOREGISTER(Script, DanchiaScript);
SDN_AUTOREGISTER(Script, DanchiaSupervisedScript);
SDN_AUTOREGISTER(Script, DanchiaCrossValidationScript);
SDN_AUTOREGISTER(Script, DanchiaConvertNetworkOutScript);
SDN_AUTOREGISTER(Script, DanchiaConvertNetworkInScript);
SDN_AUTOREGISTER(Script, JngiamScript1);
SDN_AUTOREGISTER(Script, AutoTuneScript);
SDN_AUTOREGISTER(Script, InvarianceScript);
SDN_AUTOREGISTER(Script, InvarianceTweakScript);

// Project Callbacks

SDN_AUTOREGISTER(ScriptCallback, InvarianceCallback);
