//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: abstractBlock.h
//Description: Interface for generic blocks.

#pragma once

#include "sdn/base.h"
#include "helpers.h"

namespace sdn {

  // Forward Declarations
  namespace serial {
    class Archive;
  }

  // 

  class AbstractBlock {
  
  public:
  
    // Intentionally disabled. 
    // Derived classes should implement this with default parameter values;
    // virtual void setConfigurableParameters() = 0;
    virtual void loadXML(XMLNode& node) = 0;
    
    virtual void initTransientParameters() = 0;
    virtual void initPersistentParameters() = 0;
    
    virtual void serialize(serial::Archive & ar) = 0;
    
    // Do not overwrite this function
    // This should work out of the box
    virtual std::string className() {
      return std::string(demangle(typeid(*this).name()));
    }
    
    virtual void quickSaveBinary(std::string filename) {
      std::ofstream fout;
      fout.open(filename.c_str(), std::ios::binary);
      serial::BinOutArchive ar(fout);
      this->serialize(ar);
      fout.close();
    }
   
    virtual void quickLoadBinary(std::string filename) {
      std::ifstream fin;
      fin.open(filename.c_str(), std::ios::binary);
      serial::BinInArchive ar(fin);
      this->serialize(ar);
      fin.close();
    }
   
  };
  
  class ComposableBlock : public AbstractBlock {
    
  public:

    virtual void forwardProp(const sdn::Matrix& input,
                             sdn::Matrix& output) = 0;
   
    virtual void backProp(const sdn::Matrix& input,
                          const sdn::Matrix& outputErrorDerivatives,
                          sdn::Matrix& inputErrorDerivatives) = 0;
    
    virtual void gradientUpdate() = 0;
  
  };
  
}
