//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: composableBlocks.cc
//Description: Implementation of blocks used in Composable networks

#include "composableBlocks.h"
#include "composableBlockRegistry.h"
#include "sdn/base.h"
#include <Eigen/Array>

namespace sdn {
  
  void LoadComposableBlocksBlock::loadXMLComposableBlocks(std::vector<boost::shared_ptr<ComposableBlock> >& blocks, XMLNode& architectureNode) {
    // We assume the blocks appear in order
    for(int i = 0; i < architectureNode.nChildNode(); i++) {
      
      XMLNode blockNode = architectureNode.getChildNode(i);
      const char* blockName;
      SDN_ASSERT(blockName = blockNode.getName());
      
      // Try to load up this block
      boost::shared_ptr<ComposableBlock> newBlock;
      newBlock = ComposableBlockRegistry().make(blockName);
      if (!newBlock) {
        SDN_ERROR("Could not load ComposableBlock with name " << blockName);
        ERROR_OUT_LIST_REGISTRY_NAMES(ComposableBlock);
        SDN_FATAL("Quitting");
      }
      
      // Add Block to our List
      blocks.push_back(newBlock);
      
      // Initialize the Block
      blocks.back()->loadXML(blockNode);
      
    }
  }

}
