//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: sharedBlocks.h
//Description: Implementation of generic blocks used everywhere

#pragma once

#include "sdn/base.h"
#include "../helpers.h"
#include "../defaultParameters.h"
#include "../abstractBlock.h"

namespace sdn {

  /// normalizeInputBlock class
  /** INCOMPLETE AS OF NOW
   *  Should have a MAX/MIN specification option etc
  */
  class NormalizeInputBlock : public AbstractBlock {

  public:
  
    virtual void loadXML(XMLNode& node) {}
    virtual void initTransientParameters() {}
    virtual void initPersistentParameters() {}
    virtual void serialize(serial::Archive & ar) {}
    
    virtual void normalizePatch(sdn::Matrix& patch);

  };
  
  /// randomizeWeightMatrix class
  /** 
  */
  class RandomizeWeightMatrix : public AbstractBlock {

  public:
  
    virtual void loadXML(XMLNode& node) {}
    virtual void initTransientParameters() {}
    virtual void initPersistentParameters() {}
    virtual void serialize(serial::Archive & ar) {}
    
    virtual void randomize(sdn::Matrix& weights, unsigned int inputSize);

  };

  /// ExponentiallyWeightedMeanBlock class
  /** Tracks activation values
  */
  class ExponentiallyWeightedMeanBlock : public AbstractBlock {

  public:
  
    /**
     * Initializes the block
     * 
     * @param defAveragingConstant the exponentially weighted averaging
     *        constant. (normally we set this to about 0.001)
     * 
     * @param defInitialValue set this to TRACKER_AUTOSET to just 
     *        initialize it using the first value you add
     *        
     * @param defTrackingSize set this to SIZE_AUTOSET and the tracker
     *        will resize on the first time you add a value
     */
    virtual void setConfigurableParameters(sdn::Scalar defAveragingConstant,
                                           sdn::Scalar defInitialValue,
                                           int defTrackingSize) ;
    virtual void loadXML(XMLNode& node);
    
    virtual void initTransientParameters() ;
    virtual void initPersistentParameters() ;
    
    virtual void serialize(serial::Archive & ar) ;
    
    // Configurable Parameters
    sdn::Scalar averagingConstant;
    sdn::Scalar initialValue;
    int trackingSize;
    
    // Transient Parameters
    sdn::Matrix activations;
    
    virtual void addValues(const sdn::Matrix& values);
    const sdn::Matrix& getMean();
    
  };
  
}
