//Author: Andrew Maas <amaas@cs.stanford.edu>
//File: vizWeightMatrixBlock.cc
//Description: Implementation of vizWeightMatrixBlock class

#include "vizWeightMatrixBlock.h"
#include "sdn/base.h"
#include <Eigen/Array>
// @todo add base src to include path to avoid ../ in includes
#include "../../gui/patchDisplay.h"

namespace sdn {

  void VizWeightMatrixBlock::vizMatrix(const sdn::Matrix& w, int pHeight, 
               int gridWidth,
               std::string saveName, bool disp) {
    const int sizeMult= 2;
    // determine size of patches 
    int pWidth = w.cols() / pHeight;
    SDN_ASSERT(w.cols() % pHeight == 0);

    // determine number of rows in the grid
    // it is okay if to have more grid spots than necessary
    int gridHeight = ceil( ((float)w.rows()) / ((float)gridWidth));
    
    SDN_DEBUG("viz display size:: " << gridWidth << " x " << gridHeight); 
    // initialize a patchDisplay
    PatchDisplay curPD = PatchDisplay("vizBlockDisplay", 
              pWidth, pHeight, sizeMult, 
              gridWidth, gridHeight);
    
    for(int i = 0; i < w.rows(); ++i) {
      curPD.addPatch(vectorToMatrix(w.row(i), 
                     pHeight,
                     pWidth),
                     false,
                     false);
      }


    // save
    if (saveName != "")
      curPD.saveImage(saveName.c_str());
    
    // show the display
    if (disp) {
      curPD.enableWindow();
    
      // wait 500ms
      curPD.wait(2000);
    }
  };
}
