//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: helpers.cc
//Description: Implementation of helpers used everywhere

#include "helpers.h"
#include "sdn/base.h"
#include <cxxabi.h>
#include <Eigen/Array>
#include <stdarg.h>

namespace sdn {

  // ----------------- HELPER FUNCTIONS -----------------
  
  std::string dirAddSlash(std::string dir) {
    if (dir == "")
      dir = ".";
    if (dir[dir.size()-1] != '/')
      dir = dir + "/";
    return dir;
  }

  std::string itos(unsigned int i, int pad) {
    
    char formatString[64] = "%u";
    if (pad > 0)
      snprintf(formatString, 63, "%s0%d%s", "%", pad, "u");

    char temp[1024];
    snprintf(temp, 1023, formatString, i);
    return temp;
  }
  
  std::string itos(int i, int pad) {
    
    char formatString[64] = "%d";
    if (pad > 0)
      snprintf(formatString, 63, "%s0%d%s", "%", pad, "d");
      
    char temp[1024];
    snprintf(temp, 1023, formatString, i);
    return temp;
  }

  const char* demangle(const char* name) {
    char buf[1024];
    size_t size=1024;
    int status;
    char* res = abi::__cxa_demangle (name, buf, &size, &status);
    return res;
  }

  void appendToFile (std::string filename, const char * format, ...) {
    FILE* f = fopen(filename.c_str(), "a");
    if (!f) {
      SDN_ERROR("Unable to write to file " << filename);
    }
    va_list args;
    va_start (args, format);
    vfprintf (f, format, args);
    va_end (args);
    fclose(f);
  }

  void clearFile (std::string filename) {
    FILE* f = fopen(filename.c_str(), "w");
    fclose(f);
  }

  std::string quickString (const char * format, ...) {
    char buffer[1024];
    va_list args;
    va_start (args, format);
    vsprintf (buffer, format, args);
    va_end (args);
    return buffer;
  }


}
