//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: helpers.h
//Description: Implementation of helpers used everywhere

#pragma once

#include "sdn/base.h"

namespace sdn {
  
  std::string dirAddSlash(std::string dir);  
  std::string itos(unsigned int i, int pad=-1);  
  std::string itos(int i, int pad=-1);  
  const char* demangle(const char* name);


  template<typename T>
  std::string getClassName(T& arg){
    return (demangle(typeid(arg).name()));
  }

  template<typename T>
  std::string getClassName(T* arg){
    return (demangle(typeid(*arg).name()));
  }
  
  /** Appends data to file, printf style
   *  @param filename the output file name
   *  @param format the printf format string, don't forget the trailing \n
   */
  void appendToFile (std::string filename, const char * format, ...) ;  
  
  /** Empties a file (presumably re-running an experiment
   *  @param filename the output file name
   */
  void clearFile (std::string filename) ;  
  
  /** Makes a std::string quickly printf style
   *  @param format printf format string
   */
  std::string quickString (const char * format, ...) ;  

}

#define ERROR_OUT_LIST_REGISTRY_NAMES(registryName) \
  std::string registryName##ErrorInfo = "Available " + std::string(SDN_MAKE_STRING(registryName)) + " include: "; \
  std::vector<std::string> registryName##Names; \
  registryName##Registry().getInstanceNames(registryName##Names); \
  for (unsigned i = 0; i < registryName##Names.size(); i++) \
    registryName##ErrorInfo += "\n\t" + registryName##Names[i]; \
  SDN_ERROR(registryName##ErrorInfo);
  
