// Copyright 2009, All Rights Reserved.
/// \author Andrew Saxe (asaxe@stanford.edu)
/// \author Ian Goodfellow
/// \file costFunction.cc

#include "costFunction.h"
#include <assert.h>

#include <Eigen/Array>

using namespace sdn;

Matrix SumOfSquaresCostFunction::output_(const Matrix& actual, 
					  const Matrix& desired) {
  assert(actual.rows() == desired.rows());
  assert(actual.cols() == desired.cols());
  
  return Matrix::Constant(1,1,(actual - desired).cwise().square().sum()/2.0);
}

Matrix SumOfSquaresCostFunction::jacobian_(


      const Matrix& actual, 
      const Matrix& desired) {


  //SDN_DEBUG("max actual: "<<actual.maxCoeff()<<" min actual: "<<actual.minCoeff());
  //SDN_DEBUG("max desired: "<<desired.maxCoeff()<<" min desired: "<<desired.minCoeff());


  assert(actual.rows() == desired.rows());
  assert(actual.cols() == desired.cols());

  return (actual - desired).transpose();
}


void SumOfSquaresCostFunction::serialize(serial::Archive & ar)
{
  SDN_DEBUG("Serializing SumOfSquaresCostFunction");
  int version = 12;

  version = ar.beginDerived(this, version, "sdn::SumOfSquaresCostFunction");
  
  switch (version)
    {
    case 12:
      ar.beginSuper();

      CostFunction::serializeAbstract(ar);

      ar.endSuper();
      ar.endDerived();

      break;
    default:
      SDN_FATAL("Invalid SumOfSquaresCostFunction version requested");
    }
}

void CostFunction::serializeAbstract(serial::Archive & ar)
{
  SDN_DEBUG("serializing CostFunction");

  /** \todo-- current serialization framework assumes that registry
  base class is the base class of the constructed object, but in the
  case of CostFunction's that's not true. Make the serialization more
  flexible so we don't have to bail here */

  int version = 312;
  
  version = ar.beginBase(this, version, "sdn::CostFunction");

  switch (version)
    {
    case 312:
      ar.attribute("lambda",lambda);
      ar.endBase();
      break;
    default:
      SDN_FATAL("Invalid CostFunction version requested");
    }
}


CostFunction::~CostFunction()
{

}

SumOfSquaresCostFunction::~SumOfSquaresCostFunction()
{

}




Matrix CrossEntropyCostFunction::output_(const Matrix& actual,
                                         const Matrix& desired) {
  assert(actual.rows() == desired.rows());
  assert(actual.cols() == desired.cols());

  return Matrix::Constant(1,1,(-desired.transpose()*(actual.cwise().log())).sum());
}

Matrix CrossEntropyCostFunction::jacobian_(const Matrix& actual,
                                           const Matrix& desired) {
  assert(actual.rows() == desired.rows());
  assert(actual.cols() == desired.cols());

  return ((-desired).cwise() / actual).transpose();
}

CrossEntropyCostFunction::~CrossEntropyCostFunction() {
}

void CrossEntropyCostFunction::serialize(serial::Archive & ar)
{
  SDN_DEBUG("Serializing CrossEntropyCostFunction");
  int version = 12;

  version = ar.beginDerived(this, version, "sdn::CrossEntropyCostFunction");
  
  switch (version)
    {
    case 12:
      ar.beginSuper();

      CostFunction::serializeAbstract(ar);

      ar.endSuper();
      ar.endDerived();

      break;
    default:
      SDN_FATAL("Invalid CrossEntropyCostFunction version requested");
    }
}







Registry<CostFunction> & sdn::CostFunctionRegistry()
{
  static Registry<CostFunction> registry("sdn::CostFunction");
  return registry;
}

SDN_AUTOREGISTER(CostFunction, SumOfSquaresCostFunction);
SDN_AUTOREGISTER(CostFunction, CrossEntropyCostFunction);
