//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: composableSupervisedNetwork.h
//Description: Uses ComposableBlocks to create an AutoEncoder

#pragma once

#include "sdn/base.h"
#include "../abstractNetwork.h"
#include "../../registry/composableBlockRegistry.h"
#include "../blocks/composableBlocks.h"
#include "../blocks/sharedBlocks.h"
#include "../blocks/standardBlocks.h"
#include "../defaultParameters.h"
#include "../math/costFunction.h"
#include "../math/normPenalty.h"

namespace sdn {
  
  class ComposableSupervisedNetwork : public AbstractSupervisedNetwork {
  
  public:

    virtual void loadXML(XMLNode& node) ;
    
    virtual void initTransientParameters() ;
    virtual void initPersistentParameters() ;
    
    virtual void serialize(serial::Archive & ar) ;


    virtual void supervisedTrain(const sdn::Matrix& example, 
                                 const sdn::Matrix& label) ;

    virtual const boost::shared_ptr<CostFunction> getCostFunction();
    
    // Equivalent to classify
    virtual void forwardProp(const sdn::Matrix& input,
                             sdn::Matrix& output) ;

    virtual const sdn::Matrix& getPrediction();
    
    // Backprops through the supervised layers only
    virtual void backProp(const sdn::Matrix& input,
                          const sdn::Matrix& outputErrorDerivatives,
                          sdn::Matrix& inputErrorDerivatives) ;
    
    virtual void gradientUpdate() ;


    virtual void useAutoEncoderNetwork(boost::shared_ptr<AbstractAutoEncoderNetwork> encodingNet) ;
    // Backprops through the autoencoder + gradient update it
    virtual void fineTuneAutoEncoder(const sdn::Matrix& input) ;


    sdn::Matrix& getOutputGradient();

    // Persistent-Transient Parameters
    
    // We do not actually serialize the blocks, but instead the parameters
    // In effect, no pointers are serialized
    std::vector<boost::shared_ptr<ComposableBlock> > blocks;
    boost::shared_ptr<AbstractAutoEncoderNetwork> encodingNetwork;
    
    // Configurable Parameters
    std::string costFunctionName;
    
    // Transient Parameters
    boost::shared_ptr<CostFunction> supervisedErrorCostFunction;
    std::vector<sdn::Matrix> intermediateRepresentations;// Representations for the values *after* the block (so [blocks.size()-1] = reconstruction)
    std::vector<sdn::Matrix> intermediateDerivatives;    // Gradients for the values *after* the block (so [blocks.size()-1] = outputGradient)
    sdn::Matrix inputDerivatives; // Gradient at the inputs (how to change the inputs to decrease error)
    sdn::Matrix fineTuneInputDerivatives;
    sdn::Matrix prediction;
    

  };
  
}
