//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: StackedAutoEncoderNetwork.h
//Description: Uses AutoEncoderNetworks to create a Deep Network

#pragma once

#include "../abstractNetwork.h"
#include "../defaultParameters.h"
#include <vector>

namespace sdn {

  /// StackedAutoEncoderNetwork class
  /** 
  */
  class StackedAutoEncoderNetwork : public AbstractAutoEncoderNetwork {
  public:

    virtual void setConfigurableParameters();
    virtual void loadXML(XMLNode& node);
  
    virtual void initTransientParameters();
    virtual void initPersistentParameters();
    
    virtual void serialize(serial::Archive & ar);
    
    virtual void unsupervisedTrain(const sdn::Matrix& example);
    virtual const boost::shared_ptr<CostFunction> getCostFunction();
    
    virtual const sdn::Matrix& encode(const sdn::Matrix& input);             // <-- calles networks in aStackOfNetworks in turn
    virtual const sdn::Matrix& decode(const sdn::Matrix& hidden);

    virtual void encodeBackProp(const sdn::Matrix& input,
                                const sdn::Matrix& hiddenErrorDerivatives,
                                sdn::Matrix& inputErrorDerivatives);

    virtual void decodeBackProp(const sdn::Matrix& hidden,
                                const sdn::Matrix& outputErrorDerivatives,
                                sdn::Matrix& hiddenErrorDerivatives);

    virtual void encodeGradientUpdate();
    virtual void decodeGradientUpdate();

    virtual const sdn::Matrix& getHiddenRepresentation();
    virtual const sdn::Matrix& getReconstruction();

    // Internal helpers
    virtual sdn::Matrix& getHiddenGradient();
    virtual sdn::Matrix& getOutputGradient();
    
    StackedAutoEncoderNetwork();
    ~StackedAutoEncoderNetwork();
    
    virtual void addAutoEncoder(boost::shared_ptr<AbstractAutoEncoderNetwork> anAutoEncoder);

    // Configurable Parameters
    std::string costFunctionName;

    // Persistent Parameters
    std::vector<boost::shared_ptr<AbstractAutoEncoderNetwork> > networkStack;
    
    // Transient Parameters
    boost::shared_ptr<CostFunction> reconstructionCostFunction;

    std::vector<sdn::Matrix> encodeIntermediateRepresentations;// Representations for the values *after* the networkStack (so [networkStack.size()-1] = hiddenRepresentation)
    std::vector<sdn::Matrix> decodeIntermediateRepresentations;// Representations for the values *after* the networkStack (so [0] = reconstruction)
    
    std::vector<sdn::Matrix> encodeIntermediateDerivatives;    // Gradients for the values *after* the networkStack (so [networkStack.size()-1] = hiddenDerivatives)
    std::vector<sdn::Matrix> decodeIntermediateDerivatives;    // Gradients for the values *after* the networkStack (so [0] = outputDerivatives)
    
    sdn::Matrix inputDerivatives; // Gradient at the inputs (how to change the inputs to decrease error)
    
    sdn::Matrix inputToEmptyNetwork;

  };
}
