
//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: StandardAutoEncoderNetwork.h
//Description: Uses AutoEncoderBlocks to create a Network

#pragma once

#include "sdn/base.h"
#include "../abstractNetwork.h"
#include "../blocks/sharedBlocks.h"
#include "../blocks/standardBlocks.h"
#include "../defaultParameters.h"
#include "../math/costFunction.h"
#include "../math/normPenalty.h"

namespace sdn {

  /// StandardAutoEncoderNetwork class
  /** 
  */
  class StandardAutoEncoderNetwork : public AbstractAutoEncoderNetwork {
  public:

    virtual void setConfigurableParameters(sdn::Scalar defTargetActivation,
                                           sdn::Scalar defLearningRateSparsity,
                                           unsigned int defHiddenLayerSize,
                                           sdn::Scalar  defAveragingConstant);
    virtual void loadXML(XMLNode& node) ;
    
    virtual void initTransientParameters();
    virtual void initPersistentParameters();
    
    virtual void serialize(serial::Archive & ar) ;
    
    virtual void unsupervisedTrain(const sdn::Matrix& example);
    virtual const boost::shared_ptr<CostFunction> getCostFunction();
    
    virtual const sdn::Matrix& encode(const sdn::Matrix& input);
    virtual const sdn::Matrix& decode(const sdn::Matrix& hidden);

    virtual void encodeBackProp(const sdn::Matrix& input,
              const sdn::Matrix& hiddenErrorDerivatives,
        sdn::Matrix& inputErrorDerivatives);

    virtual void decodeBackProp(const sdn::Matrix& hidden,
              const sdn::Matrix& outputErrorDerivatives,
        sdn::Matrix& hiddenErrorDerivatives);

    virtual void encodeGradientUpdate();
    virtual void decodeGradientUpdate();
    
    virtual const sdn::Matrix& getHiddenRepresentation();
    virtual const sdn::Matrix& getReconstruction();
    
    virtual void setupGoodInitialParameters();
    
    StandardAutoEncoderNetwork();
    ~StandardAutoEncoderNetwork();

    // Persistent Parameters
    AffineBlock affineEncoder, affineDecoder;
    TanhBlock   sigmEncoder, sigmDecoder;
    
    // Configurable Parameters
    sdn::Scalar  targetActivation;
    sdn::Scalar  activationAveragingConstant; 
    sdn::Scalar  learningRateSparsity; 
    unsigned int hiddenLayerSize;
    
    // Transient Parameters
    ExponentiallyWeightedMeanBlock activationTracker;
    sdn::Matrix hiddenRepresentation, affineRepEnc, affineRepDec;
    sdn::Matrix inputGradient, hiddenGradient, outputGradient;
    sdn::Matrix reconstruction;
    
    // Fixed Parameters, could technically load this 
    boost::shared_ptr<CostFunction> costFunc;
    
  };

}
  
