//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: sharedCallbacks.h
//Description: 

#pragma once

#include "../../blocks/sharedBlocks.h"
#include "../../abstractScript.h"
#include <boost/shared_ptr.hpp>

namespace sdn {
  
  // Forward Declarations
  class CostFunction;
  

  /// ReconstructionErrorCallback interface and class
  /** To use this callback, have your script implement this interface
  */
  class ReconstructionErrorCallbackInterface {
  public:
    virtual const boost::shared_ptr<CostFunction> getReconstructionCostFunction() = 0 ;
    virtual const sdn::Matrix& getReconstruction() = 0 ;
    virtual const sdn::Matrix& getOriginal() = 0;
  };

  class ReconstructionErrorCallback : public AbstractScriptCallback {

  public:
  
    virtual void loadXML(XMLNode& node) ;

    virtual void initTransientParameters() ;
    virtual void initPersistentParameters() ;

    virtual void serialize(serial::Archive & ar) ;
 
    virtual void execute(ScriptRuntimeManager& scriptManager,
                         AbstractScript& script,
                         unsigned int iteration) ;
    
    virtual long int itersToNextCall();
    virtual long int firstIter();
    
    ReconstructionErrorCallback() {}
    ~ReconstructionErrorCallback() {}

    // Configurable
    double averagingConstant;
    long int reportStep;
    long int runFirstIter;
    std::string saveFile;
    
    // Transient
    ExponentiallyWeightedMeanBlock errorTracker;
  
  };

  /// SupervisedErrorCallback interface and class
  /** To use this callback, have your script implement this interface
  */
  class SupervisedErrorCallbackInterface {
  public:
    virtual const boost::shared_ptr<CostFunction> getSupervisedCostFunction() = 0 ;
    virtual const sdn::Matrix& getTarget() = 0 ;
    virtual const sdn::Matrix& getPrediction() = 0;
  };

  class SupervisedErrorCallback : public AbstractScriptCallback {

  public:
  
    virtual void loadXML(XMLNode& node) ;

    virtual void initTransientParameters() ;
    virtual void initPersistentParameters() ;

    virtual void serialize(serial::Archive & ar) ;
 
    virtual void execute(ScriptRuntimeManager& scriptManager,
                         AbstractScript& script,
                         unsigned int iteration) ;
    
    virtual long int itersToNextCall();
    virtual long int firstIter();
    
    SupervisedErrorCallback() {}
    ~SupervisedErrorCallback() {}

    // Configurable
    double averagingConstant;
    long int reportStep;
    long int runFirstIter;

		std::string saveFile;
    
    // Transient
    ExponentiallyWeightedMeanBlock errorTracker;
  
  };



  /// ProgressReportCallback interface and class
  /** To use this callback, have your script implement this interface
  */
  class ProgressReportCallbackInterface {
  public:
    virtual void reportProgress(std::ostream& o) = 0 ;
  };

  class ProgressReportCallback : public AbstractScriptCallback {

  public:
  
    virtual void loadXML(XMLNode& node) ;

    virtual void initTransientParameters() {}
    virtual void initPersistentParameters() {}

    virtual void serialize(serial::Archive & ar) ;
 
    virtual void execute(ScriptRuntimeManager& scriptManager,
                         AbstractScript& script,
                         unsigned int iteration) ;
    
    virtual long int itersToNextCall();
    virtual long int firstIter();
    
    ProgressReportCallback() {}
    ~ProgressReportCallback() {}

    // Configurable
    long int reportStep;
    
  
  };
  
  /// DisableOtherCallback class
  class DisableOtherCallback  : public AbstractScriptCallback {

  public:
  
    virtual void loadXML(XMLNode& node) ;

    virtual void initTransientParameters() {}
    virtual void initPersistentParameters() {}

    virtual void serialize(serial::Archive & ar) ;
 
    virtual void execute(ScriptRuntimeManager& scriptManager,
                         AbstractScript& script,
                         unsigned int iteration) ;
    
    virtual long int itersToNextCall();
    virtual long int firstIter();
    
    DisableOtherCallback() {}
    ~DisableOtherCallback() {}

    // Configurable
    long int disableIter;
    std::string targetCallback;
  
  };

  /// SaveModelCallback interface and class
  /** To use this callback, have your script implement this interface
  */
  class SaveModelCallbackInterface {
  public:
    virtual const void saveModelInDirectory(std::string dir) = 0;
  };

  class SaveModelCallback : public AbstractScriptCallback {

  public:
  
    virtual void loadXML(XMLNode& node) ;

    virtual void initTransientParameters() ;
    virtual void initPersistentParameters() ;

    virtual void serialize(serial::Archive & ar) ;
 
    virtual void execute(ScriptRuntimeManager& scriptManager,
                         AbstractScript& script,
                         unsigned int iteration) ;
    
    virtual long int itersToNextCall();
    virtual long int firstIter();
    
    SaveModelCallback() {}
    ~SaveModelCallback() {}

    // Configurable
    long int step;
    long int runFirstIter;
    std::string saveDir;
    
  };

}
