//Author: Andrew Maas <amaas@cs.stanford.edu>
//File: vizCallbacks.cc
//Description: Set of callback interfaces and callback classes for display.

#include "vizCallbacks.h"
#include "../../helpers.h"
#include "sdn/base.h"

namespace sdn {

  void VizWeightsCallback::loadXML(XMLNode & node) {
    AttributeParser p;

    p.addInt("reportStep");
    p.addInt("firstReportStep", 0);
    p.addInt("displayGridWidth");
    p.addBool("showWindow");
    
    p.parse(node);

    reportStep = p.getInt("reportStep");
    firstReportStep = p.getInt("firstReportStep");
    displayGridWidth = p.getInt("displayGridWidth");
    showWindow = p.getBool("showWindow");

    initPersistentParameters();
    initTransientParameters();

  }

  
  long int VizWeightsCallback::itersToNextCall() {
    return reportStep;
  }
  
  long int VizWeightsCallback::firstIter() {
    return firstReportStep;
  }
  
  void VizWeightsCallback::execute(ScriptRuntimeManager& scriptManager,
                                            AbstractScript& script,
                                            unsigned int iteration) {
    SDN_DEBUG("vizWeightsCallback"); 
    
    // cast to inferface class
    VizWeightsCallbackInterface* vScript= NULL;
    vScript = dynamic_cast<VizWeightsCallbackInterface*> (&script);
    SDN_ASSERT(vScript != NULL);
    
    //@todo more options for displaying enc/denc weights
    // call the display block
    VizWeightMatrixBlock tmpViz;
    tmpViz.vizMatrix(vScript->getEncoderWeights(), 
         vScript->getWeightPatchHeight(), 
         displayGridWidth, quickString("viz-%010u.png", iteration), showWindow);

       
  }

  void VizWeightsCallback::initTransientParameters() {
  }
  
  void VizWeightsCallback::initPersistentParameters() {
  }
  
  void VizWeightsCallback::serialize(serial::Archive & ar) {
    SDN_FATAL("VizWeightsCallback not serializable yet");
  }

  VizWeightsCallback::VizWeightsCallback() {
    // in case xml does not specify
    reportStep = 0;
    firstReportStep = 0;
  }
  
  VizWeightsCallback::~VizWeightsCallback() {
  }
  
}
