//Author: Andrew Maas <amaas@cs.stanford.edu>
//File: vizCallbacks.h
//Description: Set of callback interfaces and callback classes for display.

#pragma once

#include "../../blocks/vizWeightMatrixBlock.h"
#include "../../abstractScript.h"

namespace sdn {

  /** 
   * Interface a script must implement to use the VizualizeMatrix Callback
   */
  class VizWeightsCallbackInterface {
  public:
    /**
     * Returns weight matrix where each row is a receptive field
     */
    virtual const sdn::Matrix& getEncoderWeights() = 0;

    /** Height of a weight patch
     *  Used in reshaping the encoder weights for display
     *  The width of the weight patch is inferred from this and matrix size
     */
    virtual const int getWeightPatchHeight() = 0;
    virtual const sdn::Matrix& getDecoderWeights() = 0;
  };

  class VizWeightsCallback : public AbstractScriptCallback {

  public:
  
    virtual void loadXML(XMLNode& node) ;

    virtual void initTransientParameters() ;
    virtual void initPersistentParameters() ;

    virtual void serialize(serial::Archive & ar) ;
 
    virtual void execute(ScriptRuntimeManager& scriptManager,
                         AbstractScript& script,
                         unsigned int iteration) ;
    
    virtual long int itersToNextCall();
    virtual long int firstIter();
    
    VizWeightsCallback();
    ~VizWeightsCallback();

  protected:
    // Configured by XML
    long int reportStep;
    long int firstReportStep;
    int displayGridWidth;
    bool showWindow;
  
  };

}
