//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: CodeCompareScript.cc
//Description: 

#include "codeCompareScript.h"
#include "../scriptRuntimeManager.h"
#include "sdn/base.h"
#include "sdn/old.h"

namespace sdn {

  void CodeCompareScript::loadXML(XMLNode& node) {}
  void CodeCompareScript::initPersistentParameters() {}
  void CodeCompareScript::loadUsingConfigManager() {}

  void CodeCompareScript::initTransientParameters() {
    // Set manager for newNetScript
    newNetScript.manager = manager;
    
    // At this point, newNetScript should have already been configured
    oldSdnNet = boost::dynamic_pointer_cast<sdn::StackedNetwork> (sdn::NetworkRegistry().make("sdn::StackedNetwork"));

    // Prevent the net from normalizing
    oldSdnNet->normalize_contrast = false;

    // Force the initial weight and bias parameters to be the same with the NN
    boost::shared_ptr<sdn::StandardLayer> sdnEncoder = boost::dynamic_pointer_cast<sdn::StandardLayer> (oldSdnNet->getLayer(0));
    boost::shared_ptr<sdn::StandardLayer> sdnDecoder = boost::dynamic_pointer_cast<sdn::StandardLayer> (oldSdnNet->getLayer(1));

#ifdef USE_COMPOSABLE_NETWORK
    boost::shared_ptr<sdn::ComposableAutoEncoderNetwork> cNet = boost::dynamic_pointer_cast<sdn::ComposableAutoEncoderNetwork> (newNetScript.network);
    boost::shared_ptr<sdn::SparseAffineTanhBlock> encBlock = boost::dynamic_pointer_cast<sdn::SparseAffineTanhBlock> (cNet->blocks[0]);
    boost::shared_ptr<sdn::AffineTanhBlock> decBlock = boost::dynamic_pointer_cast<sdn::AffineTanhBlock> (cNet->blocks[1]);
    
    sdnEncoder->setWeights(encBlock->transferBlock.affineB.weights);
    sdnEncoder->setBiases(encBlock->transferBlock.affineB.bias);
    
    sdnDecoder->setWeights(decBlock->affineB.weights);
    sdnDecoder->setBiases(decBlock->affineB.bias);
#else
    sdnEncoder->setWeights(newNetScript.network.affineEncoder.weights);
    sdnEncoder->setBiases(newNetScript.network.affineEncoder.bias);
    
    sdnDecoder->setWeights(newNetScript.network.affineDecoder.weights);
    sdnDecoder->setBiases(newNetScript.network.affineDecoder.bias);
#endif    
    SDN_MSG("Matched Initial Weights/Bias");
    
  }
  
  void CodeCompareScript::serialize(serial::Archive & ar) {
    SDN_FATAL(className() << " not serializable.");
  }

  void CodeCompareScript::executeOneIteration()  {

    newNetScript.executeOneIteration();

    oldSdnNet->forwardProp(newNetScript.inputExample);
    oldSdnNet->presentTarget(newNetScript.inputExample);
    oldSdnNet->backProp();
    oldSdnNet->gradientDescentUpdate();
    oldSdnNet->gradientDescentUpdateSparsity();
    
  }
  
  CodeCompareScript::CodeCompareScript() {
    initTransientParameters();
    initPersistentParameters();
  }

  CodeCompareScript::~CodeCompareScript() {
  }

#ifdef USE_COMPOSABLE_NETWORK
  boost::shared_ptr<sdn::ComposableAutoEncoderNetwork> CodeCompareScript::getNewAutoEncoder() {
    return boost::dynamic_pointer_cast<sdn::ComposableAutoEncoderNetwork> (newNetScript.network);
  }
#else
  StandardAutoEncoderNetwork* CodeCompareScript::getNewAutoEncoder() {
    return &newNetScript.network;
  }
#endif
  
  
  boost::shared_ptr<sdn::StackedNetwork> CodeCompareScript::getOldAutoEncoder() {
    return oldSdnNet;
  }

  // -------------------------------------------------------------------
 
  // Callback
  void CodeCompareScriptCompareDifferenceCallback::loadXML(XMLNode & node) {
    p.parse(node);
    step = p.getInt("step");
    init();
  }
  
  void CodeCompareScriptCompareDifferenceCallback::execute(ScriptRuntimeManager& scriptManager,
                                                           AbstractScript& script,
                                                           unsigned int iteration) {
    
    CodeCompareScriptCompareDifferenceCallbackInterface* cScript;
    SDN_ASSERT(cScript = dynamic_cast<CodeCompareScriptCompareDifferenceCallbackInterface*> (&script));

#ifdef USE_COMPOSABLE_NETWORK
    boost::shared_ptr<sdn::ComposableAutoEncoderNetwork> network = cScript->getNewAutoEncoder();

    boost::shared_ptr<sdn::SparseAffineTanhBlock> encBlock = boost::dynamic_pointer_cast<sdn::SparseAffineTanhBlock> (network->blocks[0]);
    boost::shared_ptr<sdn::AffineTanhBlock> decBlock = boost::dynamic_pointer_cast<sdn::AffineTanhBlock> (network->blocks[1]);
    
#else
    StandardAutoEncoderNetwork* network = cScript->getNewAutoEncoder();
#endif

    boost::shared_ptr<sdn::StandardLayer> sdnEncoder = boost::dynamic_pointer_cast<sdn::StandardLayer> (cScript->getOldAutoEncoder()->getLayer(0));
    boost::shared_ptr<sdn::StandardLayer> sdnDecoder = boost::dynamic_pointer_cast<sdn::StandardLayer> (cScript->getOldAutoEncoder()->getLayer(1));
    
    sdn::Matrix drift;
    
#ifdef USE_COMPOSABLE_NETWORK
    drift = encBlock->transferBlock.affineB.weights - sdnEncoder->getWeights();
    SDN_MSG( "[" << itos(iteration, 10) << "] Encoding Weight Matrix Drift: " << drift.cwise().abs().maxCoeff() );
    drift = encBlock->transferBlock.affineB.bias - sdnEncoder->getBiases();
    SDN_MSG( "[" << itos(iteration, 10) << "] Encoding Bias Drift: " << drift.cwise().abs().maxCoeff() );
    drift = decBlock->affineB.weights - sdnDecoder->getWeights();
    SDN_MSG( "[" << itos(iteration, 10) << "] Decoding Weight Matrix Drift: " << drift.cwise().abs().maxCoeff() );
    drift = decBlock->affineB.bias - sdnDecoder->getBiases();
    SDN_MSG( "[" << itos(iteration, 10) << "] Decoding Bias Drift: " << drift.cwise().abs().maxCoeff() );
#else
    drift = network->affineEncoder.weights - sdnEncoder->getWeights();
    SDN_MSG( "[" << itos(iteration, 10) << "] Encoding Weight Matrix Drift: " << drift.cwise().abs().maxCoeff() );
    drift = network->affineEncoder.bias - sdnEncoder->getBiases();
    SDN_MSG( "[" << itos(iteration, 10) << "] Encoding Bias Drift: " << drift.cwise().abs().maxCoeff() );
    drift = network->affineDecoder.weights - sdnDecoder->getWeights();
    SDN_MSG( "[" << itos(iteration, 10) << "] Decoding Weight Matrix Drift: " << drift.cwise().abs().maxCoeff() );
    drift = network->affineDecoder.bias - sdnDecoder->getBiases();
    SDN_MSG( "[" << itos(iteration, 10) << "] Decoding Bias Drift: " << drift.cwise().abs().maxCoeff() );
#endif

  }

  void CodeCompareScriptCompareDifferenceCallback::init() {
  }
  
  void CodeCompareScriptCompareDifferenceCallback::initTransientParameters() {
  }
  
  void CodeCompareScriptCompareDifferenceCallback::initPersistentParameters() {
  }

  void CodeCompareScriptCompareDifferenceCallback::serialize(serial::Archive & ar) {
  }    

  CodeCompareScriptCompareDifferenceCallback::CodeCompareScriptCompareDifferenceCallback() {
    p.addInt("step");
    init();
  }
  
  CodeCompareScriptCompareDifferenceCallback::~CodeCompareScriptCompareDifferenceCallback() {
  }
  
  long int CodeCompareScriptCompareDifferenceCallback::itersToNextCall() {
    return step;
  }

  long int CodeCompareScriptCompareDifferenceCallback::firstIter() {
    return step;
  }

  
}
