//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: CodeCompareScript.h
//Description: Runs the redesign code and the old network in step to make sure it is working!
//             This is just meant for code testing, not development

#pragma once

#include "../blocks/sharedBlocks.h"
#include "../abstractScript.h"
#include "../networks/composableAutoEncoderNetwork.h"
#include "unsupervisedTrainingScript.h"
#include "standardAutoEncoderScript.h"
#include "sdn/base.h"

// #define USE_COMPOSABLE_NETWORK

namespace sdn {
  
  class StackedNetwork;

  class CodeCompareScriptCompareDifferenceCallbackInterface {
  public:
#ifdef USE_COMPOSABLE_NETWORK
    virtual boost::shared_ptr<sdn::ComposableAutoEncoderNetwork> getNewAutoEncoder() = 0;
#else
    virtual StandardAutoEncoderNetwork* getNewAutoEncoder() = 0;
#endif
    virtual boost::shared_ptr<sdn::StackedNetwork> getOldAutoEncoder() = 0;
  };
  
  /// CodeCompareScriptCompareDifferenceCallback class
  /** 
  */
  class CodeCompareScriptCompareDifferenceCallback : public AbstractScriptCallback {

  protected:
    AttributeParser p;

  public:
  

    virtual void initTransientParameters();
    virtual void initPersistentParameters();

    virtual void serialize(serial::Archive & ar) ;
    
    virtual void loadXML(XMLNode & node);
    virtual void execute(ScriptRuntimeManager& scriptManager,
                         AbstractScript& script,
                         unsigned int iteration) ;

    virtual long int itersToNextCall();
    virtual long int firstIter();
    
    virtual void init() ;
    
    CodeCompareScriptCompareDifferenceCallback();
    ~CodeCompareScriptCompareDifferenceCallback();
    
    long int step;
  
  };


  /// CodeCompareScript class
  /** 
  */
  class CodeCompareScript : public AbstractScript, 
                            public CodeCompareScriptCompareDifferenceCallbackInterface {

  public:

    virtual void loadXML(XMLNode& node);

    virtual void initTransientParameters(); 
    virtual void initPersistentParameters();

    virtual void serialize(serial::Archive & ar);

    virtual void loadUsingConfigManager();

    virtual void executeOneIteration() ;
    
    CodeCompareScript();
    ~CodeCompareScript();
           
    // Transient
#ifdef USE_COMPOSABLE_NETWORK
    UnsupervisedTrainingScript newNetScript;
#else
    StandardAutoEncoderScript newNetScript;
#endif
    boost::shared_ptr<sdn::StackedNetwork> oldSdnNet;

    // Callback Support
#ifdef USE_COMPOSABLE_NETWORK
    virtual boost::shared_ptr<sdn::ComposableAutoEncoderNetwork> getNewAutoEncoder() ;
#else
    virtual StandardAutoEncoderNetwork* getNewAutoEncoder();
#endif
    virtual boost::shared_ptr<sdn::StackedNetwork> getOldAutoEncoder();
  };


}
