//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: standardAutoEncoderScript.h
//Description: 

#pragma once

#include "../blocks/sharedBlocks.h"
#include "../abstractScript.h"
#include "../abstractNetwork.h"
#include "../networks/standardAutoEncoderNetwork.h"
#include "callbacks/sharedCallbacks.h"
#include "callbacks/vizCallbacks.h"

namespace sdn {

  // Forward Declarations
  class Dataset;

  /// StandardAutoEncoderScript class
  /** 
  */
  class StandardAutoEncoderScript : public AbstractScript, 
                                    public ReconstructionErrorCallbackInterface,
                                    public VizWeightsCallbackInterface,
                                    public SaveModelCallbackInterface {

  public:
  

    virtual void loadXML(XMLNode& node);

    virtual void initTransientParameters(); 
    virtual void initPersistentParameters();

    virtual void serialize(serial::Archive & ar);

    virtual void loadUsingConfigManager();

    virtual void executeOneIteration();
    
    StandardAutoEncoderScript();
    ~StandardAutoEncoderScript();

    // Configurable       
    std::string datasetName;
    int inputHeight;
    int inputWidth;

    // Persistent Parameters
    StandardAutoEncoderNetwork network;
    
    // Transient
    boost::shared_ptr<Dataset> dataset;
    sdn::Matrix inputExample;
    
    // Fixed Parameters
    NormalizeInputBlock normInput;
    
    // ---- Reconstruction Callback ----
    virtual const boost::shared_ptr<CostFunction> getReconstructionCostFunction()  ;
    virtual const sdn::Matrix& getReconstruction() ;
    virtual const sdn::Matrix& getOriginal() ;
  
    // ---- VizWeights Callback ----
    virtual const sdn::Matrix& getEncoderWeights();
    virtual const int getWeightPatchHeight();
    virtual const sdn::Matrix& getDecoderWeights();

    // ---- Save Model Callback ----
    virtual const void saveModelInDirectory(std::string dir);
  };
  
  
}
