//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: trainStackedAutoEncodersScript.h
//Description: 

#pragma once

#include "../blocks/sharedBlocks.h"
#include "../abstractScript.h"
#include "../abstractNetwork.h"
#include "../networks/stackedAutoEncoderNetwork.h"
#include "callbacks/sharedCallbacks.h"
#include "boost/shared_ptr.hpp"

namespace sdn {

  // Forward Declarations
  class Dataset;

  // Helper Struct
  struct newNetworkConfig {
    long int addNetworkIteration;
    bool trainAll;
    std::string networkName;
    XMLNode networkConfig;
    newNetworkConfig(long int iter, bool fT, std::string name, XMLNode& node) {
      addNetworkIteration = iter;
      trainAll = fT;
      networkName = name;
      networkConfig = node;
    }
  };


  /// TrainStackedAutoEncodersScript class
  /** 
  */
  class TrainStackedAutoEncodersScript : public AbstractScript, 
                                         public ReconstructionErrorCallbackInterface,
                                         public SaveModelCallbackInterface {
                                     
  public:
  

    // Inherited and Autoset ScriptRuntimeManager* manager;

    virtual void loadXML(XMLNode& node);

    virtual void initTransientParameters(); 
    virtual void initPersistentParameters();

    virtual void serialize(serial::Archive & ar);

    virtual void loadUsingConfigManager();

    virtual void executeOneIteration();
    
    TrainStackedAutoEncodersScript();
    ~TrainStackedAutoEncodersScript();

    // Configurable  
    std::vector<newNetworkConfig> stackConfig; // this should be loaded from the config.xml
    std::string datasetName;
    int inputHeight;
    int inputWidth;

    // Persistent Parameters
    boost::shared_ptr<StackedAutoEncoderNetwork> stackedNetwork;
    boost::shared_ptr<AbstractAutoEncoderNetwork> newNetwork;
    bool trainAll;
    
    // Transient
    boost::shared_ptr<Dataset> dataset;
    sdn::Matrix inputExample;
    
    // Fixed Parameters
    NormalizeInputBlock normInput;
    
    // Operations
    void loadStackConfig(XMLNode& node);
    
    // ---- Reconstruction Callback ----
    virtual const boost::shared_ptr<CostFunction> getReconstructionCostFunction() ;
    virtual const sdn::Matrix& getReconstruction() ;
    virtual const sdn::Matrix& getOriginal() ;

    // ---- Save Model Callback ----
    virtual const void saveModelInDirectory(std::string dir);
  
  };
  
  
}
