//Author: Jiquan Ngiam <jngiam@cs.stanford.edu>
//File: UnsupervisedTrainingScript.h
//Description: 

#pragma once

#include "../blocks/sharedBlocks.h"
#include "../abstractScript.h"
#include "../abstractNetwork.h"
#include "callbacks/sharedCallbacks.h"
#include "boost/shared_ptr.hpp"

namespace sdn {

  // Forward Declarations
  class Dataset;

  /// UnsupervisedTrainingScript class
  /** 
  */
  class UnsupervisedTrainingScript : public AbstractScript, 
                                     public ReconstructionErrorCallbackInterface,
                                     public SaveModelCallbackInterface {

  public:
  

    virtual void loadXML(XMLNode& node);

    virtual void initTransientParameters(); 
    virtual void initPersistentParameters();

    virtual void serialize(serial::Archive & ar);

    virtual void loadUsingConfigManager();

    virtual void executeOneIteration();
    
    UnsupervisedTrainingScript();
    ~UnsupervisedTrainingScript();

    // Configurable       
    std::string networkName;
    std::string datasetName;
    int inputHeight;
    int inputWidth;

    // Configurable-Persistent Parameters
    boost::shared_ptr<AbstractAutoEncoderNetwork> network;
    
    // Transient
    boost::shared_ptr<Dataset> dataset;
    sdn::Matrix inputExample;
    
    // Fixed Parameters
    NormalizeInputBlock normInput;
    
    // ---- Reconstruction Callback ----
    virtual const boost::shared_ptr<CostFunction> getReconstructionCostFunction() ;
    virtual const sdn::Matrix& getReconstruction() ;
    virtual const sdn::Matrix& getOriginal() ;

    // ---- Save Model Callback ----
    virtual const void saveModelInDirectory(std::string dir);
  
  };
  
  
}
