/// \author Ian Goodfellow


#include "sdn/base.h"
#include <Eigen/Core>
#include <Eigen/Array>
#include "gtest/gtest.h"
#include <vector>

using namespace sdn;
using namespace std;

TEST(MatrixUtils, MatrixToVector)
{
  unsigned rows = 5;
  unsigned cols = 10;
 

  Matrix m = Matrix::Random(rows,cols);

  Vector v = matrixToVector(m);

  /// Check that both forms of matrixToVector are equivalent
  Vector w(rows*cols);
  matrixToVector(m,w);
  ASSERT_EQ(v,w);

  /// Check that vectorCoord uses the same mapping as matrixToVector
  for (unsigned i = 0; i < rows; i++)
    for (unsigned j = 0; j < cols; j++)
      ASSERT_EQ(m(i,j), v(vectorCoord(i,j,rows,cols)));
}

TEST(MatrixUtils, mean)
{
  unsigned rows = 5;
  unsigned cols = 10;
  double tolerance = 1e-7;

  Matrix m = Matrix::Random(rows,cols);

  Matrix mean;

  matrixMean(m,mean);

  ASSERT_EQ(mean.rows(),1);
  ASSERT_EQ(mean.cols(),cols);

  for (unsigned i = 0; i < cols; i++)
    {
      ASSERT_LE( abs(mean(0,i) - m.col(i).sum()/double(rows)), tolerance);
    }

}
