/// \author Ian Goodfellow
#include "sdn/base.h"
#include "sdn/old.h"
#include <Eigen/Core>
#include <Eigen/Array>
#include "gtest/gtest.h"
using namespace std;
USING_PART_OF_NAMESPACE_EIGEN
using namespace sdn;
using namespace boost;


namespace sdn
{
  class TestClassA
  {
  public:
    virtual void serialize(serial::Archive & ar)
    {
      SDN_MSG("Ran TestClassA serialization");


      int version = 3;
      
      version = ar.beginBase(this,version,"sdn::TestClassA");
      
      switch(version)
	{
	case 3:
	  ar.endBase();
	  break;
	default:
	  SDN_FATAL("Tried to run version "<<version);
	}
    }
  };

  
Registry<TestClassA> & TestClassARegistry()
{
  static Registry<TestClassA> registry("sdn::TestClassA");
  return registry;
}
  
  
  class TestClassB : public TestClassA
{
public:
  virtual void serialize(serial::Archive & ar)
  {

    SDN_MSG("Ran TestClassB serialization");

    int version = 3;

    version = ar.beginDerived(this,version,"sdn::TestClassB");

    switch(version)
      {
      case 3:
	ar.beginSuper();
	TestClassA::serialize(ar);
	ar.endSuper();
	
	ar.endDerived();
	break;
      default:
	SDN_FATAL("");
      }
  }
};
  
  
  class TestClassC : public TestClassB
{
public:
  virtual void serialize(serial::Archive & ar)
  {
    SDN_MSG("Ran TestClassC serialization");
    int version = 3;
    
    version = ar.beginDerived(this,version,"sdn::TestClassC");
    
    switch(version)
      {
      case 3:
	ar.beginSuper();
	TestClassB::serialize(ar);
	ar.endSuper();
	
	ar.endDerived();

	break;
      default:
	SDN_FATAL("");
      }
  }
};
}


SDN_AUTOREGISTER(TestClassA, TestClassA);
SDN_AUTOREGISTER(TestClassA, TestClassB);
SDN_AUTOREGISTER(TestClassA, TestClassC);

/// Tests that a class with a grandparent can be serialized correctly
TEST(Serialization, DeepInheritance)
{
   ofstream s("../tmp.dat",ios::binary);
  //ofstream s(ios::binary | ios::out);
  serial::BinOutArchive writer(s);

  TestClassC myObj;
  myObj.serialize(writer);	
  s.close();
  
  //stringstream s2(s.str(),ios::in | ios::binary);
  ifstream s2("../tmp.dat",ios::binary);
  serial::BinInArchive reader(s2);
  shared_ptr<TestClassA> tc = TestClassARegistry().load(reader);
  
  TestClassC * p = dynamic_cast<TestClassC *>(tc.get());
  //ASSERT_TRUE(p != NULL);
  SDN_WARN("This test should fail, but has been gutted for now because it will be a while before I can fix it");
}





/** Tests that a StackedNetwork can be serialized to a binary archive,
and reloaded without knowing its type **/
TEST(Serialization, StackedNetworkBinary)
{
  sdn::Logger::setLogLevel(LOGLEVEL_DEBUG);
  
  StackedNetwork sn1;
  
  sn1.setInputSize(13,17);	
  
  ofstream s("../tmp.dat",ios::binary);
  //ofstream s(ios::binary | ios::out);
  serial::BinOutArchive writer(s);
  sn1.serialize(writer);	
  s.close();
  
  //stringstream s2(s.str(),ios::in | ios::binary);
  ifstream s2("../tmp.dat",ios::binary);
  serial::BinInArchive reader(s2);
  shared_ptr<Network> sn2 = NetworkRegistry().load(reader);
  
  StackedNetwork * p = dynamic_cast<StackedNetwork *>(sn2.get());
  ASSERT_TRUE(p != NULL);
  ASSERT_TRUE(sn2->equals(sn1));
  
}

/** Tests that a StackedNetwork can be serialized to a binary archive,
and reloaded without knowing its type, with multiple autoencoder
layers**/
TEST(Serialization, StackedNetworkMultilayerAutoencoderBinary)
{
  sdn::Logger::setLogLevel(LOGLEVEL_DEBUG);
  
  StackedNetwork sn1;
  
  sn1.setInputSize(13,17);
  sn1.addAutoencoder(300);
  sn1.addAutoencoder(200);
  sn1.addAutoencoder(100);
  
  ofstream s("../tmp.dat",ios::binary);
  //ofstream s(ios::binary | ios::out);
  serial::BinOutArchive writer(s);
  sn1.serialize(writer);	
  s.close();
  
  //stringstream s2(s.str(),ios::in | ios::binary);
  ifstream s2("../tmp.dat",ios::binary);
  serial::BinInArchive reader(s2);
  shared_ptr<Network> sn2 = NetworkRegistry().load(reader);
  
  StackedNetwork * p = dynamic_cast<StackedNetwork *>(sn2.get());
  ASSERT_TRUE(p != NULL);
  ASSERT_TRUE(sn2->equals(sn1));
  
}

/** Tests that a StackedNetwork can be serialized to a binary archive,
and reloaded without knowing its type, with multiple autoencoder
layers and weight decay**/
TEST(Serialization, StackedNetworkMultilayerAutoencoderWeightDecayBinary)
{
  sdn::Logger::setLogLevel(LOGLEVEL_DEBUG);
  
  StackedNetwork sn1;

stringstream options;
 options  << "<sdn::Autoencoder "
	  << "learning_rate=\"" << .01 << "\""
	  << "learning_rate_sparsity=\"0\""
	  << "target_sparsity=\"0\""
	  << "weight_decay=\"0.003\"/>";
	  
  XMLNode params = parseXML(options.str());

  
  sn1.setInputSize(13,17);
  sn1.addAutoencoder(300, params);
  //sn1.addAutoencoder(200, params);
  //sn1.addAutoencoder(100, params);
  
  ofstream s("../tmp.dat",ios::binary);
  //ofstream s(ios::binary | ios::out);
  serial::BinOutArchive writer(s);
  sn1.serialize(writer);	
  s.close();
  
  //stringstream s2(s.str(),ios::in | ios::binary);
  ifstream s2("../tmp.dat",ios::binary);
  serial::BinInArchive reader(s2);
  shared_ptr<Network> sn2 = NetworkRegistry().load(reader);
  
  StackedNetwork * p = dynamic_cast<StackedNetwork *>(sn2.get());
  ASSERT_TRUE(p != NULL);
  ASSERT_TRUE(sn2->equals(sn1));
  
}

