/// \author Ian Goodfellow


#include "sdn/base.h"
#include <Eigen/Core>
#include <Eigen/Array>
#include "gtest/gtest.h"
#include <vector>

using namespace sdn;
using namespace std;

TEST(VarianceEstimator, Basic)
{
  Scalar tolerance = 1e-4;
  unsigned rows = 5;
  unsigned cols = 10;
  unsigned n = 20;
  
  vector<Matrix> samples(n);
  
  VarianceEstimator e;

  Matrix m = Matrix::Zero(rows,cols);
  Matrix v = m;

  for (unsigned i = 0; i < n; i++)
    {
      samples[i] = Matrix::Random(rows,cols);
      m += samples[i];
      e.processSample(samples[i]);
    }

  m /= Scalar(n);

  ASSERT_LE( ( m - e.getMean()).cwise().abs().sum(), tolerance);

  for (unsigned i = 0; i < n; i++)
    {
      v += (samples[i] - m).cwise().square();
    }

  v  /= Scalar ( n -1);

  ASSERT_LE ( ( v - e.getVariance()).cwise().abs().sum(), tolerance);
}
