ORIG_PWD = $(shell pwd)

USE_GTEST = 1
USE_OPENCV = 1
USE_EIGEN = 1

-include ${SDN_PATH}/make.mk

BIN_PATH=${SDN_PATH}/tests/bin

TEMP_DIR=$(shell pwd)/_intermediates

APP_PROG_NAMES = $(APP_SRC:.cc=)
APP_OBJ=${addprefix ${TEMP_DIR}/,$(APP_SRC:.cc=.o)}
OTHER_OBJ=${addprefix ${TEMP_DIR}/,$(OTHER_SRC:.cc=.o)}
APP_DEPS=${APP_OBJ:.o=.d}
OTHER_DEPS=${OTHER_OBJ:.o=.d}
DEPS=${APP_DEPS} ${OTHER_DEPS}

CFLAGS += -I${ORIG_PWD}

.PHONY: clean temp_dirs
.PRECIOUS: ${APP_OBJ}


all: temp_dirs ${addprefix ${BIN_PATH}/,${APP_PROG_NAMES}}
	@echo "------------------- all ------------------------------"

temp_dirs:
	@echo "------------------- temp_dirs -------------------------"
	mkdir -p ${TEMP_DIR}

${BIN_PATH}/%: ${TEMP_DIR}/%.o ${LIBSDN} ${OTHER_OBJ}
	@echo "----------------- BIN_PATH/.o -------------------------"
	@echo $*.o
	@echo $(@:.o=)
	${CCC} ${TEMP_DIR}/$*.o -o $(@:.o=) ${OTHER_OBJ} $(LIBSDN) $(LFLAGS)
	@echo "******"

${TEMP_DIR}/%.o : %.cc
	@echo "----------------- TEMP_DIR/.o $@ -----------------"
	#make dependencies
	@$(call make-depend,$<,$@,$(subst .o,.d,$@))
	#compile
	${CCC} ${CFLAGS} -c $< -o $@

#usage of make-depend function: $(call make-depend,source-file,object-file,depend-file)
define make-depend
  $(CXX) -MM -MF $3 -MP -MT $2 $(CFLAGS) $1
endef


${LIBSDN}:
	@echo "-------------------- LIBSDN -----------------------------"
	cd ${SDN_PATH} && make lib && cd ${ORIG_PWD}

clean:
	@echo "------------------- clean -------------------------------"
	-rm ${APP_OBJ}
	-rm ${OTHER_OBJ}
	-rm ${DEPS}
	-rm ${addprefix ${BIN_PATH}/,${APP_PROG_NAMES}}

-include ${DEPS}
