#include "sdn/old.h"
#include "sdn/stable.h"
#include "derivativeChecker.h"
#include "gtest/gtest.h"

using namespace sdn;

class ActivationJacobianChecker : public JacobianChecker {

public:
  ActivationJacobianChecker(ActivationFunction* fn) {
    fn_ = fn;
  } 

  ~ActivationJacobianChecker() {
    delete fn_;
    fn_ = NULL;
  }
 
  /// \todo: convert to pass by reference instead of returning matrix 
  Matrix AnalyticalJacobian(const Vector& x) {
    Matrix result;
    fn_->outputDerivative(x, result);
    return result;
  }

  /// \todo: convert to pass by reference instead of returning matrix
  Vector EvaluateFunction(const Vector& x) {
    Matrix result;
    fn_->output(x, result);
    return result;
  }

  ActivationFunction* fn_;
};


TEST(CommonActivationFunctions, Linear) {
  Scalar epsilon = 1e-4;
  Scalar tolerance = 1e-3;
  Vector x = Vector::Random(10);
  
  ActivationJacobianChecker ajc(new LinearActivationFunction);
  //ajc.PrintComparison(epsilon,x);
  ASSERT_LE(ajc.CheckJacobian(epsilon, x), tolerance);
}

TEST(CommonActivationFunctions, Logistic) {
  Scalar epsilon = 1e-4;
  Scalar tolerance = 1e-3;
  Vector x = Vector::Random(10);
  
  ActivationJacobianChecker ajc(new LogisticActivationFunction);
  //ajc.PrintComparison(epsilon,x);
  ASSERT_LE(ajc.CheckJacobian(epsilon, x), tolerance);
}

TEST(CommonActivationFunctions, Tanh) {
  Scalar epsilon = 1e-4;
  Scalar tolerance = 1e-3;
  Vector x = Vector::Random(10);
  
  ActivationJacobianChecker ajc(new TanhActivationFunction);
  //ajc.PrintComparison(epsilon,x);
  ASSERT_LE(ajc.CheckJacobian(epsilon, x), tolerance);
}
TEST(CommonActivationFunctions, SoftSign) {
  Scalar epsilon = 1e-4;
  Scalar tolerance = 1e-3;
  Vector x = Vector::Random(10);
  
  ActivationJacobianChecker ajc(new SoftSignActivationFunction);
  //ajc.PrintComparison(epsilon,x);
  ASSERT_LE(ajc.CheckJacobian(epsilon, x), tolerance);
}


TEST(CommonActivationFunctions, SoftMaxNormalization) {
  //Scalar epsilon = 1e-4;
  //Scalar tolerance = 1e-3;
  Matrix x = Matrix::Random(10,15);
  
  SoftmaxActivationFunction saf;
  Matrix out;
  saf.output(x, out);
  
  for(int c = 0; c < out.cols(); c++)
    EXPECT_FLOAT_EQ(out.col(c).sum(), 1);
}

TEST(CommonActivationFunctions, Softmax) {
  Scalar epsilon = 1e-4;
  Scalar tolerance = 1e-3;
  Vector x = Vector::Random(10);
  
  ActivationJacobianChecker ajc(new SoftmaxActivationFunction);
  //ajc.PrintComparison(epsilon,x);
  ASSERT_LE(ajc.CheckJacobian(epsilon, x), tolerance);
}
