#include "sdn/old.h"
#include "gtest/gtest.h"

using namespace sdn;
using namespace std;

TEST(ClassificationPRCurve, Basic) {
  vector<double> posScores;
  vector<double> negScores;

  posScores.push_back(1.0);
  posScores.push_back(3.0);

  negScores.push_back(0.0);
  negScores.push_back(2.0);



  //  Threshold  TP    FP    TN     FN     P      R     F
  //  0          2     2     0      0      1/2    1     2/3
  //  1          2     1     1      0      2/3    1     4/5
  //  2          1     1     1      1      1/2    1/2   1/2
  //  3          1     0     2      1      1      1/2   1/3    
  //  3+         0     0     2      2      1      0     0

  ClassificationPRCurve pr(posScores,negScores);

  ASSERT_EQ(pr.numPoints(), 5);

  ASSERT_EQ(pr.getPoint(0).threshold, 0);
  ASSERT_EQ(pr.getPoint(1).truePositives, 2);
  ASSERT_EQ(pr.getPoint(2).falsePositives, 1);
  ASSERT_EQ(pr.getPoint(3).trueNegatives,2);
  ASSERT_EQ(pr.getPoint(4).falseNegatives,2);
  ASSERT_EQ(pr.getPoint(0).precision, 0.5);
  ASSERT_EQ(pr.getPoint(2).recall,0.5);
  ASSERT_EQ(pr.getPoint(1).fScore(),0.8);
  ASSERT_EQ(pr.getPoint(0).recall,1.0);
  ASSERT_EQ(pr.getPoint(1).recall,1.0);

  ASSERT_EQ(pr.maxF(), 0.8);

  // Sorted values
  // R    P
  // 0    1
  // 1/2  1
  // 1/2  1/2
  // 1    2/3
  // 1    1/2


  // Discard points under curve, compute area contribution for each
  // R    P     Area
  // 0    1     1/2
  // 1/2  1     1/2+1/2*(2/3-1/2)
  // 1    2/3

  double tolerance = 0.00001;
  ASSERT_LE( abs(pr.area() - (0.5+0.5*(2.0/3.0+0.5*1.0/3.0))) , tolerance);
}


