#include "sdn/old.h"
#include "sdn/stable.h"
#include "derivativeChecker.h"
#include <Eigen/Core>
#include <Eigen/Array>
#include "gtest/gtest.h"
                                                                                
using namespace sdn;

class CostJacobianChecker : public JacobianChecker {

public:
  CostJacobianChecker(CostFunction* fn, Vector& target) {
    fn_ = fn;
    target_ = target;
  } 

  ~CostJacobianChecker() {
    delete fn_;
    fn_ = NULL;
  }
 
  Matrix AnalyticalJacobian(const Vector& x) {
    return fn_->jacobian(x, target_);
  }

  Vector EvaluateFunction(const Vector& x) {
    return fn_->output(x,target_);
  }

  CostFunction* fn_;
  Vector target_;
};


TEST(CommonCostFunctions, SumOfSquares) {
  Scalar epsilon = 1e-4;
  Scalar tolerance = 1e-3;
  Vector x = Vector::Random(10);
  Vector target = Vector::Random(10);
  
  CostJacobianChecker cjc(new SumOfSquaresCostFunction, target);
  cjc.PrintComparison(epsilon,x);
  ASSERT_LE(cjc.CheckJacobian(epsilon, x), tolerance);
}
