/// \author Ian Goodfellow

#include "sdn/base.h"
#include "sdn/old.h"
#include <Eigen/Core>
#include <Eigen/Array>
#include "gtest/gtest.h"
#include <vector>

using namespace sdn;
using namespace std;

TEST(GraphicalTransformer, TranslationTransformer)
{
  sdn::Logger::setLogLevel(LOGLEVEL_DEBUG);
  Scalar tolerance = 1e-4;
  int rows = 6;
  int cols = 7;
  
  int minOffset = -3;
  int maxOffset = 3;

  Matrix input;
  Matrix output(Matrix::Zero(rows,cols));

  TranslationTransformer t;
  t.setOutputSize(rows,cols);
  t.setOffsetRange(minOffset,maxOffset);
  t.setDirection(0);
  
  input = Matrix::Random(t.getInputRows(),t.getInputCols());

  ASSERT_EQ(input.cols(),cols+maxOffset-minOffset);

  int offset = 0;
  t.setOffset(offset);
  SDN_DEBUG("mul1");  
output = (* t.getWeights()) * matrixToVector(input);

  SDN_DEBUG("eq1");
  ASSERT_EQ(output,matrixToVector(input.block(0,offset-minOffset,rows,cols)));
  
  t.setOffset(minOffset);
SDN_DEBUG("mul2");
  output = (* t.getWeights()) * matrixToVector(input);

  SDN_DEBUG("eq2");
  ASSERT_EQ(output, matrixToVector(input.block(0,0,rows,cols)));

  t.setOffset(maxOffset);
  output = (* t.getWeights()) * matrixToVector(input);

  SDN_DEBUG("eq3");
  ASSERT_EQ(output, matrixToVector(input.block(0,maxOffset-minOffset,rows,cols)));

}
