#include <fstream>
#include <iostream>
#include <sstream>
#include "gtest/gtest.h"
#include "sdn/base.h"
#include <cuda_runtime.h>

#include "../../../src/lib/old/invariantNetwork.h"

using namespace std;
using namespace sdn;

TEST(InvariantNetwork, Train) {
  sdn::Logger::setLogLevel(LOGLEVEL_DEBUG);

#if SDN_USE_CUDA
  SDN_DEBUG("Using cuda");
  cudaSetDevice(1);
#endif
  
  Scalar alpha = 10;
  Scalar alpha_v = 1;
  Scalar alpha_t = 0.001; // 0.00015;
  int iterations = 10000;
  int batchSize = 5000;
  int neighborhoodSize = 0;
  Scalar tolerance = 0.5;
  Scalar lambdaV = 0.0002;
  Scalar lambdaW = 1.0;
  Scalar lambdaT = 0;
  int W_rows = 1600; // as of now, must be square

  SDN_DEBUG("Learning rate: " << alpha);
  SDN_DEBUG("Learning rate for V: " << alpha_v);
  SDN_DEBUG("Iterations: " << iterations);
  SDN_DEBUG("Batch size: " << batchSize);
  SDN_DEBUG("Neighborhood size: " << neighborhoodSize);
  SDN_DEBUG("Convergence tolerance factor: " << tolerance);
  //SDN_DEBUG("Regularization constant: " << lambda);
  SDN_DEBUG("Rows of W: " << W_rows);

  char filePrefix[128];
  sprintf(filePrefix,"_bs=%d_i=%d_ns=%d_lV=%lf_lW=%lf_wr=%d", batchSize, iterations, neighborhoodSize, lambdaV, lambdaW, W_rows);
 
  ifstream data ("/home/jeffrey/libsdn-trunk/tests/lib/network/pcaDataSlowRot.txt");
  
  //ifstream W ("/home/jeffrey/libsdn-trunk/tests/bin/invarianceResults/weights1_bs=5000_i=4000_lV=0.000200_lW=0.200000_wr=10000_sc_square_cuda.mat");
  //ifstream V ("/home/jeffrey/libsdn-trunk/tests/bin/invarianceResults/weights2_bs=3125_i=6400_lam=0.000200_wr=10000_ldc_square_cuda.mat");
  //ifstream T ("/home/jeffrey/libsdn-trunk/tests/bin/invarianceResults/weights3_bs=5000_i=4000_lV=0.000200_lW=0.200000_wr=1600_ldc_square_cuda.mat");

  // testing 10x overcomplete
  //ifstream W ("/home/jeffrey/libsdn-trunk/tests/bin/invarianceResults/weights1_bs=3125_i=6400_lam=0.000200_wr=10000_ldc_square_cuda.mat");
  //ifstream T ("/home/jeffrey/libsdn-trunk/tests/bin/invarianceResults/weights3_bs=3125_i=6400_lam=0.000200_wr=10000_ldc_square_cuda.mat");

  //ifstream W ("/home/jeffrey/libsdn-trunk/tests/bin/invarianceResults/goodW_2.mat");

  ifstream cifarTrain ("/home/jeffrey/libsdn-trunk/tests/lib/network/pcaCifarTrain.txt");
  ifstream cifarTrainLabels ("/home/jeffrey/libsdn-trunk/tests/lib/network/cifarTrainLabels.txt");
  ifstream cifarTest ("/home/jeffrey/libsdn-trunk/tests/lib/network/pcaCifarTest.txt");
  ifstream cifarTestLabels ("/home/jeffrey/libsdn-trunk/tests/lib/network/cifarTestLabels.txt");

  ifstream norbTrain ("/home/jeffrey/libsdn-trunk/tests/lib/network/pcaNorbTrain.txt");
  ifstream norbTrainLabels ("/home/jeffrey/libsdn-trunk/tests/lib/network/norbTrainLabels.txt");
  ifstream norbTest ("/home/jeffrey/libsdn-trunk/tests/lib/network/pcaNorbTest.txt");
  ifstream norbTestLabels ("/home/jeffrey/libsdn-trunk/tests/lib/network/norbTestLabels.txt");

  InvariantNetwork net(neighborhoodSize, W_rows);
  net.setInputsOnCPU(true);
  
  net.readInput(data, SLOWROT);
  //net.readInput(cifarTrain, CIFAR);

  net.initRandWeights();
  net.train(alpha, iterations, batchSize, tolerance, true, false, false, 0,
            lambdaW, SPARSE_COST, SQUARE_SQRT_ACT);

  //net.train(alpha_v, iterations, batchSize, tolerance, false, true, false,
  //          lambdaV, 0, LOGCOSH_DIFF_COST, SQUARE_SQRT_ACT);
  net.saveWeights(string(filePrefix));
  
  int inputsPerExample, classes;

  inputsPerExample = 1;
  classes = 10;
  net.readInput(cifarTrain, CIFAR);
  net.readInput(cifarTrainLabels, CIFAR_LABELS);
  batchSize = 5000; // 1000;

  //inputsPerExample = 18;
  //classes = 5;
  //net.readInput(norbTrain, NORB);
  //net.readInput(norbTrainLabels, NORB_LABELS);
  //batchSize = 300;
  
  //net.initWeight(W, W_MAT);
  //net.initWeight(V, V_MAT);
  //net.initWeight(T, T_MAT);
  
  tolerance = 0.0001;
  net.trainSoftmax(alpha_t, iterations, batchSize, tolerance, inputsPerExample, classes, SQUARE_SQRT_ACT, L2, lambdaT);
  
  net.testAccuracy(batchSize, inputsPerExample, SQUARE_SQRT_ACT);
  //net.train(alpha_t, iterations * 3 * 2, batchSize, tolerance, false, false, true, 0, 0, LOGCOSH_DIFF_COST, SQUARE_SQRT_ACT);
  //net.train(alpha_t*1.5, iterations / 4 * 3, batchSize, tolerance, false, false, true, 0, 0, LOGCOSH_DIFF_COST, SQUARE_SQRT_ACT);
  //net.saveT(string(filePrefix));

  net.saveFeatures(filePrefix, batchSize, inputsPerExample, SQUARE_SQRT_ACT, NORMAL, false);
  net.saveFeatures(filePrefix, batchSize, inputsPerExample, SQUARE_SQRT_ACT, LIBSVM, false);

  net.readInput(cifarTest, CIFAR_TEST);
  net.readInput(cifarTestLabels, CIFAR_TEST_LABELS);
  net.testAccuracy(batchSize, inputsPerExample, SQUARE_SQRT_ACT);

  bool test = true;
  net.saveFeatures(filePrefix, batchSize, inputsPerExample, SQUARE_SQRT_ACT, 
                   NORMAL, test);

  net.saveFeatures(filePrefix, batchSize, inputsPerExample, SQUARE_SQRT_ACT, 
                   LIBSVM, test);

  //net.readInput(norbTest, NORB_TEST);
  //net.readInput(norbTestLabels, NORB_TEST_LABELS);
  //net.testAccuracy(batchSize, inputsPerExample, SQUARE_SQRT_ACT);
}
